#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include "GSPJob.h"
#include "List.h"
#include "GSPMachine.h"

extern HashTable_t     machineList ;

int
Job_t::Parse()
{
	if ( jobConfigFile.empty() ) 
		return -1 ;

	int startNum = 1 ; 
	FILE* fp = fopen(jobConfigFile.chars(),"r") ;

	if ( fp == NULL )
	{
		cerr << "Unable to open the jobConfiguration file " 
		     << jobConfigFile.chars() << endl  ;

		return -1 ;
	}

	// Expected Completion time in terms of seconds
	// TO BE ENCHANCED LATER 

	// Error Checking has to be done in the parser planning to write

	fscanf(fp,"%d\n",&expCompletionTime) ;

	String tmp ;
	
	tmp.alloc(BUFSIZ) ;

	while( !feof(fp) )
	{
		int pos = 0 ;
		int follow = 0 ; 
		String instances ;

		if ( fgets(tmp, BUFSIZ, fp) == NULL )
			break ;

		// TO DO: Check for non-allocation 
		Entry_t* entry = new Entry_t ;

		cout.form("Looping %p\n",entry) ;  

		// (void) fgetc(fp) ; 

		// TO DO: Currently each of the parameters
		// is separated by EXACTLY ONE SPACE
		// has to be improved

		// Get the number of instances
		follow = tmp.index( ' ',pos ) ;
		instances = tmp( pos, follow - pos ) ;
		pos = follow + 1 ;
		entry->numInstances = atoi(instances.chars() ) ;

		// Get the file name
		follow = tmp.index( ' ', pos ) ;
		entry->fileName = tmp( pos, follow - pos ) ;
		pos = follow + 1 ;

		// Get the parameters
		entry->parameters = tmp( pos, tmp.length() - pos ) ;

		entry->startEntryNum = startNum ;
		entry->endEntryNum = startNum + entry->numInstances - 1 ;  
		startNum = startNum + entry->numInstances ;  

		entryList.InsertAtEnd(entry) ; 
	}
	
	fclose(fp) ;

	return 0 ; 
}

Job_t::~Job_t()
{
	if ( entryList.IsEmpty() )
		return ;

	Entry_t* tmpEntry = NULL ;

	for ( entryList.StartTraversal(), tmpEntry = entryList.Element() ; tmpEntry ; 
		tmpEntry = entryList.Element() ) 
	{
		entryList.DeleteAndAdvance() ;

		cout.form("Deleting %p\n",tmpEntry)  ;

		delete tmpEntry ;
	}
}

ProcessInfo_t*
RunningEntry_t::Search( int entryNum )
{
	if ( processList.IsEmpty() )
		return NULL ;

	ProcessInfo_t*	tmpProcess = NULL ;
	
	for ( processList.StartTraversal() , tmpProcess = processList.Element() ;
		tmpProcess ;  
		tmpProcess = !processList.Advance() ? processList.Element() : NULL ) 
	{
		if ( tmpProcess->entryNum == entryNum )
			return tmpProcess ;

	}

	return NULL ; 
}	


int
RunningEntry_t::Insert( int entryNum, struct timeb& timeComp )
{
	ProcessInfo_t* anInstance = new ProcessInfo_t(entryNum) ;

	if ( anInstance != NULL )
	{
		anInstance->compTime.time = timeComp.time ; 
		anInstance->compTime.millitm = timeComp.millitm ; 
		processList.Insert( anInstance ) ;

		return 0 ;  
	}

	return -1 ; 
}

int 
RunningEntry_t::Delete( int entryNum )
{
	if ( processList.IsEmpty() )    
		return -1 ;

	ProcessInfo_t*  tmpProcess = NULL ;
											 
	for ( processList.StartTraversal(), tmpProcess = processList.Element() ;
	      tmpProcess ; tmpProcess = !processList.Advance() ? 
			         processList.Element() : NULL ) 
	{
		if ( entryNum == tmpProcess->entryNum )
		{
			processList.DeleteAndAdvance() ;

			delete tmpProcess ;  

			return 0 ;
		}
	}

	return -1 ;
}


void 
RunningEntry_t::DeleteAll()
{
	if ( processList.IsEmpty() )    
		return ;

	ProcessInfo_t*  tmpProcess = NULL ;
											 
	for ( processList.StartTraversal(), tmpProcess = processList.Element() ;
	      tmpProcess ; tmpProcess = processList.Element() ) 
	{
		processList.DeleteAndAdvance() ;

		delete tmpProcess ;  
	}
}


void 
RunningEntry_t::Print( )
{
	cout << "Start Entry : " << startEntryNum << endl ;

	cout << "End Entry : " << endEntryNum << endl ;

	entryPtr->Print() ;

	if ( processList.IsEmpty() )    
		return ;

	ProcessInfo_t*  tmpProcess = NULL ;
											 
	for ( processList.StartTraversal(), tmpProcess = processList.Element() ;
	      tmpProcess ; tmpProcess = !processList.Advance() ? 
			         processList.Element() : NULL ) 
	{
		cout << tmpProcess->entryNum << " " << tmpProcess->compTime.time << " " 
					   << tmpProcess->compTime.millitm << endl ; 
	}
}

RunningEntry_t* 
MachInfo_t::Search( int entryNum )
{
	if ( entryList.IsEmpty() )    
		return NULL ;

	RunningEntry_t*  tmpentry = NULL ;
											 
	for ( entryList.StartTraversal(), tmpentry = entryList.Element() ;
	      tmpentry ; tmpentry = !entryList.Advance() ? 
			         entryList.Element() : NULL ) 
	{
		if ( tmpentry->startEntryNum <= entryNum &&
		     entryNum <= tmpentry->endEntryNum )
		{
			return tmpentry ; 	
		}
	}

	return NULL ; 
}

void
MachInfo_t::DeleteAll()
{
	if ( entryList.IsEmpty() )    
		return ;

	RunningEntry_t*  tmpentry = NULL ;
											 
	for ( entryList.StartTraversal(), tmpentry = entryList.Element() ;
	      tmpentry ; tmpentry = entryList.Element() ) 
	{
		entryList.DeleteAndAdvance() ;

		delete tmpentry ; // Implicitly calls DeleteAll via destructor
	}
}

void 
MachInfo_t::Print( )
{
	if ( !machPtr->ipAddress.empty() ) 
		cout << "IpAddress : " << machPtr->ipAddress << " "
		     << "prsr " << prsrAllocated << endl ;

	
	if ( entryList.IsEmpty() )    
		return ;

	RunningEntry_t*  tmpentry = NULL ;
											 
	for ( entryList.StartTraversal(), tmpentry = entryList.Element() ;
	      tmpentry ; tmpentry = !entryList.Advance() ? 
			         entryList.Element() : NULL ) 
	{
		tmpentry->Print() ;  
	}
}

MachInfo_t*
RunningJob_t::Search(Machine_t* mPtr)
{
	if ( runningMachList.IsEmpty() )
		return NULL ;

	MachInfo_t*  tmpMach ;

	for ( runningMachList.StartTraversal(), tmpMach = runningMachList.Element() ;
	      tmpMach ; tmpMach = !runningMachList.Advance() ? 
			         runningMachList.Element() : NULL ) 
	{
		if ( tmpMach->machPtr == mPtr )
			return tmpMach ; 
	}

	return NULL ; 
}

void
RunningJob_t::DeleteAll()
{
	if ( runningMachList.IsEmpty() )
		return ;

	MachInfo_t*  tmpMach ;

	for ( runningMachList.StartTraversal(), tmpMach = runningMachList.Element() ;
	      tmpMach ; tmpMach = runningMachList.Element() ) 
	{
		runningMachList.DeleteAndAdvance() ;

		delete tmpMach ; 	// Implicitly calls DeleteAll via destructor
	}
}

MachInfo_t*
RunningJob_t::Search(const char* ipAddress)
{
	if ( runningMachList.IsEmpty() )
		return NULL ;

	MachInfo_t*  tmpMach ;

	for ( runningMachList.StartTraversal(), tmpMach = runningMachList.Element() ;
	      tmpMach ; tmpMach = !runningMachList.Advance() ? 
			         runningMachList.Element() : NULL ) 
	{
		assert( tmpMach->machPtr != NULL ) ;

		if ( !::strcmp( tmpMach->machPtr->ipAddress.chars(),ipAddress ) )
			return tmpMach ; 
	}

	return NULL ; 
}

void 
RunningJob_t::Print()
{
	if ( runningMachList.IsEmpty() )
		return ;

	MachInfo_t*  tmpMach ;

	for ( runningMachList.StartTraversal(), tmpMach = runningMachList.Element() ;
	      tmpMach ; tmpMach = !runningMachList.Advance() ? 
			         runningMachList.Element() : NULL ) 
	{
		tmpMach->Print() ; 	
	}
}

// oo_List<Job_t> is ordered in increasing order of job id
// that is the arrival order
Job_t*
Search( oo_List<Job_t>& jobCollect, int jobId )
{
	if ( jobCollect.IsEmpty() )
		return NULL ;

	Job_t*	tmpJob = NULL ;
	
	for ( jobCollect.StartTraversal() , tmpJob = jobCollect.Element() ;
		tmpJob && tmpJob->jobId < jobId ; 
		tmpJob = !jobCollect.Advance() ? jobCollect.Element() : NULL ) ;

	return  tmpJob == NULL ? NULL : (jobId == tmpJob->jobId ? tmpJob : NULL) ;
}
	

void
Insert( oo_List<Job_t>& jobCollect, Job_t* aJob ) 
{
	assert( aJob != NULL ) ;

	if ( jobCollect.IsEmpty() )
	{
		jobCollect.Insert(aJob) ;

		return ;
	}

	Job_t* tmpJob = NULL ;

	for ( jobCollect.StartTraversal() , tmpJob = jobCollect.Element() ;
		tmpJob && tmpJob->jobId < aJob->jobId ;
		tmpJob = !jobCollect.Advance() ? jobCollect.Element() : NULL ) ;

	if ( !tmpJob )
	{
		jobCollect.InsertAtEnd(aJob) ;
	}
	else
	{
		jobCollect.Insert(aJob) ;
	}
	
	return ;
}

int
Delete( oo_List<Job_t>& jobCollect, int jobId )
{
	if ( jobCollect.IsEmpty() )
		return -1 ;

	Job_t*	tmpJob = NULL ;
	
	for ( jobCollect.StartTraversal() , tmpJob = jobCollect.Element() ;
		tmpJob && tmpJob->jobId < jobId ;
		tmpJob = !jobCollect.Advance() ? jobCollect.Element() : NULL ) ;

	if ( jobId == tmpJob->jobId )
	{
		jobCollect.DeleteAndAdvance() ;

		delete tmpJob ;

		return 0 ;
	}

	return -1 ;
}

void
DeleteAll( oo_List<Job_t>& jobCollect )
{
	if ( jobCollect.IsEmpty() )
		return ;

	Job_t*	tmpJob = NULL ;
	
	for ( jobCollect.StartTraversal() , tmpJob = jobCollect.Element() ;
		tmpJob ;
		tmpJob = jobCollect.Element() ) ;
	{
		jobCollect.DeleteAndAdvance() ;

		delete tmpJob ;
	}
}

/*
int
DeleteAll( oo_List<JobLSPInfo_t>& jobLSP )
{
	if ( jobLSP.IsEmpty() )
		return -1 ;

	JobLSPInfo_t*	tmpLSP = NULL ;
	
	for ( jobLSP.StartTraversal() , tmpLSP = jobLSP.Element() ;
		tmpLSP ;
		tmpLSP = !jobLSP.Advance() ? jobLSP.Element() : NULL ) 
	{
		jobLSP.DeleteAndAdvance() ;

		delete tmpLSP ;

		return 0 ;
	}

	return -1 ;
}

*/

void
Print( oo_List<Job_t>& jobCollect )
{
	void Print( oo_List<Entry_t>& ) ;

	if ( jobCollect.IsEmpty() )
		return ;

	jobCollect.StartTraversal() ;

	for ( Job_t* tmpJob = jobCollect.Element() ; tmpJob ;  
		tmpJob = !jobCollect.Advance() ? jobCollect.Element() : NULL ) 
	{
		cout << tmpJob->jobId << "\t" << tmpJob->jobConfigFile
		     << "\t" << tmpJob->jobArrivalTime.time << "\t"
		     << tmpJob->jobArrivalTime.millitm << endl ;

		Print( tmpJob->entryList ) ; 	
	}

	cout << "------------------------" << endl ;
}
		
void
Print( oo_List<RunningJob_t>& runCollect )
{
	if ( runCollect.IsEmpty() )
		return ;

	runCollect.StartTraversal() ;

	for ( RunningJob_t* tmpJob = runCollect.Element() ; tmpJob ;  
		tmpJob = !runCollect.Advance() ? runCollect.Element() : NULL ) 
	{
		tmpJob->Print() ; 
	}
}
