#include <stdio.h>
#include <stdlib.h>

#include "GSPMachine.h"

int
Machine_t::InsertJob( Job_t* aJob )
{
	JobInstances_t* anInstance = SearchJob( aJob->jobId ) ;

	if ( anInstance == NULL ) 
	{
		anInstance = new JobInstances_t(aJob) ;

		if ( anInstance != NULL )
		{
			jobs.Insert( anInstance ) ;
		}
		else
			return -1 ; 
	
	}

	return 0 ; 
}
  
int
Machine_t::DeleteJob ( int jobId ) 
{
	if ( jobs.IsEmpty() )    
		return -1 ;

	JobInstances_t*  tmpJob = NULL ;
											 
	for ( jobs.StartTraversal(), tmpJob = jobs.Element() ;
	      tmpJob ; tmpJob = !jobs.Advance() ? 
			         jobs.Element() : NULL ) 
	{
		if ( jobId == tmpJob->ptrToJob->jobId )
		{
			jobs.DeleteAndAdvance() ;

			delete tmpJob ;  

			return 0 ;
		}
	}

	return -1 ;
}

int
Machine_t::DeleteJob ( Job_t* aJob ) 
{
	if ( jobs.IsEmpty() )    
		return -1 ;

	JobInstances_t*  tmpJob = NULL ;
											 
	for ( jobs.StartTraversal() , tmpJob = jobs.Element() ;
	      tmpJob ; tmpJob = !jobs.Advance() ? 
			         jobs.Element() : NULL ) 
	{
		if ( aJob == tmpJob->ptrToJob )
		{
			jobs.DeleteAndAdvance() ;

			delete tmpJob ;  

			return 0 ;
		}
	}

	return -1 ;
}

void
Machine_t::Print()
{
	JobInstances_t*  tmpJob = NULL ;
											 
	cout << ipAddress << "\t" << numProcessors << "\t" << numProcessorsFree ;

	if ( jobs.IsEmpty() )    
		return ;

	cout << "\tJobId" ;  

	for ( jobs.StartTraversal() , tmpJob = jobs.Element() ;
			tmpJob ;
			tmpJob = !jobs.Advance() ? jobs.Element() : NULL ) 
	{
		cout << " " << tmpJob->ptrToJob->jobId ;
	}
}

JobInstances_t*
Machine_t::SearchJob( int jobId )
{
	if ( jobs.IsEmpty() )
		return NULL ;

	JobInstances_t*  tmpJob = NULL ;

	for ( jobs.StartTraversal() , tmpJob = jobs.Element() ;
			tmpJob ;
			tmpJob = !jobs.Advance() ? jobs.Element() : NULL ) 
	{
		if ( jobId == tmpJob->ptrToJob->jobId )
		{	
			return tmpJob ;  

		}
	}

	return NULL ; 
}

int
Machine_t::SearchJob( Job_t* aJob )
{
	if ( jobs.IsEmpty() )
		return NULL ;

	JobInstances_t*  tmpJob = NULL ;

	for ( jobs.StartTraversal() , tmpJob = jobs.Element() ;
			tmpJob ;
			tmpJob = !jobs.Advance() ? jobs.Element() : NULL ) 
	{
		if ( aJob == tmpJob->ptrToJob)
		{	
			return 0 ;  

		}
	}

	return -1 ; 
}

Machine_t* 
HashTable_t::Search( const char* key )
{
	assert( key != NULL ) ;

	int hashValue = HashFunction(key) ;
		
	if ( collisionLists[hashValue].IsEmpty() )
		return NULL ;

	Machine_t* tmpMachine = NULL ;

	collisionLists[hashValue].StartTraversal() ;

	for ( collisionLists[hashValue].StartTraversal(), 
	      tmpMachine = collisionLists[hashValue].Element() ;
	      tmpMachine ; 
	      tmpMachine = !collisionLists[hashValue].Advance() ? 
			      collisionLists[hashValue].Element() : NULL ) 
	{
		if ( !::strncmp(key, tmpMachine->ipAddress, MAXHOSTNAMELEN) ) 
		{
			return tmpMachine ;
		}
	}

	return NULL ;
} 

int 	
HashTable_t::Insert( Machine_t* aMachine )
{
	assert( aMachine != NULL ) ;

	int hashValue = HashFunction( aMachine->ipAddress ) ;
		
	collisionLists[hashValue].StartTraversal() ;

	collisionLists[hashValue].Insert(aMachine) ;

	// processorsFree += aMachine->numProcessors ;

	return 0 ; 
} 

// This implements one policy. We might need another policy  
Machine_t*
HashTable_t::GetFreeMachine( int reqNumber, int& allocated )
{
	int startScanPos = lastAllocated ;	

	if ( processorsFree < reqNumber )
		return NULL ;

	if ( currMachine && currMachine->status == OPERATIONAL 
	     && currMachine->numProcessorsFree > 0	)
	{
		finish :

		allocated = reqNumber >= 
			currMachine->numProcessorsFree ?
			currMachine->numProcessorsFree : reqNumber ; 

		processorsFree -= allocated ;

		currMachine->numProcessorsFree -= allocated ;

		return currMachine ;	
	}

	// Now we have to search for a machine that has
	// free processors 

	// Handling for the first time 
 	if ( currMachine == NULL )
		currMachine = collisionLists[lastAllocated].FirstElement() ;

	while (1)
	{
		if ( currMachine && currMachine->status == OPERATIONAL 
	         		 && currMachine->numProcessorsFree > 0 )
		{
			goto finish ; 		
		}

		if ( currMachine == NULL )
		{
			lastAllocated = ++lastAllocated % Constant::MAX_HASHTABLELISTS ;

			if ( lastAllocated == startScanPos )  
				break ; 

			currMachine = 
			collisionLists[lastAllocated].FirstElement() ;
		}
		else
		{
			currMachine = collisionLists[lastAllocated].FindNext(currMachine) ;	
		}
	}

	cout << "Unable to allocate nodes :" << currMachine->ipAddress 
		<< " " << reqNumber << endl ; 

	return NULL ;
}

// Since this is a static hash table we might not need delete 
Machine_t* 
HashTable_t::Delete( const char* key )
{
	assert( key != NULL ) ;

	int hashValue = HashFunction( key ) ;
		
	if ( collisionLists[hashValue].IsEmpty() )
		return NULL ;

	Machine_t* tmpMachine = NULL ;  

	for ( collisionLists[hashValue].StartTraversal(), tmpMachine = collisionLists[hashValue].Element() ;
	      tmpMachine ; 
	      tmpMachine = !collisionLists[hashValue].Advance() ? collisionLists[hashValue].Element() : NULL ) 
	{
		if ( !::strncmp(key, tmpMachine->ipAddress, MAXHOSTNAMELEN) ) 
		{
			collisionLists[hashValue].DeleteAndAdvance() ;

			return tmpMachine ;
		}
	}

	return NULL ;
} 
		
void
HashTable_t::Print()
{
	Machine_t*	tmpMachine = NULL ;

	for ( int i = 0 ; i < Constant::MAX_HASHTABLELISTS ; i++ )
	{
		if ( collisionLists[i].IsEmpty() )
			continue ;	
			
		collisionLists[i].StartTraversal() ;
	
		for ( tmpMachine = collisionLists[i].Element();
		      tmpMachine ;
		      tmpMachine = !collisionLists[i].Advance() ? 
			      collisionLists[i].Element() : NULL ) 
		{
			cout << i << " " ;

			tmpMachine->Print() ;

			cout << endl ; 
		}
	}
} 
		
int
BuildMachineList( HashTable_t& machList, const char* configFileName )
{
	int count = 0 ; 

	assert ( configFileName != NULL ) ;

	FILE* fp = fopen(configFileName,"r") ;

	if ( !fp )
	{
		cerr << "Error opening configuration file " << configFileName << endl ;

		return -1 ;
	}

	char hostAddress[BUFSIZ] ;

	char noOfProcessors[BUFSIZ] ;

	Machine_t*	aMachine = NULL ;

	while ( !feof(fp) )
	{
		fscanf(fp,"%s %s\n",hostAddress,noOfProcessors ) ;

		if ( machList.Search( hostAddress ) ) 
		{
			cerr << "Duplicate Entry in " << configFileName 
			     << " for " << hostAddress << endl ;

			continue ;
		}

		if ( (aMachine = new Machine_t ) == NULL )
		{
			cerr << "Unable to allocate memory" << endl ;

			continue ; 
		}

		aMachine->ipAddress = hostAddress ;

		// For GSP this might have to be zero
		aMachine->numProcessors = 
		aMachine->numProcessorsFree = atoi(noOfProcessors) ;

		machList.Insert( aMachine ) ;

		count++ ; 
	}

	fclose(fp) ; 

	return count ;
}

int
ActivateMachine( HashTable_t& machList, const char* ipAddress, int numProcessors )
{
	Machine_t* tmpMachine = machList.Search(ipAddress) ;

	if ( tmpMachine )
	{
		// Change the status to the fact that the
		// machine is operational
		tmpMachine->status = OPERATIONAL ;

		// The machine might be adverstising only
		// a certain number of processors. Use only
		// them.
		tmpMachine->numProcessors = numProcessors ;

		// Set the number of free processors
		tmpMachine->numProcessorsFree = numProcessors ;

		// Increase the total number of free processors
		// in the system
		machList.processorsFree += numProcessors ;

		return 0 ;
	}

	return -1 ;
}

int
DeActivateMachine( HashTable_t& machList, const char* ipAddress, int numProcessors )
{

	Machine_t* tmpMachine = machList.Search(ipAddress) ;

	// First check whether the machine has not been scheduled 
	// any processes
	if ( tmpMachine && tmpMachine->numProcessorsFree == tmpMachine->numProcessors &&
		tmpMachine->status == OPERATIONAL )
	{
		// Decrease the total number of free processors
		// in the system
		machList.processorsFree -= numProcessors ;

		// Change the status to the fact that the
		// machine is not operational
		tmpMachine->status = NOT_AVAILABLE ;

		tmpMachine->numProcessors = 0 ;

		// Set the number of free processors
		tmpMachine->numProcessorsFree = 0 ;

		return 0 ;
	}

	return -1 ;
}

