#include <stdio.h>
#include <sys/param.h>
#include "Message.h"
#include "GSPEntry.h"
#include "GSPMachine.h"
#include "GSPJob.h"

extern  HashTable_t                     machineList ;
extern  oo_List<RunningJob_t>           runningList ;
extern  oo_List<Job_t>           	jobList ;
extern  char                            MyIPAddress[MAXHOSTNAMELEN] ;

int
SendMessage( MsgTypes_t message_type , Job_t* aJob, char* hostAddr )
{
	assert( hostAddr != NULL ) ;

	int clientSock = -1;

	MessHeader_t msg(MyIPAddress) ;

	clientSock = CreateClientSocket( hostAddr, Constant::LSPPortNumber ) ;

	if ( clientSock < 0 )
	{
		return -1 ; 
	}

	msg.msgType = message_type ;

	msg.jobId = aJob->jobId ; 

	msg.Send( clientSock ) ;

	close(clientSock) ;

	return 0 ; 
}

int
BuildAndSendMessage( MsgTypes_t message_type , Job_t* aJob )
{
	assert ( aJob != NULL ) ;

	// Get the running entry pointer for the job aJob
	RunningJob_t*	aRunJob = aJob->runEntryPtr ;

	MachInfo_t*  tmpMach ;

	if ( aRunJob->runningMachList.IsEmpty() )
		return -1 ;

	aRunJob->runningMachList.StartTraversal();

	for (  tmpMach = aRunJob->runningMachList.Element() ;
	      tmpMach ; tmpMach = !aRunJob->runningMachList.Advance() ? 
			         aRunJob->runningMachList.Element() : NULL ) 
	{
		// Send to the machine the message 
		cout << tmpMach->machPtr->ipAddress << endl ; 

		SendMessage( message_type, aJob, tmpMach->machPtr->ipAddress) ;

		if ( message_type == MSG_KILL )
		{
			// Increase the number of free processors on this node 
			tmpMach->machPtr->numProcessorsFree += tmpMach->prsrAllocated ;

			// Release all the processor allocated for this job 
			tmpMach->prsrAllocated = 0  ;

			tmpMach->machPtr->DeleteJob( aJob->jobId ) ;	
		}	
	}

	return 0 ;
}

int
KillAJob( int jobId )
{
	// Get the pointer to the job entry 
	Job_t*	aJob = Search( jobList, jobId ) ;  

	if ( aJob == NULL )
		return -1 ; 

	BuildAndSendMessage( MSG_KILL, aJob ) ;

	// Update the number of processors in the system
	machineList.processorsFree += aJob->numNodesRequired ;

	// Release the memory  
	unlink( aJob->jobConfigFile.chars() ) ;

	// Delete the running instance of this job from the list
	runningList.FindAndDelete( aJob->runEntryPtr ) ;

	// Free the memory for running instance
	// by implicitly calling destructors
	delete aJob->runEntryPtr ;

	// Delete the job entry from the jobList
	jobList.FindAndDelete( aJob ) ;

	// Free the memory for the job
	delete aJob ;     // Implicitly frees the memory for the entries

	return 0 ; 
}

int
SuspendJob( int jobId ) 
{
	// Get the pointer to the job entry 
	Job_t*	aJob = Search( jobList, jobId ) ;  

	if ( aJob == NULL )
		return -1 ; 

	if ( aJob->jobStatus != JOB_RUNNING )
		return -1 ; 

	BuildAndSendMessage( MSG_SUSPEND, aJob ) ;

	aJob->jobStatus = JOB_SUSPEND ;  
		
	return 0 ;
}

int
ResumeJob( int jobId ) 
{
	// Get the pointer to the job entry 
	Job_t*	aJob = Search( jobList, jobId ) ;  

	if ( aJob == NULL )
		return -1 ; 

	if ( aJob->jobStatus != JOB_SUSPEND )
		return -1 ; 

	BuildAndSendMessage( MSG_RESUME, aJob ) ;

	aJob->jobStatus = JOB_RUNNING ;  
		
	return 0 ;
}
		
