#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <errno.h>
#include <assert.h>

#include <sys/types.h>
#include <sys/time.h> 
#include <sys/timeb.h>

#include "Functions.h"
#include "Message.h"
#include "GSPJob.h"
#include "List.h"
#include "queue.h"

extern	String			submitDir ;
extern	job_queue		policies ; 
extern	oo_List<Job_t>		jobList ;
extern	char			MyIPAddress[MAXHOSTNAMELEN] ; 
int	globalJobIdCounter = 0 ;


// --------------------------------------------------------------------------------
// SubmitJob -- Given a jobId and the job Configuration File creates a new entry
//		for the job and appends it into jobList 
// 		Returns 0 on success
//		Returns -1 otherwise
// --------------------------------------------------------------------------------

int
SubmitJob( int jobId, int numNodes, String& jobConfigFile )
{
	assert( jobId > 0 ) ;	

	if ( CheckFileExistence( jobConfigFile ) ) 
		return -1 ;
	
	Job_t*	newJob = new Job_t(jobId,numNodes) ;

	if ( newJob == NULL )
	{
		cerr << " Unable to allocate memory " << endl ;

		return -1 ;
	}	

	newJob->jobConfigFile = jobConfigFile ;

	// Set the priority equal to the number of nodes
	newJob->currPriority = numNodes ;

	// Store the arrival time of the job
	ftime( &(newJob->jobArrivalTime) ) ;

	Insert( jobList, newJob ) ;

	policies.enqueue( newJob ) ; 

	return 0 ; 
}

inline int
GetTransientJobId()
{
	int jobId = (globalJobIdCounter+1) % Constant::MAXJOBID ;

	if ( jobId == 0 ) 
		return jobId + 1 ;

	return jobId ;  
}

inline int
GetPermanentJobId()
{
	int jobId = ++globalJobIdCounter % Constant::MAXJOBID ;

	if ( jobId == 0 ) 
	{
		++globalJobIdCounter ;

		return globalJobIdCounter ;
	}

	return jobId ; 
}

int
ProcessSubmit( int clientS ) 
{
	MessHeader_t submitMsg(MyIPAddress) ;

	int jobId = GetTransientJobId() ;

	submitMsg.ConstructMessage( MSG_SUBMIT_REPLY,jobId ) ;

	// Send the transient jobId 
	if ( submitMsg.Send( clientS ) < 0 )
	{
		cerr << "Error in sending submit reply" << sys_errlist[errno] << endl ;

		return -1 ; 
	}

	// Wait for the commit or abort message 
	if ( submitMsg.Receive( clientS ) < 0 )
	{
		cerr << "Error in receving submit commit or abort" << sys_errlist[errno] << endl ;

		return -1 ;
	}

	cout << submitMsg << endl ;

	if ( submitMsg.msgType == MSG_SUBMIT_COMMIT )
	{
		String	fileName ;

		char	number[32] ; 

		// Get the permanent job id 	
		jobId = GetPermanentJobId() ; 

		fileName = submitDir ;

		sprintf(number,"%d",jobId) ;

		fileName += "job." ;

		fileName += number ;  

		return SubmitJob(jobId, submitMsg.numNodes, fileName ) ;
	}
	else if ( submitMsg.msgType == MSG_SUBMIT_ABORT )
	{
		return -1 ;
	}

	return -1 ; 
}
