#include <iostream.h>
#include <assert.h> 
#include <stdlib.h>
#include <String.h>

#include <netdb.h> 
#include "Constant.h"
#include "Functions.h"
#include "Message.h"

main(int argc,char* argv[])
{
	extern int 	optind ;
	extern char*	optarg ; 

	int 		rbytes = 0 ; 
	int		rsofar = 0 ;
	int		queryPort ; 
	char*		jobId = NULL ;
	char*		GSPHostName = NULL ;
	char*		portNumber = NULL ;

	int		c ;
	int		kill_f = 0 ;
	int		suspend_f = 0 ;
	int		resume_f = 0 ;


	while ( (c = getopt(argc,argv,"k:s:p:u:r:") ) != EOF )
	{
		switch(c)
		{
			case 'k' :
				kill_f = 1 ; 
				jobId = optarg ;
				break ;

			case 'u' :
				suspend_f = 1 ; 
				jobId = optarg ;
				break ;

			case 'r' :
				resume_f = 1 ;
				jobId = optarg ;
				break ;

			case 's' :
				GSPHostName = optarg ;
				break ;

			case 'p' :
				portNumber = optarg ;
				break ;

			case '?' :

				cerr << "Usage: " << argv[0] << 
				" -s <HostName> {-k -r -u }<job id> [-p Query Port Number] " << endl ;
				exit(1) ;	
		}
	}

	if ( !GSPHostName )
	{
		cerr << "Usage: " << argv[0] << 
			" -s <HostName> {-k -r -u }<job id> [-p Query Port Number] " << endl ;
		exit(1) ;
	}

	if ( !jobId )
	{
		cerr << "Usage: " << argv[0] << 
			" -s <HostName> {-k -r -u }<job id> [-p Query Port Number] " << endl ;
		exit(1) ;
	}

	// Check the validity of the host name
	struct hostent* hostEnt = OurGetHostEntry( GSPHostName ) ;

	if ( !hostEnt )
	{
		cerr << "Server " << GSPHostName << " does not exist." << endl ;

		exit(1) ;
	}	

	// Check the validity of the port number  
	if ( ( queryPort = ValidatePortNumber( portNumber, QUERY_PORT_NUMBER ) ) < 0 )
	{
		cerr << "Query Port Number should be a valid integer" << endl ;

		exit(1) ;
	}

	// Get my host name
	char* hostName = GetHostName() ; 

	if ( hostName == NULL )
	{
		cerr << "Unable to get my host name." << endl ;

		exit(1) ;
	}

	char* ipAddress = OurGetHostAddress( hostName ) ;

	if ( ipAddress == NULL )
	{
		cerr << "Unable to get my ip Address." << endl ;

		exit(1) ;
	}

	int clientSock = CreateClientSocket( GSPHostName, queryPort ) ;  

	if ( clientSock < 0 )
	{
		cerr << "Unable to connect to "  << GSPHostName 
		     << ". Quitting..." << endl ;

		exit(1) ;
	}

	MessHeader_t message(ipAddress) ; 

	char* answer ;

	if ( jobId && kill_f ) 
	{	
		// Kill a specific job
		message.ConstructMessage( MSG_KILL, atoi(jobId) ) ;
	}
	else if ( jobId && suspend_f )
	{
		// Suspend a specific job
		message.ConstructMessage( MSG_SUSPEND, atoi(jobId) ) ;
	}
	else if ( jobId && resume_f )
	{
		// Resume a specific job
		message.ConstructMessage( MSG_RESUME, atoi(jobId) ) ;
	}
	else
	{
		cerr << "Job Id is not given"  << endl ; 

		goto finish ;
	}


	if ( message.Send( clientSock ) < 0 )
	{
		goto finish ;
	}

	close(clientSock) ;

	cout.flush() ;

	exit(0) ;

finish :

	close(clientSock) ;

	exit(1) ;
}
