#include <iostream.h>
#include <assert.h> 
#include <stdlib.h>
#include <sys/signal.h>

#include <netdb.h> 
#include <String.h>
#include "Constant.h"
#include "Functions.h"
#include "Message.h"


// This is global since in the signal handler we might have to remove the file
static String	jobFileName ;
static int	clientSock ; 	
static char*	ipAddress ; 	
static char*	serverName ;
int			abortPoint = 0 ; 

// TO DO:
// Parse the job configuration file to get the number nodes
// Copy the job configuration file to an area of scheduling
// so that user cannot change it 
// Check for the error condition that the job does not contain
// zero nodes  

int
SendAbortMessage()
{
	MessHeader_t msg(ipAddress) ;

	msg.ConstructMessage( MSG_SUBMIT_ABORT, -1 ) ;

	if ( msg.Send( clientSock ) < 0 )
	{
		cerr << "Unable to send abort message to GSP " 
			<< ". Quitting..."  << endl ;

		return -1 ;
	}	

	return 0 ; 
}

int
GetNumNodes( const char* jobConfigFile)
{
	long	expCompletionTime ;

	int	numNodes = 0 ; 

	FILE* fp = fopen(jobConfigFile,"r") ;

	if ( fp == NULL )
	{
		cerr << "Unable to open the jobConfiguration file " 
		     << jobConfigFile << endl  ;

		return -1 ;
	}

	// Expected Completion time in terms of seconds
	// TO BE ENCHANCED LATER 

	// Error Checking has to be done in the parser planning to write

	fscanf(fp,"%d\n",&expCompletionTime) ;

	String tmp ;
	
	tmp.alloc(BUFSIZ) ;

	while( !feof(fp) )
	{
		int pos = 0 ;
		int follow = 0 ; 
		String instances ;

		if ( fgets(tmp, BUFSIZ, fp) == NULL )
			break ;


		// TO DO: Currently the each parameters
		// is seperated by EXACTLY ONE SPACE
		// has to be improved

		// Get the number of instances
		follow = tmp.index( ' ',pos ) ;
		instances = tmp( pos, follow - pos ) ;
		pos = follow + 1 ;
		numNodes += atoi(instances.chars() ) ;
	}
	
	fclose(fp) ;

	return numNodes ; 
}

void
signal_INT( int x )
{
	if ( abortPoint )
	{
		// Check the fileName has been formed
		if ( jobFileName.empty() ) 
		{
			unlink( jobFileName.chars() ) ;
		}

		SendAbortMessage() ; 
	}

	close(clientSock) ;

	exit(1) ;
}

	

main(int argc,char* argv[])
{
	extern int 	optind ;
	extern char*	optarg ; 

	String		cmd ;
	String		submitDir ;
	char		number[32] ;
	char*		fileName = NULL ;
	char*		serverName = NULL ;
	char*		portNumber = NULL ;
	char*		jobDir = NULL ;	

	int		c ;
	int		numNodes = 0 ; 

	while ( (c = getopt(argc,argv,"f:s:p:d:") ) != EOF )
	{
		switch(c)
		{
			case 'f' :
				fileName = optarg ;
				break ;
			case 's' :
				serverName = optarg ;
				break ;

			case 'p' :
				portNumber = optarg ;
				break ;

			case 'd' :
				jobDir = optarg ;
				break ;

			case '?' :
				cerr << "Usage: " << argv[0] << 
				" -f <Job Configuration File> -s <GSP HostName> "
				<< "[ -p <GSP User Port Number> -d <Job Submission Dir> ]" << endl ;
				exit(1) ;	
		}
	}

	if ( !fileName || !serverName )
	{
		cerr << "Usage: " << argv[0] << 
		" -f <Job Configuration File> -s <GSP HostName> "
		<< "[ -p <GSP User Port Number> "
		<< "-d <Job Submission Dir> ]" << endl ;
		exit(1) ;
	}

	// Check whether the job configuration file exists or not
	if ( CheckFileExistence( fileName ) )
	{
		cerr << fileName << " does not exist. Quitting..." << endl ;

		exit(1) ;
	}

	// Check the validity of the port number  
	if ( portNumber && CheckValidInteger(portNumber) )
	{
		cerr << "User Port Number should be a valid integer" << endl ;

		exit(1) ;
	}

	if ( !jobDir ) 
	{
		submitDir = JobSubmissionDir ;
	}
	else
	{
		submitDir = jobDir ; 
	}

	// Check the validity of the host name
	struct hostent* hostEnt = OurGetHostEntry( serverName ) ;

	if ( !hostEnt )
	{
		cerr << "GSP Server " << serverName << " does not exist." << endl ;

		exit(1) ;
	}	

 	ipAddress = OurGetHostAddress( serverName ) ;

	if ( ipAddress == NULL )
	{	
		cerr << "Unable to get my IP Address " << serverName 
			<< ". Quitting..." << endl ;

		exit(1) ;
	}
	
	// Install the signal interrupt handler
	signal( SIGINT,signal_INT ) ; 		

	// Get the number of nodes 
	if ( (numNodes = GetNumNodes( fileName )) < 1 )
	{
		cerr << "Cannot submit a job that requires 0 nodes" << endl ;

		exit(1) ;
	}

	MessHeader_t msg(ipAddress) ;

	clientSock = CreateClientSocket( serverName, 
				portNumber ? atoi(portNumber) : Constant::UserPortNumber ) ;

	if ( clientSock < 0 )
	{
		cerr << "Unable to connect to GSP server "  << serverName 
		     << ". Quitting..." << endl ;

		exit(1) ;
	}

	msg.ConstructMessage( MSG_SUBMIT, -1 ) ;

	if ( msg.Send( clientSock ) < 0 ) 
	{
		cerr << "Unable to send to GSP server " << serverName 
			<< ". Quitting..." << endl ;

		goto finish ; 
	}

	// TO DO: Get the number of nodes TO DO
	// Parse the number of nodes while the GSP server is processing the request

	if ( msg.Receive( clientSock ) < 0 )
	{
		cerr << "Unable to receive jobid from GSP server " << serverName 
			<< ". Quitting..."  << endl ;
	
		goto finish ; 
	}

	abortPoint = 1 ;

	jobFileName = submitDir ;
	jobFileName += "/job." ; 
	sprintf( number,"%d",msg.jobId ) ; 
	jobFileName += number ;  

	// Form the command 
	cmd 	= 	"cp " ;
	cmd 	+= 	fileName ;
	cmd	+=	" " ;   
	cmd	+=	jobFileName ;   

	// Copy the file into the designated job area
	if ( system(cmd.chars()) != 0 ) 
	{
		SendAbortMessage() ;

		cerr << "Unable to copy the file"  << endl ; 

		goto finish ; 	
	}

	// Send the committed message 
	msg.SetIPAddress( ipAddress ) ;

	msg.ConstructMessage( MSG_SUBMIT_COMMIT, -1 ) ;

	// TO DO Set the number of nodes field
	msg.numNodes = numNodes ; 

	if ( msg.Send( clientSock ) < 0 )
	{
		cerr << "Unable to send the commit message to " << serverName
			<< ". Quitting..."  << endl ;

		goto finish ;
	}
		
	close(clientSock) ;

	cout << "Your job has been successfully submitted." << endl ;

	cout << "Job Id is " << number << endl ;

	exit(0) ;

finish : 

	close(clientSock) ;

	exit(1) ;
}
