#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <errno.h>
#include <assert.h>

// Socket related include files
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <sys/time.h>
#include "Functions.h"
#include "JobTable.h"
#include "Message.h"

extern char* 	sys_errlist[] ; 

//JobTable	LocalJobTable ;

const int	hostNameRetries = 7 ;	

// -----------------------------------------------------------------------------------
// OurGetHostByName -- This is due to the slowness of the name binding software. If it
//                     fails for the first name, then it might be due to non caching
//		       of names.  	 
//                     The number of times we retry is adhoc
// -----------------------------------------------------------------------------------
struct hostent*
OurGetHostByName( const char* hostName )
{
	register struct hostent*	tmpHostEnt ;

	assert( hostName != NULL ) ;

	for ( int i = 0 ; i < hostNameRetries ; i++ )
	{
		tmpHostEnt = gethostbyname( hostName ) ;

		if ( tmpHostEnt )
		{
			return tmpHostEnt ;
		}
	}

	return NULL ;
}

// -----------------------------------------------------------------------------------
// OurGetHostByAddr -- This is due to the slowness of the name binding software. If it
//                     fails for the first name, then it might be due to non caching
//		       of names.  	 
//                     The number of times we retry is adhoc
// -----------------------------------------------------------------------------------
struct hostent*
OurGetHostByAddr( const char* hostAddr, int Length, int addrType )
{
	register struct hostent*	tmpHostEnt ;

	assert( hostAddr != NULL ) ;

	for ( int i = 0 ; i < hostNameRetries ; i++ )
	{
		tmpHostEnt = gethostbyaddr( hostAddr, Length, addrType ) ;

		if ( tmpHostEnt )
		{
			return tmpHostEnt ;
		}
	}

	return NULL ;
}

int
CreateServerSocket( int portNumber )  
{
	register int sockFd ;

	struct sockaddr_in servAddr ;  

	sockFd = socket(AF_INET,SOCK_STREAM,0) ;

	if ( sockFd < 0 )
	{
		cerr << "Error creating server socket:" << sys_errlist[errno] << endl ;
		return -1 ;
	}


	bzero( (char *)&servAddr, sizeof(struct sockaddr_in) ) ;  

	servAddr.sin_family = AF_INET ;
	servAddr.sin_addr.s_addr = htonl(INADDR_ANY) ;
	servAddr.sin_port = htonl(portNumber) ;

	if ( bind(sockFd,(struct sockaddr *)&servAddr, sizeof(servAddr) ) < 0 )
	{
		cerr << "Error binding server socket:" << sys_errlist[errno] << endl ;
		close(sockFd) ;
		return -1 ; 
	}

	return sockFd ;
}

// -----------------------------------------------------------------------------------
// CreateClientSocket - Creates a client socket for communication with GSP. 
//                      Has to be converted to be generic so that the parameter
//                      hostName is either a real host name or dotted ip address
// -----------------------------------------------------------------------------------
int
CreateClientSocket(const char* hostNameORAddr, int servPortNumber )
{
	register int 			sockFd ;
	register struct hostent* 	servHostEnt ;
	register struct in_addr* 	inAddrEnt ;
	struct sockaddr_in 		servAddr ;  

	int	 			error ;

 	assert( hostNameORAddr != NULL ) ;

	sockFd = socket(AF_INET,SOCK_STREAM,0) ;

	if ( sockFd < 0 )
	{
		cerr << "Error creating client socket:" << sys_errlist[errno] << endl ;
		return -1 ;
	}

	bzero( (char *)&servAddr, sizeof(servAddr) ) ;  

	// Assuming the host name is 'real name' not ip address 
	// has to be converted later

	servHostEnt = OurGetHostByName( hostNameORAddr ) ;

	if ( !servHostEnt )
	{
		cerr << "Error gethostbyname for " << hostNameORAddr << endl ;
		close(sockFd) ;
		return -1 ;
	}	

	servAddr.sin_family = AF_INET ;
	servAddr.sin_port = htonl(servPortNumber) ;

	// Loop thru' the host entry address list since it is possible
	// some GSP machines have multiple IP addresses

	inAddrEnt = (struct in_addr *) *(servHostEnt->h_addr_list) ;

	while ( inAddrEnt )
	{
		// Try connecting the current iterating address 
		servAddr.sin_addr.s_addr = inet_addr( inet_ntoa(*inAddrEnt) ) ;

		error = connect(sockFd, (struct sockaddr *)&servAddr,sizeof(servAddr)) ; 

		// Successfully connected 
		if ( error >= 0 ) 
			return sockFd ;

		// try other addresses  
		servHostEnt->h_addr_list++ ;

		inAddrEnt = (struct in_addr *) *(servHostEnt->h_addr_list) ;
	}

	// cerr << "Error connecting to " << servHostEnt->h_name << ':' << sys_errlist[errno] << endl ;
	close(sockFd) ;
	return -1 ;	
}

void
main(int argc, char* argv[] )
{
	MessHeader_t msg(MSG_SCHEDULE,5,100,0) ;

	struct sockaddr_in clientAddr ; 

	int clientAddrLen ;

	assert( argc == 3 ) ;

	int clientSock = 10 ;

	int serverSock ; 

	// serverSock = CreateServerSocket( atoi(argv[2]) ) ;

	fd_set readset ;

	// Create a client socket so that it can send to another machine 

	// while ( (clientSock = CreateClientSocket( argv[1], atoi(argv[2]) ) ) < 0 ) ;

	// if ( clientSock < 0 )
	//	exit(1) ;

	// msg.Send(clientSock) ;

	while (1)
	{
		while ( (clientSock = CreateClientSocket( argv[1], atoi(argv[2]) ) ) < 0 ) ;

		if ( clientSock < 0 )
			exit(1) ;

		msg.Send(clientSock) ;

		close(clientSock) ;

		/*
		FD_ZERO(&readset) ;

		FD_SET(serverSock,&readset) ;

		int s = select( serverSock+1 , (fd_set *)&readset, NULL,NULL, NULL ) ;

		// listen( serverSock, 5) ;

		// Receive on server socket  
		if ( s > 0 && FD_ISSET( serverSock, &readset ) ) 
		{ 
			int clientD = accept( serverSock, 
					(struct sockaddr *)&clientAddr, &clientAddrLen) ;
			
			if (clientD < 0 )
			{
				cerr << "Error in accept:" << sys_errlist[errno] << endl ;
				break ;
			}
			
			msg.Receive(serverSock) ;

			close(clientD) ;	
			cout << msg << endl ; 
		}
		else if ( s < 0 )
		{
			cerr << "Error in select:" << sys_errlist[errno] << endl ;
			break ;
		}
		*/
	}
}
