#include <iostream.h>

#include "GSPJob.h"
#include "queue.h"

job_queue::job_queue()
{
	int i, j;

	// Initialize the header nodes.
	for (i = 0; i <= N_ALGS; i++)
	{
		head_ptr[i] = new node;
		assert(head_ptr[i]);
	}

	curr_q = GLOBAL_Q;	// currently insertions goto the global queue.
	for (i = 0; i <= N_ALGS; i++)
	{
		for (j = 0; j <= N_ALGS; j++)
		{
			head_ptr[i]->prev[j] = NULL;
			head_ptr[i]->next[j] = NULL;
		}
		head_ptr[i]->dat.JobPtr = NULL;	// init data
		head_ptr[i]->prev[i] = head_ptr[i];
		head_ptr[i]->next[i] = head_ptr[i];
		head_ptr[i]->dat.priority = -1;
	}
	 
}


bool job_queue::enqueue(Job_t* newjob)
{

	struct node * ptr;

	ptr = new node;
	assert(ptr);
		
	ptr->dat.JobPtr = newjob;

	ptr->dat.priority = newjob->numNodesRequired ;

	if (head_ptr[GLOBAL_Q]->next[GLOBAL_Q] == head_ptr[GLOBAL_Q])
        {
		// Queues are empty.
		ptr->prev[GLOBAL_Q] = head_ptr[GLOBAL_Q];
		ptr->next[GLOBAL_Q] = head_ptr[GLOBAL_Q];
		head_ptr[GLOBAL_Q]->next[GLOBAL_Q] = ptr;
		head_ptr[GLOBAL_Q]->prev[GLOBAL_Q] = ptr;

	}
	else
	{
		ptr->prev[GLOBAL_Q] = head_ptr[GLOBAL_Q];
		ptr->next[GLOBAL_Q] = head_ptr[GLOBAL_Q]->next[GLOBAL_Q];
		head_ptr[GLOBAL_Q]->next[GLOBAL_Q] = ptr;
		ptr->next[GLOBAL_Q]->prev[GLOBAL_Q] = ptr;
	}


	// Currently all of them insert elements in a LIFO order.
	if (! p1_insert(ptr))
		return FALSE;

	if (! p2_insert(ptr, newjob->numNodesRequired))
		return FALSE;

// The best match list need not be maintained seperately.
// We can just scan the FIFO list to determine the best match.
#if 0
	if (! p3_insert(ptr))
		return FALSE;
#endif
		
	return TRUE;
}


bool job_queue::p1_insert(struct node * ptr)
{
	// This implements a FIFO queue
	if (head_ptr[A1_Q]->next[A1_Q] == head_ptr[A1_Q])
        {
		// Queues are empty.
		ptr->prev[A1_Q] = head_ptr[A1_Q];
		ptr->next[A1_Q] = head_ptr[A1_Q];
		head_ptr[A1_Q]->next[A1_Q] = ptr;
		head_ptr[A1_Q]->prev[A1_Q] = ptr;

	}
	else
	{
		ptr->prev[A1_Q] = head_ptr[A1_Q]->prev[A1_Q];
		ptr->next[A1_Q] = head_ptr[A1_Q];
		head_ptr[A1_Q]->prev[A1_Q] = ptr;
		ptr->prev[A1_Q]->next[A1_Q] = ptr;
	}
	
	return TRUE;
}	


// This inserts `ptr' into a priority queue.
bool job_queue::p2_insert(struct node * ptr, int num_nodes)
{
	ptr->dat.priority = num_nodes;
	if (head_ptr[A2_Q]->next[A2_Q] == head_ptr[A2_Q])
        {
		// Queues are empty.
		ptr->prev[A2_Q] = head_ptr[A2_Q];
		ptr->next[A2_Q] = head_ptr[A2_Q];
		head_ptr[A2_Q]->next[A2_Q] = ptr;
		head_ptr[A2_Q]->prev[A2_Q] = ptr;
		return TRUE;
	}

	struct node* scan_ptr = head_ptr[A2_Q]->next[A2_Q];

	while (scan_ptr != head_ptr[A2_Q])
	{
		if (scan_ptr->dat.priority > ptr->dat.priority)
			break;

		scan_ptr = scan_ptr->next[A2_Q] ; 
	}

	// Insert the node pointed to by ptr BEFORE
	// the node pointed to by scan_ptr
	ptr->prev[A2_Q] = scan_ptr->prev[A2_Q];
	ptr->next[A2_Q] = scan_ptr;
	scan_ptr->prev[A2_Q]->next[A2_Q] = ptr;
	scan_ptr->prev[A2_Q] = ptr;
	
	return TRUE;
}	


bool job_queue::p3_insert(struct node * ptr)
{
	if (head_ptr[A3_Q]->next[A3_Q] == head_ptr[A3_Q])
        {
		// Queues are empty.
		ptr->prev[A3_Q] = head_ptr[A3_Q];
		ptr->next[A3_Q] = head_ptr[A3_Q];
		head_ptr[A3_Q]->next[A3_Q] = ptr;
		head_ptr[A3_Q]->prev[A3_Q] = ptr;

	}
	else
	{
		ptr->prev[A3_Q] = head_ptr[A3_Q];
		ptr->next[A3_Q] = head_ptr[A3_Q]->next[A3_Q];
		head_ptr[A3_Q]->next[A3_Q] = ptr;
		ptr->next[A3_Q]->prev[A3_Q] = ptr;
	}
	
	return TRUE;
}	


//
// policy:
//	FIFO, BEST, PRIORITY
//
bool job_queue::dequeue(int policy, int processorsFree, Job_t*& data)
{
	int q_num;

	switch (policy)
	{
		case FIFO:	// A1_Q
		case BEST:	// A3_Q
			q_num = A1_Q;
			break ;
		case PRIORITY:
			q_num = A2_Q;
			break ; 
		default:
			cerr << "Invalid QUEUE\n";
			break ;
	}
		
	// A1_Q is FIFO, A2_Q is priority based, A3_Q is best fit.

	if (head_ptr[0]->next[0] == head_ptr[0])
		return FALSE;

	struct node * ptr = head_ptr[q_num]->next[q_num];
	struct node * temp = NULL;
	struct node * scan_ptr, *scan_ptr2, *temp_ptr;

	// Now. find the pointer to the node to be deleted.

	if (policy == BEST)	// adjust ptr
	{
		temp = ptr;
		while (ptr != head_ptr[q_num])
		{
			if (ptr->dat.JobPtr->numNodesRequired == processorsFree)
			{
				temp = ptr;
				break;
			}
			else if (ptr->dat.JobPtr->numNodesRequired < processorsFree)
			{
				if (temp->dat.JobPtr->numNodesRequired < 
					ptr->dat.JobPtr->numNodesRequired)
				{
					temp = ptr;
				}
			}
			ptr = ptr->next[q_num] ;
		}
		ptr = temp;
	}

	if (ptr->dat.JobPtr->numNodesRequired > processorsFree)
		return FALSE ;
		

//	cout << ptr->dat.JobPtr->numNodesRequired << endl ;

	data = ptr->dat.JobPtr;

	int i;

	for (i = 0; i <= N_ALGS; i++)
	{
		ptr->prev[i]->next[i] = ptr->next[i];
		ptr->next[i]->prev[i] = ptr->prev[i];
	}

	scan_ptr = head_ptr[A1_Q]->next[A1_Q];
	while (scan_ptr != head_ptr[A1_Q])
	{
		temp_ptr = scan_ptr->next[A2_Q];
		if (scan_ptr->dat.JobPtr->jobId < data->jobId)
		{
			scan_ptr->dat.priority--;
			scan_ptr2 = head_ptr[A2_Q]->next[A2_Q];
			while (scan_ptr2 != scan_ptr)
			{
				if (scan_ptr2->dat.priority >= scan_ptr->dat.priority && 
				    scan_ptr != scan_ptr2)
				{
					scan_ptr->prev[A2_Q]->next[A2_Q] = scan_ptr->next[A2_Q];
					scan_ptr->next[A2_Q]->prev[A2_Q] = scan_ptr->prev[A2_Q];
					scan_ptr->prev[A2_Q] = scan_ptr2->prev[A2_Q];
					scan_ptr->next[A2_Q] = scan_ptr2;
					scan_ptr2->prev[A2_Q]->next[A2_Q] = scan_ptr;
					scan_ptr2->prev[A2_Q] = scan_ptr;

					break;
				}
				scan_ptr2 = scan_ptr2->next[A2_Q];
			}
		}
		scan_ptr = temp_ptr;
	}

	delete ptr;

	return TRUE;
}


void job_queue::switch_queue(int new_queue_num)
{
	curr_q = new_queue_num;
	
	assert(curr_q == GLOBAL_Q || curr_q == A1_Q ||
	       curr_q == A2_Q     || curr_q == A3_Q);
}


void job_queue::print(int q_num)
{
	struct node * ptr, * hd;
	hd = head_ptr[q_num];
	ptr = hd->next[q_num];

	while (ptr != hd)
	{
//		cout << ptr->dat.junk << "  ";
		ptr = ptr->next[q_num];
	}
	cout << endl;
}
