#include <iostream.h>
#include <assert.h> 
#include <stdlib.h>
#include <String.h>

#include <netdb.h> 
#include "Constant.h"
#include "Functions.h"
#include "Message.h"

main(int argc,char* argv[])
{
	extern int 	optind ;
	extern char*	optarg ; 

	int 		rbytes = 0 ; 
	int		rsofar = 0 ;
	int		lspPort ; 
	char*		jobId = NULL ;
	char*		queryHostName = NULL ;
	char*		portNumber = NULL ;

	int		c ;

	int		withdraw_f = 0 ;
	int		hello_f = 0 ;

	
	while ( (c = getopt(argc,argv,"s:p:wh") ) != EOF )
	{
		switch(c)
		{
			case 's' :
				queryHostName = optarg ;
				break ;

			case 'p' :
				portNumber = optarg ;
				break ;

			case 'w' :
				withdraw_f = 1  ;	
				break ;
				
			case 'h' :
				hello_f = 1 ; 	
				break ;

			case '?' :
				cerr << "Usage: " << argv[0] << 
				" -s <HostName> {-w -h} [ -p <Query Port Number> -w -h ]" << endl ;
				exit(1) ;	
		}
	}

	if ( !queryHostName )
	{
		cerr << "Usage: " << argv[0] << 
		" -s <HostName> {-w -h} [ -p <Query Port Number> ]" << endl ;

		exit(1) ;
	}

	if ( !hello_f && !withdraw_f )
	{
		cerr << "Usage: " << argv[0] << 
		" -s <HostName> {-w -h} [ -p <Query Port Number> ]" << endl ;

		exit(1) ;
	}
		
	// Check the validity of the host name
	struct hostent* hostEnt = OurGetHostEntry( queryHostName ) ;

	if ( !hostEnt )
	{
		cerr << "Server " << queryHostName << " does not exist." << endl ;

		exit(1) ;
	}	

	// Check the validity of the port number  
	if ( ( lspPort = ValidatePortNumber( portNumber, LSP_PORT_NUMBER ) ) < 0 )
	{
		cerr << "Query Port Number should be a valid integer" << endl ;

		exit(1) ;
	}

	// Get my host name
	char* hostName = GetHostName() ; 

	if ( hostName == NULL )
	{
		cerr << "Unable to get my host name." << endl ;

		exit(1) ;
	}

	char* ipAddress = OurGetHostAddress( hostName ) ;

	if ( ipAddress == NULL )
	{
		cerr << "Unable to get my ip Address." << endl ;

		exit(1) ;
	}

	int clientSock = CreateClientSocket( queryHostName, lspPort ) ;  

	if ( clientSock < 0 )
	{
		cerr << "Unable to connect to "  << queryHostName << ". Quitting..." << endl ;

		exit(1) ;
	}

	MessHeader_t message(ipAddress) ; 

	char* answer ;

	if ( withdraw_f ) 
	{	
		// Withdraw 
		message.ConstructWithdrawMessage( 0 ) ;
	}
	else if ( hello_f )   
	{
		message.ConstructStartupMessage( 0 ) ;
	}

	if ( message.Send( clientSock ) < 0 )
	{
		goto finish ;
	}

	close(clientSock) ;

	cout.flush() ;

	exit(0) ;

finish :

	close(clientSock) ;

	exit(1) ;
}
