#include "Constant.h"
#include "LSPJobTable.h"

int
JobTable::Get( int Index )
{
	for ( int i = 0, count = 0 ; i < Constant::MAX_JOBS ; i++ )
	{
		// Possibly an error point
		if ( JobIdTable[i] ) 
		{
			if ( count++ == Index ) 
				return JobIdTable[i] ;
		}
	}

	return -1 ; 
}

int
JobTable::GetJobId(int ProcessId)
{
	for ( int i = 0 ; i < Constant::MAX_JOBS ; i++ )
	{
		if ( JobIdTable[i] )
		{
			for ( int j = 0 ; j < Constant::MAX_PROCESSORS; j++ )
			{
				if ( ProcessId == ProcessIdTable[i][j] ) 
					return JobIdTable[i] ; 
			}
		}
	}

	return PROCESS_ID_NOT_FOUND ; 
}

int
JobTable::GetProcessId(int JobId, int* ProcessIds )
{
	for ( int i = 0 ; i < Constant::MAX_JOBS ; i++ )
	{
		if ( JobId ==  JobIdTable[i] )
		{
			int n = 0 ;

			// Copy the Process Ids into the array given by the user  
			for ( int j = 0 ; j < Constant::MAX_PROCESSORS ; j++ )
			{
				if ( ProcessIdTable[i][j] ) 
				{
					ProcessIds[n] = ProcessIdTable[i][j] ;

					n++ ;
				}
			}

			return n ;
		}
	}

	return JOB_ID_NOT_FOUND ;  
}
					
int
JobTable::InsertJobId( int JobId )
{
	for ( int i = 0 ; i < Constant::MAX_JOBS  ; i++ )
	{
		if ( JobId == JobIdTable[i] )
		{
			return JOB_ID_ALREADY_EXISTS ; 		
		}
		else if ( !JobIdTable[i] ) 
		{
			JobIdTable[i] = JobId ; 

			return 0 ; 	
		}
	}

	return NO_SPACE_FOR_NEW_JOB ; 
}

int
JobTable::InsertProcessId( int JobId, int ProcessId )
{
	// Assumes that the JobId exists in the JobId Table 

	for ( int i = 0 ; i < Constant::MAX_JOBS ; i++ )
	{
		if ( JobId == JobIdTable[i] )
		{
			for ( int j = 0 ; j < Constant::MAX_PROCESSORS ; j++ ) 	
			{
				if ( ProcessId == ProcessIdTable[i][j] )
				{
					return PROCESS_ID_ALREADY_EXISTS ;
				}
				else if ( !ProcessIdTable[i][j] )
				{
					ProcessIdTable[i][j] = ProcessId ;

					return 0 ;
				}
			}

			return NO_SPACE_FOR_NEW_PROCESS ; 
		}
	}

	return JOB_ID_NOT_FOUND ; 
}	

int
JobTable::DeleteJobId( int JobId )
{
	for ( int i = 0 ; i < Constant::MAX_JOBS ; i++ )
	{
		if ( JobId == JobIdTable[i] )
		{
			JobIdTable[i] = 0 ;

			bzero( (char *)ProcessIdTable[i], sizeof(int) 
				* Constant::MAX_PROCESSORS ) ;   	
			return 0 ; 
		}
	}

	return JOB_ID_NOT_FOUND ; 
}

int
JobTable::DeleteProcessId( int JobId, int ProcessId )
{
	for ( int i = 0 ; i < Constant::MAX_JOBS ; i++ )
	{
		if ( JobId == JobIdTable[i] )
		{
			for ( int j = 0 ; j < Constant::MAX_PROCESSORS ; j++ )
			{
				if ( ProcessId == ProcessIdTable[i][j] )				
				{
					ProcessIdTable[i][j] = 0 ;

					return 0 ;
				}	
			}

			return PROCESS_ID_NOT_FOUND ; 
		}
	}	

	return JOB_ID_NOT_FOUND ; 
}

int
JobTable::GetActiveNumberOfJobs()
{
	int count = 0 ;

	for ( int i = 0 ; i < Constant::MAX_JOBS ; i++ )
	{
		if ( JobIdTable[i] )   
		{
			count++ ;  
		}
	}

	return count ;
}


int
JobTable::GetActiveNumberOfProcesses(int JobId)
{
	int count = 0 ;

	for ( int i = 0 ; i < Constant::MAX_JOBS ; i++ )
	{
		if ( JobId == JobIdTable[i] )
		{
			for ( int j = 0 ; j < Constant::MAX_PROCESSORS ; j++ )
			{
				if ( ProcessIdTable[i][j] )					
				{
					count++ ;
				}
			}

			return count ; 
		}	
	}

	return JOB_ID_NOT_FOUND ;
}


/*
main()
{
	JobTable LocalJobTable ;

	JobTable a = LocalJobTable ; 

	LocalJobTable.InsertJobId(5) ;

	LocalJobTable.InsertProcessId(5,10) ;

	LocalJobTable.InsertProcessId(5,20) ;

	LocalJobTable.InsertJobId(10) ;

	LocalJobTable.InsertProcessId(10,25) ;

	LocalJobTable.InsertProcessId(20,25) ;

	a = LocalJobTable ; 

	LocalJobTable.DeleteJobId(10) ; 
}
*/
