#include <iostream.h>
#include "LSP.h"
#include "List.h"

extern int			GSPportNumber ;
extern int			LSPportNumber ;
extern char			GSPipAddress[] ; 
extern char			MyIPAddress[] ;

extern JobTable			LocalHostJobs ;
extern oo_List<LSPStat_t>	jobDetails ;


int
KillJobWithoutLock( int jobId )
{
	int processIds[Constant::MAX_PROCESSORS] ; 

	int numProcess = LocalHostJobs.GetProcessId( jobId, processIds ) ; 

	// Check whether it is a valid jobId for which at least a process exist
	if ( numProcess <= 0 )
	{
		return -1 ; 
	}

	for ( int i = 0 ; i < numProcess ; i++ )
	{
		// Issue the kill signal for each of the process	
		kill(processIds[i],SIGKILL) ;

		// Remove the processId from the Job Table
		LocalHostJobs.DeleteProcessId(jobId, processIds[i] ) ;

		// Delete its entry from the jobDetails list. We have to 
		// consider whether to send the details of the killed job 
		Delete( jobDetails, processIds[i] ) ;  			
	}	

	// Delete the job Id 
	LocalHostJobs.DeleteJobId( jobId ) ; 

	return 0 ;
}

int
KillJob( int jobId )
{
	// Block the SIGCHLD. Even though the killed process will generate
	// SIGCHLD. They will not cause the signal handler to send
	// the terminate message since the job id to process id mapping will not exist

	signal_lock() ;
	
	KillJobWithoutLock( jobId ) ;

	// Unblock the SIGCHLD. 
	signal_unlock() ;
}

// Might not want to send an acknowledgement back to GSP
// since it makes the design of GSP complex.
// Another issue is that the  
// This routine might be useful in GSP to kill a job
int
SendKillMessage(int jobId)
{
	// Send an acknowledgement to the GSP that the job has been killed
	// Create a kill message 	
	MessHeader_t message(MyIPAddress) ;
	
	message.ConstructMessage(MSG_KILL,jobId) ;

	// Create a socket and connect to GSP 
	int clientSock = CreateClientSocket( GSPipAddress, GSPportNumber ) ; 
	
	if ( clientSock < 0 )
	{
		goto finish ;
	}

	if ( message.Send( clientSock ) < 0 )
	{
		goto finish ; 
	}	

	close(clientSock) ;

	return 0 ; 

finish:
	cerr << "Error Sending Kill Message for job " << jobId << endl ;

	if ( clientSock >= 0 )
		close(clientSock) ;

	return -1 ; 
}			

