#include <stdio.h>
#include "LSPStat.h"
#include "List.h"
#include "LSPJobTable.h"
#include "Functions.h"


extern	JobTable	LocalHostJobs ;	
oo_List<LSPStat_t>	jobDetails ; 

LSPStat_t*
Search( oo_List<LSPStat_t>& statCollect, pid_t pid )
{
	if ( statCollect.IsEmpty() )
		return NULL ;

	LSPStat_t*  tmpLSPStat = NULL ;

	for ( statCollect.StartTraversal() , tmpLSPStat = statCollect.Element() ;
		tmpLSPStat ;
		tmpLSPStat = !statCollect.Advance() ? statCollect.Element() : NULL ) 
	{
		if ( tmpLSPStat->processId == pid ) 
			return tmpLSPStat ; 
	}

	return  NULL ;
}

int
Delete( oo_List<LSPStat_t>& statCollect, int pid )
{
	if ( statCollect.IsEmpty() )
		return -1 ;

	LSPStat_t*  tmpLSPStat = NULL ;

	for ( statCollect.StartTraversal() , tmpLSPStat = statCollect.Element() ;
		tmpLSPStat ;
		tmpLSPStat = !statCollect.Advance() ? statCollect.Element() : NULL ) 

	{
		if ( pid == tmpLSPStat->processId )
		{
			statCollect.DeleteAndAdvance() ;

			delete tmpLSPStat ;

			return 0 ;
		}
	}

	return -1 ;
}

void
Print( oo_List<LSPStat_t>& statCollect )
{
	if ( statCollect.IsEmpty() )
		return ;
	
	LSPStat_t*  tmpLSPStat = NULL ;

	for ( statCollect.StartTraversal() , tmpLSPStat = statCollect.Element() ;
		tmpLSPStat ;
		tmpLSPStat = !statCollect.Advance() ? statCollect.Element() : NULL ) 

	{
		cout << tmpLSPStat->processId << endl ;
	}
}

int
Linearize( String& msg, oo_List<LSPStat_t>& statCollect, int jobId)
{
	if ( statCollect.IsEmpty() )
		return -1 ;

	int pids[Constant::MAX_PROCESSORS] ;

	char number[64] ;

	LSPStat_t* tmpLSPStat = NULL ; 

	int numProcess = LocalHostJobs.GetProcessId(jobId,pids) ; 

	if ( numProcess <= 0 )
		return -1 ; 

	struct timeb	currTime ;

	ftime( &currTime ) ; 

	// Put the jid first
	msg += "Job Id :" ;

	sprintf(number,"%d\n",jobId) ; 

	msg += number ;

	for ( int j = 0 ; j < numProcess ; j++ )
	{
		tmpLSPStat = Search( jobDetails, pids[j] ) ; 

		if ( tmpLSPStat )
		{	
			struct timeb timeTaken ;

			sprintf( number,"%d\t",pids[j] ) ; 

			msg += number ;  

			// Tell whether the process is suspended or not
			msg += (tmpLSPStat->status == SUSPENDED ? "S\t" : "R\t") ;

			// Find the amount of time the process has run so far
			DiffTime( currTime, tmpLSPStat->scheduleTime, timeTaken ) ;  

			sprintf( number,"%d:%d\t",timeTaken.time,timeTaken.millitm ) ;

			msg += number ;

			msg += tmpLSPStat->fileName ; 

			msg += "\t" ;

			msg += tmpLSPStat->parameters ;

			msg += "\n" ;
		}
	}

	return 0 ; 
}

int
Linearize( String& msg, oo_List<LSPStat_t>& statCollect )
{
	if ( statCollect.IsEmpty() )
		return -1 ;

	int numJobs = LocalHostJobs.GetActiveNumberOfJobs() ;  

	int jid ; 

	char number[64] ; 

	for ( int i = 0 ; i < numJobs ; i++ )
	{
		jid = LocalHostJobs.Get(i) ;

		Linearize( msg, statCollect, jid ) ;

	}
							
	return 0 ;
}
