#include <fstream.h>
#include "LSP.h"
#include "List.h"

extern int                      GSPportNumber ;
extern int                      LSPportNumber ;
extern char                     GSPipAddress[] ;
extern char                     MyIPAddress[] ;

extern JobTable                 LocalHostJobs ;
extern oo_List<LSPStat_t>       jobDetails ;


// -------------------------------------------------------------------------------------------
// SendTerminatedMessage --  Given a terminated child pid. Maps into the appropriate job Id.
//			     Creates a socket and connects to GSP and sends a terminate message 
//			     to the GSP with the jobid to indicate that a process of the job has 
//			     finished. 
//		 	     Returns 0 if successfully send
//			     Returns -1 if not sent
// -------------------------------------------------------------------------------------------
int
SendTerminatedMessage( int childPid )
{
	int jobId = LocalHostJobs.GetJobId( childPid ) ;

	int clientSock ;

	if ( jobId > 0 )
	{
		// Create a message 	
		MessHeader_t message(MyIPAddress) ; 

		// Get the entry from the jobDetails for the terminated process
		LSPStat_t* lspStat = Search(jobDetails,childPid ) ;

		assert( lspStat != NULL ) ; 

		struct timeb compTime, diffTime ;

		// Get the current time 
		ftime( &compTime ) ; 

		// Calculate the time taken for the process to complete  
		DiffTime( compTime, lspStat->scheduleTime, diffTime ) ;

		// Construct a terminate message with jobId and completed time 
		message.ConstructTerminateMessage( jobId, lspStat->entryNum, &diffTime ) ;  

		// Create a socket and connect to GSP 
		clientSock = CreateClientSocket( GSPipAddress, GSPportNumber ) ; 
		
		if ( clientSock < 0 )
		{
			goto finish ; 
		}

		cout << "Sending: " << message << endl ;

		if ( message.Send( clientSock ) < 0 )
		{
			goto finish ; 
		}	

		close(clientSock) ;

		cout << "Finished: " << message << endl ;

		// Delete the entry for the terminated  process from the jobDetails details 
		Delete(jobDetails,childPid) ; 

		Print(jobDetails) ;

		// Remove pid from the Job table
		LocalHostJobs.DeleteProcessId(jobId,childPid);

		// If this is the last process of the job then delete the job entry also 
		if ( LocalHostJobs.GetActiveNumberOfProcesses( jobId ) == 0 ) 
			LocalHostJobs.DeleteJobId( jobId ) ;
		
		return 0 ; 
	}

	// This case is used when the job is killed. Killing of a job
	// will also issue a SIGCHLD and finds out that the jobId for that
	// process does not exist and return  
	return -1 ;

finish :

	cerr << "Error Sending Terminated Process Message " << childPid << endl ;

	if ( clientSock >= 0 )
		close(clientSock) ;

	Delete(jobDetails,childPid) ; 

	return -1 ; 
}
