#include <iostream.h>
#include <assert.h> 
#include <stdlib.h>
#include <String.h>

#include <netdb.h> 
#include "Constant.h"
#include "Functions.h"
#include "Message.h"

main(int argc,char* argv[])
{
	extern int 	optind ;
	extern char*	optarg ; 

	int		gspPort ; 
	char*		jobId = NULL ;
	char*		gspName = NULL ;
	char*		portNumber = NULL ;

	int		c ;

	while ( (c = getopt(argc,argv,"f:s:p:") ) != EOF )
	{
		switch(c)
		{
			case 'j' :
				jobId = optarg ;
				break ;
			case 's' :
				gspName = optarg ;
				break ;

			case 'p' :
				portNumber = optarg ;
				break ;

			case '?' :
				cerr << "Usage: " << argv[0] << 
				" -s <GSP HostName> [ -p <GSP Service Port Number> -j <job id> ]" << endl ;
				exit(1) ;	
		}
	}

	if ( !gspName )
	{
		cerr << "Usage: " << argv[0] << 
		" -s <GSP HostName> [ -p <GSP Service Port Number> -j <job id> ]" << endl ;

		exit(1) ;
	}

	// Check the validity of the host name
	struct hostent* hostEnt = OurGetHostEntry( gspName ) ;

	if ( !hostEnt )
	{
		cerr << "GSP Server " << gspName << " does not exist." << endl ;

		exit(1) ;
	}	

	// Check the validity of the port number  
	if ( ( gspPort = ValidatePortNumber( portNumber, GSP_PORT_NUMBER ) ) < 0 )
	{
		cerr << "GSP Port Number should be a valid integer" << endl ;

		exit(1) ;
	}

	// Get my host name
	char* hostName = GetHostName() ; 

	if ( hostName == NULL )
	{
		cerr << "Unable to get my host name." << endl ;

		exit(1) ;
	}

	char* ipAddress = OurGetHostAddress( hostName ) ;

	if ( ipAddress == NULL )
	{
		cerr << "Unable to get my ip Address." << endl ;

		exit(1) ;
	}

	int clientSock = CreateClientSocket( gspName, gspPort ) ;  

	if ( clientSock < 0 )
	{
		cerr << "Unable to connect to GSP "  << gspName << ". Quitting..." << endl ;

		exit(1) ;
	}

	MessHeader_t message(ipAddress) ; 

	String answer ;

	if ( jobId ) 
	{	
		// Query for a specific jobId
		message.ConstructMessage( MSG_SUSPEND, atoi(jobId) ) ;
	}
	else
	{
		// Query for all jobIds  
		message.ConstructMessage( MSG_SUSPEND, -1 ) ;
	}

	if ( message.Send( clientSock ) < 0 )
	{
		goto finish ;
	}

	// Wait for reply 
	if ( message.Receive( clientSock ) < 0 )
	{
		goto finish ; 
	}
		
	answer.alloc(message.msgLen+2) ;

	// Read the information directly from the socket and print it out
	if ( read( clientSock, answer.chars(), message.msgLen ) < 0 )  
	{
		cerr << "Unable to receive answer from GSP " << gspName << ". Quitting..."  << endl ;
		goto finish ; 
	}
					
	close(clientSock) ;

	cout << answer << endl ;

	exit(0) ;

finish :

	close(clientSock) ;

	exit(1) ;
}
