#include <iostream.h>
#include <assert.h> 
#include <stdlib.h>
#include <String.h>

#include <netdb.h> 
#include "Constant.h"
#include "Functions.h"
#include "Message.h"

main(int argc,char* argv[])
{
	extern int 	optind ;
	extern char*	optarg ; 

	int 		rbytes = 0 ; 
	int		rsofar = 0 ;
	int		lspPort ; 
	char*		jobId = NULL ;
	char*		queryHostName = NULL ;
	char*		portNumber = NULL ;

	int		c ;

	while ( (c = getopt(argc,argv,"j:s:p:") ) != EOF )
	{
		switch(c)
		{
			case 'j' :
				jobId = optarg ;
				break ;
			case 's' :
				queryHostName = optarg ;
				break ;

			case 'p' :
				portNumber = optarg ;
				break ;

			case '?' :
				cerr << "Usage: " << argv[0] << 
				" -s <HostName> [ -p <Query Port Number> -j <job id> ]" << endl ;
				exit(1) ;	
		}
	}

	if ( !queryHostName )
	{
		cerr << "Usage: " << argv[0] << 
		" -s <HostName> [ -p <Query Port Number> -j <job id> ]" << endl ;

		exit(1) ;
	}

	// Check the validity of the host name
	struct hostent* hostEnt = OurGetHostEntry( queryHostName ) ;

	if ( !hostEnt )
	{
		cerr << "Server " << queryHostName << " does not exist." << endl ;

		exit(1) ;
	}	

	// Check the validity of the port number  
	if ( ( lspPort = ValidatePortNumber( portNumber, LSP_PORT_NUMBER ) ) < 0 )
	{
		cerr << "Query Port Number should be a valid integer" << endl ;

		exit(1) ;
	}

	// Get my host name
	char* hostName = GetHostName() ; 

	if ( hostName == NULL )
	{
		cerr << "Unable to get my host name." << endl ;

		exit(1) ;
	}

	char* ipAddress = OurGetHostAddress( hostName ) ;

	if ( ipAddress == NULL )
	{
		cerr << "Unable to get my ip Address." << endl ;

		exit(1) ;
	}

	int clientSock = CreateClientSocket( queryHostName, lspPort ) ;  

	if ( clientSock < 0 )
	{
		cerr << "Unable to connect to "  << queryHostName << ". Quitting..." << endl ;

		exit(1) ;
	}

	MessHeader_t message(ipAddress) ; 

	char* answer ;

	if ( jobId ) 
	{	
		// Query for a specific jobId
		message.ConstructMessage( MSG_QUERY, atoi(jobId) ) ;
	}
	else
	{
		// Query for all jobIds  
		message.ConstructMessage( MSG_QUERY, -1 ) ;
	}

	if ( message.Send( clientSock ) < 0 )
	{
		goto finish ;
	}

	// Wait for reply 
	if ( message.Receive( clientSock ) < 0 )
	{
		goto finish ; 
	}
		
	answer = new char [message.msgLen+2] ;

	if ( answer == NULL )
	{
		cerr << "Unable to allocate memory for receiving reply from " 
		     << queryHostName << ". Quitting..."  << endl ;
		
		goto finish ; 
	}	

	bzero( answer, sizeof(char) * (message.msgLen + 2) ) ;  

	// Read the information directly from the socket and print it out

	while ( rsofar < message.msgLen )
	{
		if ( (rbytes = read( clientSock, answer+rsofar, message.msgLen-rsofar ))  < 0 )  
		{
			cerr << "Unable to receive answer from " << queryHostName 
			     << ". Quitting..."  << endl ;
			goto finish ; 
		}

		rsofar += rbytes ;
	}
					
	close(clientSock) ;

	cout << answer << endl ;

	cout.flush() ;

	exit(0) ;

finish :

	close(clientSock) ;

	exit(1) ;
}
