#if !defined (__JOB_H)
#define	__JOB_H

#include <stdio.h>
#include <String.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include "Constant.h"
#include "Functions.h"
#include "GSPEntry.h"
#include "List.h"

struct Machine_t ;

struct RunningJob_t ; 

const	int	JOB_RUNNING = 0 ;
const	int	JOB_SUSPEND = 1 ;

struct Job_t
{
	public :

		int  			jobId ;

		String 			jobConfigFile ;
	
		int  			numNodesRequired ;

		int  			currPriority ;

		RunningJob_t*		runEntryPtr ;

		long 			expCompletionTime ;

		int			jobStatus ;	// Either Running or suspended

		struct timeb 		jobArrivalTime ;

		struct timeb 		jobStartScheduleTime ;

		struct timeb 		jobFinishScheduleTime ;

		struct timeb 		jobEarliestCompletionTime ;

		struct timeb 		jobLatestCompletionTime ;
		
		oo_List<Entry_t>	entryList ; 

		Job_t(int id = 0 , int nodes = -1 ) : jobId(id), numNodesRequired(nodes)
		{
			currPriority = numNodesRequired ;

			bzero( (char *)&jobArrivalTime, sizeof(struct timeb) ) ;

			bzero( (char *)&jobStartScheduleTime, sizeof(struct timeb) ) ;

			bzero( (char *)&jobFinishScheduleTime, sizeof(struct timeb) ) ;

			bzero( (char *)&jobEarliestCompletionTime, sizeof(struct timeb) ) ;

			bzero( (char *)&jobLatestCompletionTime, sizeof(struct timeb) ) ;

			runEntryPtr = NULL ;

			jobStatus = JOB_RUNNING ; 
		} ;

		int
		Parse() ; 
	
		virtual
		~Job_t() ; 
			
} ;

struct ProcessInfo_t
{
	public :

		int			entryNum ;

		struct	timeb		compTime ;

		ProcessInfo_t(int num = 0) : entryNum(num)
		{
			bzero((char*)&compTime, sizeof(struct timeb)) ; 
		}	
} ;
				
struct RunningEntry_t
{
	public :

		Entry_t*		entryPtr ;

		int			startEntryNum ;

		int			endEntryNum ;

		oo_List<ProcessInfo_t>	processList ;

		RunningEntry_t(Entry_t* ePtr = NULL, 
				int startnum = 0 , 
				int endnum = 0 ) : 
			       entryPtr(ePtr),startEntryNum(startnum),
			       endEntryNum(endnum)
		{
		} ;

		ProcessInfo_t*
		Search( int entryNum )  ;

		int
		Insert( int entryNum, struct timeb& atime) ;

		int
		Delete( int entryNum ) ;

		void
		DeleteAll() ; 
			
		void
		Print() ;  
			
		virtual	
		~RunningEntry_t()	
		{
			DeleteAll() ;
		} ; 
} ;

struct MachInfo_t 
{
	public :
		
		Machine_t*		machPtr ;

		int			prsrAllocated ; 	

		// For reporting of data	
		int			initialPrsrAllocated ; 

		oo_List<RunningEntry_t>	entryList ;

		MachInfo_t(Machine_t* mPtr = NULL ) : machPtr(mPtr)
		{
			prsrAllocated = 0 ;
;
			initialPrsrAllocated  = 0 ; 
		}	

		// Given an entry number finds the entry in the entry list   
		// that contains the range which includes the entryNum.
		RunningEntry_t*
		Search ( int entryNum ) ; 

		void
		DeleteAll() ;

		void
		Print() ;  

		virtual
		~MachInfo_t()
		{
			DeleteAll() ;
		} ; 
} ;

struct RunningJob_t
{
	public :
		
		Job_t*				job ;

		int				numCompleted ;
		
		oo_List<MachInfo_t>		runningMachList ;		
				
		RunningJob_t( Job_t* j = NULL ) : job(j) 
		{
			numCompleted = 0 ;
		} 

		MachInfo_t*
		Search( Machine_t* mPtr ) ;  

		MachInfo_t*
		Search( const char* ipAddress ) ;  

		void
		Print() ; 

		void
		DeleteAll() ;

		virtual
		~RunningJob_t()
		{
			DeleteAll() ;
		} ;  
} ;

Job_t*
Search ( oo_List<Job_t>& , int jobId ) ;

void
Insert( oo_List<Job_t>& , Job_t * aJob ) ;

int
Delete( oo_List<Job_t>&, int jobId ) ;

void
DeleteAll( oo_List<Job_t>& ) ;

void
Print( oo_List<Job_t>& ) ;

void
Print( oo_List<RunningJob_t>& ) ;

void
Print( oo_List<MachInfo_t>& ) ;

int
Linearize( String&, oo_List<Job_t>&, Job_t* ) ;

int
Linearize( String&, oo_List<Job_t>& ) ;

#endif

