#if !defined(__MACHINE_LIST_H)
#define	__MACHINE_LIST_H

#include <sys/param.h>
#include <string.h>

#include "Functions.h"
#include "List.h"
#include "GSPJob.h"

enum NodeStatus_t {

	OPERATIONAL,
	NOT_AVAILABLE
} ;

struct JobInstances_t
{
	public :

		Job_t*		ptrToJob ;

		JobInstances_t( Job_t* aJob = NULL) : 
			ptrToJob(aJob)
		{
		} 
} ;

struct Machine_t 
{
	public :

		String			ipAddress ;  

		int			numProcessors ;

		int			numProcessorsFree ;

		NodeStatus_t		status ;	

		oo_List<JobInstances_t>	jobs ;

		Machine_t()	
		{
			numProcessors = numProcessorsFree = -1 ;

			status = NOT_AVAILABLE ;
		} 

		int 
		InsertJob( Job_t* aJob ) ;

		int
		DeleteJob ( int JobId ) ;

		int
		DeleteJob ( Job_t* aJob ) ;

		JobInstances_t*	
		SearchJob ( int JobId ) ; 

		int	
		SearchJob ( Job_t* aJob ) ; 

		void
		Print() ; 
} ;

struct HashTable_t
{
	public :

		HashTable_t()
		{
			lastAllocated = 0 ;

			processorsFree =  0 ;

			currMachine = NULL ; 			
				
		}

		Machine_t* 
		Search( const char* key ) ;

		int 	
		Insert( Machine_t* aMachine ) ;

		Machine_t*
		GetFreeMachine( int reqNumber, int& allocated ) ; 

		// Since this is a static hash table we might not need delete 

		Machine_t* 
		Delete( const char* key ) ;
				
		void
		Print() ; 
				
		int			processorsFree ;	

	private	 :		

		int 
		HashFunction( const char* key )
		{
			int value = 0 ;

			assert ( key != NULL ) ;

			while ( *key )
			{
				value += *key++ ;
			}

			return (value % Constant::MAX_HASHTABLELISTS) ;
		} ;

		oo_List<Machine_t>	collisionLists[Constant::MAX_HASHTABLELISTS] ; 

		// Queue for the free List 

		int			lastAllocated ;
		Machine_t*		currMachine ; 
} ; 

int
BuildMachineList( HashTable_t& , const char* ) ;

int
ActivateMachine( HashTable_t& , const char*, int ) ;
		
int
DeActivateMachine( HashTable_t& , const char*, int ) ;

#endif 
