#include <string.h>
#include <stdlib.h>
#include "Constant.h"

extern "C" {

void bzero( char *, int ) ;
void bcopy( char *, char *, int ) ;
int  bcmp ( char *, char *, int ) ;

}

#if !defined(__JOB_TABLE_H)
#define	__JOB_TABLE_H

class JobTable 
{ 
	public :

		// Constructors and Destructors
		JobTable() ;

		JobTable(const JobTable& aJobTable) ;

		virtual
		~JobTable() ;

		JobTable&
		operator =(const JobTable& aJobTable) ;

		int
		operator ==(const JobTable& aJobTable) ;

		// Selectors 
		int
		GetJobId(int ProcessId) ;	

		int
		Get( int Index ) ;

		int
		GetProcessId(int JobId,int* ProcessIds) ; 

		// Anabolizers 
		int
		InsertJobId(int JobId) ; 

		int
		InsertProcessId(int JobId, int ProcessId) ; 
		
		// Catabolizers	
		int
		DeleteJobId(int JobId) ; 

		int
		DeleteProcessId(int JobId, int ProcessId) ; 


		// Miscellaneous
		int
		GetActiveNumberOfJobs() ;

		int
		GetActiveNumberOfProcesses(int JobId) ;

		// Error Codes specific to instances of this
		// class

		static const int JOB_ID_NOT_FOUND 		= -1 ;  		
		static const int PROCESS_ID_NOT_FOUND		= -2 ; 
		static const int JOB_INDEX_OUT_OF_RANGE		= -3 ; 	 
		static const int PROCESS_INDEX_OUT_OF_RANGE	= -4 ; 	 
		static const int JOB_ID_ALREADY_EXISTS		= -5 ; 
		static const int PROCESS_ID_ALREADY_EXISTS	= -6 ; 
		static const int NO_SPACE_FOR_NEW_JOB		= -7 ; 
		static const int NO_SPACE_FOR_NEW_PROCESS	= -8 ; 

	protected :

		int
		JobIdTable[Constant::MAX_JOBS] ;

		int  
		ProcessIdTable[Constant::MAX_JOBS][Constant::MAX_PROCESSORS] ; 
} ;

inline 
JobTable::JobTable()
{
	bzero( (char *)JobIdTable, sizeof(int) * Constant::MAX_JOBS ) ;

	bzero( (char *)ProcessIdTable, sizeof(int) * 
					Constant::MAX_JOBS *
					Constant::MAX_PROCESSORS ) ; 
}

inline
JobTable::JobTable(const JobTable& aJobTable)
{
	bcopy( (char *)aJobTable.JobIdTable, (char *)JobIdTable, 
			sizeof(int) * Constant::MAX_JOBS ) ;  

	bcopy( (char *)aJobTable.ProcessIdTable, (char *)ProcessIdTable,
			sizeof(int) * 
			Constant::MAX_JOBS *
			Constant::MAX_PROCESSORS ) ; 
}

inline
JobTable::~JobTable()
{
}

inline 
JobTable&
JobTable::operator =(const JobTable& aJobTable)
{
	bcopy( (char *)aJobTable.JobIdTable, (char *)JobIdTable, 
			sizeof(int) * Constant::MAX_JOBS ) ;  

	bcopy( (char *)aJobTable.ProcessIdTable, (char *)ProcessIdTable,
			sizeof(int) * 
			Constant::MAX_JOBS *
			Constant::MAX_PROCESSORS ) ; 

	return *this ; 
}

inline 
int
JobTable::operator ==(const JobTable& aJobTable)
{
	int cmp_result1 = bcmp( (char *)aJobTable.JobIdTable, 
				(char *)JobIdTable, sizeof(JobIdTable) ) ; 

	int cmp_result2 = bcmp( (char *)aJobTable.ProcessIdTable, (char *)ProcessIdTable, 
				 sizeof(ProcessIdTable) ) ; 

	return cmp_result1 && cmp_result2 ;
}

			
#endif 
