#if !defined(__MESSAGE_H)
#define	__MESSAGE_H

#include <stdio.h>
#include <iostream.h> 
#include <stdlib.h>
#include <assert.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/timeb.h>

#include "Functions.h"

extern char* sys_errlist[] ;

// The configuration file can be used to designate a master/slave server
// relation ship so that if GSP dies then one of LSP can promote
// itself to be become GSP. 

enum MsgTypes_t 
{	
	MSG_NONE,
	MSG_SUBMIT,
	MSG_SUBMIT_REPLY,	
	MSG_SUBMIT_COMMIT,
	MSG_SUBMIT_ABORT,
	MSG_STARTUP,		// Used by LSP to inform GSP about its configuration
	MSG_QUERY,		
	MSG_QUERY_ANSWER,
	MSG_SCHEDULE,
	MSG_SUSPEND,
	MSG_SUSPEND_REPLY,
	MSG_RESUME,
	MSG_RESUME_REPLY,
	MSG_KILL,
	MSG_KILL_REPLY,
	MSG_TERMINATED,
	MSG_WITHDRAW,
} ;

class MessHeader_t
{
	public :

		MsgTypes_t	msgType ; 		
		int		msgLen ;	
		int		jobId ;			
		char		ipAddress[MAXHOSTNAMELEN] ; 
		int		entryNum ; 

		union 
		{
			// Used for MSG_STARTUP 
			int		numNodes ;		

			// Used for MSG_TERMINATED  
			struct timeb 	timeTakenToComplete ;	
		} ;

		void
		SetIPAddress( const char* ip )
		{
			assert( ip != NULL ) ;

			::strcpy(ipAddress,ip) ;
		}

		MessHeader_t(char* myIP) 
		{
			msgType = MSG_NONE ;

			msgLen = 0 ;

			jobId = -1 ; 

			entryNum = 0 ; 

			bzero( (char *)ipAddress, sizeof(ipAddress) ) ;

			// Initialize the ip address
			bcopy( (char *)myIP, (char *)ipAddress, sizeof(char) * ::strlen(myIP) ) ;  
		} ;

		void
		ConstructStartupMessage( int nodes )  
		{
			msgType = MSG_STARTUP ;

			msgLen = 0 ;

			jobId = -1 ;

			entryNum = 0 ; 
			
			numNodes = nodes ;
		}

		void
		ConstructWithdrawMessage( int nodes )  
		{
			msgType = MSG_WITHDRAW ;

			msgLen = 0 ;

			jobId = -1 ;

			entryNum = 0 ; 
			
			numNodes = nodes ;
		}

		void
		ConstructTerminateMessage( int jobid, int endnum, struct timeb* compTime )
		{
			msgType = MSG_TERMINATED ;

			msgLen = 0 ;

			jobId = jobid ; 

			entryNum = endnum ; 
			
			bcopy( (char *)compTime, 
			       (char *)&timeTakenToComplete, sizeof(timeb) ) ;  
		}

		void
		ConstructQueryAnswerMessage( int length )
		{
			msgType = MSG_QUERY_ANSWER ;

			msgLen = length ;

			jobId = -1 ;

			entryNum = 0 ; 

			numNodes = 0 ;
		}

		void
		ConstructMessage( MsgTypes_t type, int jid )
		{
			msgType = type ;

			msgLen = 0 ;

			jobId = jid ;  

			entryNum = 0 ; 

			numNodes = 0 ;
		}

		int
		Send( int sockD )
		{
			register int error = 0 ; 

			error = write( sockD, (char *)this, sizeof(MessHeader_t) ) ;

			return error ;
		}

		int
		Receive( int sockD )
		{
			register int error = 0 ;

			error = read( sockD, (char *)this,sizeof(MessHeader_t) ) ;

			return error ;
		}	

		friend ostream& 
		operator <<(ostream& os, const MessHeader_t& aMessHeader_t) ;
} ;

struct Schedule_t
{
	int numInstances ;

	char buffer[4096] ;  
} ;

struct GSPQuery_t
{
	int  numNodes ;			//  Number of nodes required for the job	
	char fileName[BUFSIZ] ;		// Job Configuration file name 
} ;

#endif
