/**********************************************************
 *	queue.h
 *
 *	o Defines a template for a queue
 *	
 */ 


# ifndef __QUEUE_H
# define __QUEUE_H

/************************************************************
 * Include Headers
 ************************************************************/

#include <iostream.h>
#include "list.h"

/*
 * Queue is implemented as a list. We use the concept of delegation
 * in order to carry out the operations of the queue.  
 * The queue operations are mapped as pass thro' calls to the underlying
 * instance of the list.
 *
 * Queue is single ended
*/

template <class QueueElementType>
class Queue
{
	public :

		// Constructors and Destructors

		// Default constructor 
		Queue() ;  

		// Convenience Constructor
		Queue(QueueElementType *pElement) ; 	

		// Destructors
		virtual
		~Queue() ;

		// Selectors
		int
		IsEmpty(void) ; 

		int
		Length(void) ; 

		// Anabolizers 

		Queue<QueueElementType> &
		Insert(QueueElementType* pNewElement) ;

		// Catabolizers

		QueueElementType* 
		Remove(void) ; 

	private :

		oo_List<QueueElementType>	m_List ; 
} ; 


template <class QueueElementType>
inline
Queue<QueueElementType>::Queue()
{
}

template <class QueueElementType>
inline
Queue<QueueElementType>::Queue(QueueElementType* pElement) : m_List(pElement)
{
}

/**************************************************************
 * Destructor destroys the underlying list. But not the data 
 * elements they are pointing to.
 */ 
template <class QueueElementType>
inline
Queue<QueueElementType>::~Queue()
{
	// Delete the whole list
	m_List.DeleteList() ; 
}

/**************************************************
 *  IsEmpty(): Checks if the queue is empty
 *  RETURNS:
 *     NON-ZERO : if the queue is empty
 *     ZERO if the queue is not empty
 **************************************************/

template <class QueueElementType>
inline
int 
Queue<QueueElementType>::IsEmpty(void)
{
	return m_List.IsEmpty() ;
}

/**************************************************
 *  Length()
 *    Finds the length of given list
 * RETURNS:
 *     an integer value
 **************************************************/
template <class QueueElementType>
inline
int
Queue<QueueElementType>::Length(void)
{
	return m_List.Length() ;
}

/**************************************************
 *  Insert():
 *   Inserts a new element into the queue 
 *      
 * RETURNS:
 *   the modified Queue object
 **************************************************/

template <class QueueElementType>
inline
Queue<QueueElementType> &
Queue<QueueElementType>::Insert(QueueElementType * pNewElement)
{
	// Insert the new element at the end of the list 
	// to simulate queue. 
	m_List.InsertAtEnd(pNewElement) ;

	return *this ;
}

/**************************************************
 *  Remove()
 *    Removes the first element in the queue 
 *    
 * RETURNS:
 *   NULL if the queue is empty  
 *  Ptr to the QueueElementType otherwise
 * 
 **************************************************/

template <class QueueElementType>
inline
QueueElementType *
Queue<QueueElementType>::Remove(void)
{
	// Get the first element from the underlying list  
	QueueElementType *pTmp = m_List.FirstElement() ;

	if ( pTmp)
	{
		// Delete the first element if the 
		// queue is not empty 

		m_List.StartTraversal() ;

		m_List.DeleteAndAdvance() ;
	}

	return pTmp ; 
}

#endif 
