#ifndef _stl_h
#define _stl_h

// Copyright 1994 Mike Stump, Niklas Hallqvist, Sean Corfield, Jay
//		  Huber, Dwight Wilson, James Gallagher, Klamer
//		  Schutte, Brad Daniels, Maurizio Vitale

// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without
// fee, provided that the above copyright notice appear in all copies
// and that author attributions below not be removed unless it's
// referent is also removed in its entirety, and that the author's
// name not be used in advertising or publicity pertaining to
// distribution of the software without specific, written prior
// permission.  The author makes no representations about the
// suitability of this software for any purpose.  It is provided "as
// is" without express or implied warranty.

// Some sections of this code may be public domain.  This copyright
// does not apply to sections marked `public domain'.

// Written by Mike Stump, Niklas Hallqvist, Sean Corfield, Jay Huber,
// James Gallagher, Klamer Schutte, Brad Daniels, Maurizio Vitale
// based upon the specification by Alexander Stepanov and Meng Lee of
// Hewlett Packard in ANSI X3J16/94-0095, to which we are indebted for
// the ideas and specification.

// Please send all bug fixes, ideas, and comments to Mike Stump
// <mrs@cygnus.com>.  See ftp.cygnus.com:pub/stl.ps.gz for
// documentation, and ftp.cygnus.com:pub/stl.h for latest copy.


// This code is ALPHA.  Some routines will be complete, and bug free.
// Some routines will be incomplete.  Some routines will have bugs.


// Key to Author Attributions
// mrs		Mike Stump <mrs@cygnus.com>
// niklas	Niklas Hallqvist <niklas@appli.se>
// sean		Sean Corfield <sean_corfield@prqa.co.uk>	ANSI C++
// brad		Brad Daniels <daniels@jabberwock.biles.com>	ANSI C++
// wilson	Dwight Wilson <wilson@blaze.cs.jhu.edu>
// ktm		Kit McCormick <ktm@netcom.com>
// schutte	Klamer Schutte <klamer@ph.tn.tudelft.nl>
// gafter	Neal Gafter <Neal.Gafter@Eng.Sun.COM>		ANSI C++
// huber	Jay Huber <huber@kazoo.cs.uiuc.edu>
// jimg		James Gallagher <jimg@dcz.gso.uri.edu>
// jody		Jody Winston <jody@sccsi.com>
// mcroberts	Shane McRoberts <mcroberts@miteksys.com>
// bsb		Bernd Brentrup <bsb@GOEDEL.UNI-MUENSTER.DE>
// mav		Maurizio Vitale <mav@cc.gatech.edu>
// hoberoi	Himanshu Oberoi <hoberoi@limerick.cbs.umn.edu>

#ifdef WANT_PRAGMA_INTERFACE
#pragma interface
#endif

#ifdef __DECCXX
#define __CONST_CONST_TEMPLATE_BUG
#endif

#ifndef __GNUG__
#define TEMPLATE_STATIC
#endif

#ifdef __BORLANDC__
#define max __hide_max
#define min __hide_min
#endif
#include <stdlib.h>
#ifdef __BORLANDC__
#undef max
#undef min
#endif

#ifdef __BORLANDC__
enum bool {false = 0, true = 1};
#endif

#include <limits.h>
#include <stddef.h>
#include <assert.h>
#include <new.h>

// Basic support routines not in Standard Library.

template <class InputIterator>
int _compare(InputIterator s1_start, InputIterator s1_end,
	     InputIterator s2_start, InputIterator s2_end) {
  while (1) {
    if (s1_start == s1_end)
      return s2_start == s2_end;
    if (s2_start == s2_end)
      return 0;
    if (*s1_start++ != *s2_start++)
      return 0;
  }
}


template <class BidirectionalIterator, class T, class Distance>
inline void
_reverse(BidirectionalIterator first, BidirectionalIterator last, T*, Distance*) {
  Distance n = 0;
  distance(first, last, n);
  --n;
  while (n > 0) {
    swap(*first++, *--last);
    n -= 2;
  }
}

// Where does this go?  (huber)

template <class InputIterator, class OutputIterator, class UnaryPredicate>
inline OutputIterator
copy_if(InputIterator first, InputIterator last,
        OutputIterator result, UnaryPredicate unary_pred) {
  while (first != last)
    if (unary_pred(*first))
      *result++ = *first++;
    else
      first++;
  return result;
}

// 5.1 Operators (mrs, complete)

template <class T>
inline bool
operator !=(const T& x, const T& y) {
  return !(x == y);
}

#ifndef __GNUG__
// This template can cause an internal compiler error with g++ 2.6.0
template <class T>
inline bool
operator >(const T& x, const T& y) {
  return y < x;
}
#endif

template <class T>
inline bool
operator <=(const T& x, const T& y) {
  return !(y < x);
}

template <class T>
inline bool
operator >=(const T& x, const T& y) {
  return !(x < y);
}

// 5.2 Tuples (mrs, complete)

// 5.2.1 Empty (mrs, complete)

struct empty {};

inline bool
operator ==( const empty&, const empty&) {
  return true;
}

inline bool
operator < (const empty&, const empty&) {
  return false;
}

// 5.2.2 Pair (mrs, complete, extra)

template <class T1, class T2>
struct pair {
  T1 first;
  T2 second;
  // typedef const T1 const_T1;
  pair(const T1& x, const T2& y) : first(x), second(y) {
  }
  // Provide this just in case.
  pair(const pair<T1, T2>& x) : first(x.first), second(x.second) {
  }
#ifdef __GNUG__
  // XXX this is to work around a bug in g++, as it instantiates a whole
  // class at a time, and some classes have pair<> x = T() in them.
  pair() /* : first(), second() */ {
  }
#endif
};

template <class T1, class T2>
inline bool
operator ==(const pair<T1, T2>& x, const pair<T1, T2>& y) {
  return x.first == y.first && x.second == y.second;
}

template <class T1, class T2>
inline bool
operator < (const pair<T1, T2>& x, const pair<T1, T2>& y) {
  return x.first < y.first || (!(y.first < x.first) && x.second < y.second);
}

template <class T1, class T2>
inline pair<T1, T2>
make_pair(const T1& x, const T2& y) {
  return pair<T1, T2>(x, y);
}

// 5.3 Restrictor (mrs, complete)

template <class T>
class restrictor {
  friend bool operator ==(const restrictor<T>& x, const restrictor<T>& y);
  friend bool operator < (const restrictor<T>& x, const restrictor<T>& y);
 public:
  restrictor(const T& x) : value(x) {
  }
 protected:
  T value;
};

template <class T>
inline bool
operator ==(const restrictor<T>& x, const restrictor<T>& y) {
  return x.value == y.value;
}

template <class T>
inline bool
operator < (const restrictor<T>& x, const restrictor<T>& y) {
  return x.value < y.value;
}

// 6.6 Iterator Tags (mrs, complete)

// 6.6.2 Library Defined primitives (mrs, complete)

struct input_iterator_tag : empty {
};

struct output_iterator_tag : empty {
};

struct forward_iterator_tag : empty {
};

struct bidirectional_iterator_tag : empty {
};

struct random_access_iterator_tag : empty {
};

template <class T, class Distance
#ifdef TEMPLATE_DEFAULT_ARG
= ptrdiff_t
#endif
>
struct input_iterator : empty {
  typedef ptrdiff_t Distance;
};

struct output_iterator : empty {
};

template <class T, class Distance
#ifdef TEMPLATE_DEFAULT_ARG
= ptrdiff_t
#endif
>
struct forward_iterator : empty {
  typedef ptrdiff_t Distance;
};

template <class T, class Distance
#ifdef TEMPLATE_DEFAULT_ARG
= ptrdiff_t
#endif
>
struct bidirectional_iterator : empty {
  typedef ptrdiff_t Distance;
};

template <class T, class Distance
#ifdef TEMPLATE_DEFAULT_ARG
= ptrdiff_t
#endif
>
struct random_access_iterator : empty {
  typedef ptrdiff_t Distance;
};

template <class T, class Distance>
inline input_iterator_tag
iterator_category(const input_iterator<T, Distance>&) {
  return input_iterator_tag();
}

inline output_iterator_tag
iterator_category(const output_iterator&) {
#if __BORLANDC__ == 0x0410
  // 0x410 is BC++ 3.1, 0x452 is BC++ 4.02, 0x450 is probably BC++ 4.00
  output_iterator_tag x;
  return x;
#else
  return output_iterator_tag();
#endif
}

template <class T, class Distance>
inline forward_iterator_tag
iterator_category(const forward_iterator<T, Distance>&) {
  return forward_iterator_tag();
}

template <class T, class Distance>
inline bidirectional_iterator_tag
iterator_category(const bidirectional_iterator<T, Distance>&) {
  return bidirectional_iterator_tag();
}

template <class T, class Distance>
inline random_access_iterator_tag
iterator_category(const random_access_iterator<T, Distance>&) {
  return random_access_iterator_tag();
}

template <class T>
inline random_access_iterator_tag
iterator_category(const T*) {
  return random_access_iterator_tag();
}

template <class T, class Distance>
inline T*
value_type(const input_iterator<T, Distance>&) {
  return (T*)0;
}

template <class T, class Distance>
inline T*
value_type(const forward_iterator<T, Distance>&) {
  return (T*)0;
}

template <class T, class Distance>
inline T*
value_type(const bidirectional_iterator<T, Distance>&) {
  return (T*)0;
}

template <class T, class Distance>
inline T*
value_type(const random_access_iterator<T, Distance>&) {
  return (T*)0;
}

template <class T>
inline T*
value_type(const T*) {
  return (T*)0;
}

#ifdef __BORLANDC__
template <class T>
inline T*
value_type(T*) {
  return (T*)0;
}

template <class T>
inline ptrdiff_t*
distance_type(T*) {
  return (ptrdiff_t*)0;
}
#endif

template <class T, class Distance>
inline Distance*
distance_type(const input_iterator<T, Distance>&) {
  return (Distance*)0;
}

template <class T, class Distance>
inline Distance*
distance_type(const forward_iterator<T, Distance>&) {
  return (Distance*)0;
}

template <class T, class Distance>
inline Distance*
distance_type(const bidirectional_iterator<T, Distance>&) {
  return (Distance*)0;
}

template <class T, class Distance>
inline Distance*
distance_type(const random_access_iterator<T, Distance>&) {
  return (Distance*)0;
}

template <class T>
inline ptrdiff_t*
distance_type(const T*) {
  return (ptrdiff_t*)0;
}

// 6.7 Iterator operations (mrs, complete)

template <class InputIterator, class Distance>
inline void
advance(InputIterator& i, Distance n) {
  __advance (i, n, iterator_category(i));
}

template <class InputIterator, class Distance>
inline void
__advance(InputIterator& i, Distance n, input_iterator_tag) {
  while (n > 0) {
    ++i;
    --n;
  }
}

template <class InputIterator, class Distance>
inline void
__advance(InputIterator& i, Distance n, forward_iterator_tag) {
  while (n > 0) {
    ++i;
    --n;
  }
}

template <class BidirectionalIterator, class Distance>
inline void
__advance(BidirectionalIterator& i, Distance n, bidirectional_iterator_tag) {
  while (n > 0) {
    ++i;
    --n;
  }
  while (n < 0) {
    --i;
    ++n;
  }
}

template <class RandomAccessIterator, class Distance>
inline void
__advance(RandomAccessIterator& i, Distance n, random_access_iterator_tag) {
  i += n;
}

template <class InputIterator, class Distance>
inline void
distance(InputIterator first, InputIterator last, Distance& n) {
  __distance(first, last, n, iterator_category(first));
}

template <class InputIterator, class Distance>
inline void
__distance(InputIterator first, InputIterator last, Distance& n,
	   input_iterator_tag) {
  while (first != last) {
    ++n;
    ++first;
  }
}

template <class InputIterator, class Distance>
inline void
__distance(InputIterator first, InputIterator last, Distance& n,
	   forward_iterator_tag) {
  while (first != last) {
    ++n;
    ++first;
  }
}

template <class InputIterator, class Distance>
inline void
__distance(InputIterator first, InputIterator last, Distance& n,
	   bidirectional_iterator_tag) {
  while (first != last) {
    ++n;
    ++first;
  }
}

template <class RandomAccessIterator, class Distance>
inline void
__distance(RandomAccessIterator first, RandomAccessIterator last,
	   Distance& n, random_access_iterator_tag) {
  n += last - first;
}

// 7 Functions Objects (mrs, complete)

// 7.1 Base (mrs, complete)

template <class Arg, class Result>
struct unary_function : empty {
  typedef Arg argument_type;
  typedef Result result_type;
};

template <class Arg1, class Arg2, class Result>
struct binary_function : empty {
  typedef Arg1 first_argument_type;
  typedef Arg2 second_argument_type;
  typedef Result result_type;
};

// 7.2 Arithmetic operations (mrs, complete)

template <class T>
struct plus : binary_function<T, T, T> {
  inline T operator () (const T& x, const T& y) const {
    return x + y;
  }
};

template <class T>
struct minus : binary_function<T, T, T> {
  inline T operator () (const T& x, const T& y) const {
    return x - y;
  }
};

template <class T>
struct times : binary_function<T, T, T> {
  inline T operator () (const T& x, const T& y) const {
    return x * y;
  }
};

template <class T>
struct divides : binary_function<T, T, T> {
  inline T operator () (const T& x, const T& y) const {
    return x / y;
  }
};

template <class T>
struct modulus : binary_function<T, T, T> {
  inline T operator () (const T& x, const T& y) const {
    return x % y;
  }
};

template <class T>
struct negate : unary_function<T, T> {
  inline T operator () (const T& x) const {
    return -x;
  }
};

// 7.3 Comparisons (mrs, complete)

template <class T>
struct equal_to : binary_function<T, T, bool> {
   inline bool operator () (const T& x, const T& y) const {
    return x == y;
  }
};

template <class T>
struct not_equal_to : binary_function<T, T, bool> {
   inline bool operator () (const T& x, const T& y) const {
    return x != y;
  }
};

template <class T>
struct greater : binary_function<T, T, bool> {
   inline bool operator () (const T& x, const T& y) const {
    return x > y;
  }
};

template <class T>
struct less : binary_function<T, T, bool> {
   inline bool operator () (const T& x, const T& y) const {
    return x < y;
  }
};

template <class T>
struct greater_equal : binary_function<T, T, bool> {
   inline bool operator () (const T& x, const T& y) const {
    return x >= y;
  }
};

template <class T>
struct less_equal : binary_function<T, T, bool> {
   inline bool operator () (const T& x, const T& y) const {
    return x <= y;
  }
};

// 7.4 Logical operations (mrs, complete)

template <class T>
struct logical_and : binary_function<T, T, bool> {
   inline bool operator () (const T& x, const T& y) const {
    return x && y;
  }
};

template <class T>
struct logical_or : binary_function<T, T, bool> {
   inline bool operator () (const T& x, const T& y) const {
    return x || y;
  }
};


template <class T>
struct logical_not : unary_function<T, bool> {
   inline bool operator () (const T& x) const {
    return !x;
  }
};

// 8 Allocators (mrs, complete)

// 8.2 The default Allocator (mrs, complete, machine dependant)

template <class T>
class allocator {
 public:
  typedef T* pointer;
  typedef const T* const_pointer;
  typedef T value_type;
  typedef size_t size_type;
  typedef ptrdiff_t difference_type;
  allocator() {}
  ~allocator() {}
  inline pointer allocate(size_type n) {
    return (pointer) operator new (n*sizeof(value_type));
  }
  inline void deallocate(pointer p) {
    operator delete (p);
  }
  /* XXX - this is machine dependent */
  inline size_t init_page_size() {
    int i = 1024/sizeof(value_type);
    if (i == 0)
      i = 1;
    return i;
  }
  /* XXX - this is machine dependent */
  inline size_t max_size() {
    return 1<<31;
  }    
};

// 9 Containers (incomplete)

// 9.1 Sequences (incomplete)

// 9.1.1 Vector (niklas, incomplete)

template <class T1>
inline void
__construct(T1* p) {
  new (p) T1();
}

template <class T,
#ifdef TEMPLATE_ARG
template <class U>
#endif
class Allocator
#ifdef TEMPLATE_DEFAULT_ARG
 = allocator
#endif
>
class vector {
 public:
  typedef Allocator::pointer iterator;
  typedef Allocator::const_pointer const_iterator;
  typedef Allocator::size_type size_type;
  typedef Allocator::difference_type difference_type;
  typedef T value_type;
  vector() : used(0), allocated(0), base(0) {}
#ifdef PEDANTIC
  vector(size_type n, const T& value = T())
    : used(n), allocated(max(mem.init_page_size(), n)) {
    base = mem.allocate(allocated);
    while (n--)
      construct(base + n, value);
  }
#else
  vector(size_type n, const T& value)
    : used(n), allocated(max(mem.init_page_size(), n)) {
    base = mem.allocate(allocated);
    while (n--)
      construct(base + n, value);
  }

  vector(size_type n)
    : used(n), allocated(max(mem.init_page_size(), n)) {
    base = mem.allocate(allocated);
    while (n--)
      __construct(base + n);
  }
#endif

  vector(const vector<T, Allocator>& x)
    : used(x.size()), allocated(max(mem.init_page_size(), x.size())) {
    base = mem.allocate(allocated);
    size_type n = used;
    while (n--)
      construct(base + n, x[n]);
  }

#ifdef TEMPLATE_MEMBER
  template <class InputIterator>
  vector(InputIterator first, InputIterator last);
  // XXX We have to dispatch different template instantiations depending
  // XXX on we're dealing with plain InputIterators or ForwardIterators
  // XXX and better.  I just don't know how, as G++ doesn't even support
  // XXX these templated members yet.
#endif  
  ~vector() {
    size_type n = used;
    while (n--)
      destroy(base + n);
    if (base)
      mem.deallocate(base);
  }
  vector<T, Allocator>& operator =(const vector<T, Allocator>& x) {
    if (this == &x)
      return *this;
    size_type n = used;
    if ((used = x.size()) > allocated) {
      while (n--)
	destroy(base + n);
      if (base)
	mem.deallocate(base);
      allocated = max(allocated ? allocated * 2 : mem.init_page_size(), used);
      base = mem.allocate(allocated);
      n = used;
      while (n--)
	construct(base + n, x[n]);
    } else {
      if (used <= n) {
	while (n > used)
	  destroy(base + --n);
	while (n--)
	  base[n] = x[n];
      } else {
	for (size_type i = n; i < used; i++)
	  construct(base + i, x[i]);
	while (n--)
	  base[n] = x[n];
      }
    }
    return *this;
  }
  void reserve(size_type n) {
    if (n > allocated) {
      allocated = max(allocated ? allocated * 2 : mem.init_page_size(), n);
      Allocator::pointer p = mem.allocate (allocated);
      n = used;
      while (n--) {
        construct(p + n, base[n]);
        destroy(base + n);
      }
      if (base)
        mem.deallocate(base);
      base = p;
    }
  }

  void swap(vector<T, Allocator>& x) {
    ::swap(base, x.base);
    ::swap(used, x.used);
    ::swap(allocated, x.allocated);
  }

  iterator begin() { return base; }
  const_iterator begin() const { return base; }
  iterator end() { return base + used; }
  const_iterator end() const { return base + used; }
  size_type size() const { return used; }
  // XXX why is Allocator::max_size not const?
  size_type max_size() const { return ((Allocator)mem).max_size(); }
  size_type capacity() const { return allocated; }
  bool empty() const { return used == 0; }
  T& operator [] (size_type n) {
    if (n < used)
      return base[n];
    if (n >= allocated)
      reserve(max(n + 1, allocated * 2));
    size_type i = used;
    while (i <= n)
      __construct(base + i++);
    used = n + 1;
    return base[n];
  }

  // XXX What should be done if N exceeds size()?  We could extend the vector
  // XXX or return a static instance of T every time.  In fact what one prefer
  // XXX is probably depending on the type of semantics T adhere to, value or
  // XXX reference.  One special problem is that &v[n] won't equal &v[0] + n,
  // XXX when n is larger than size() if we use the static return value.
  // That's why we currently use the more expensive extension of vector.
  const T& operator [] (size_type n) const {
    return (*(vector<T, Allocator>*)this)[n];
  }

  T& front() { return *base; }
  const T& front() const { return *base; }
  T& back() { return (*this)[used]; }
  const T& back() const { return (*this)[used]; }
  void push_back(const T& x) { insert(base + used, 1, x); }

#ifdef PEDANTIC
  iterator insert(iterator position, const T& x = T()) {
    size_type pos = position - base;
    insert (position, 1, x);
    return base + pos;
  }

  void insert(iterator position, size_type n, const T& x = T()) {
    size_type pos = position - base;
    size_type new_used = used + n;
    register size_type i;
    if (new_used > allocated) {
      allocated
	= max(allocated ? allocated * 2 : mem.init_page_size(), new_used);
      Allocator::pointer new_base = mem.allocate(allocated);
      i = 0;
      while (i < pos) {
        construct(new_base + i, base[i]);
        destroy(base + i++);
      }
      i = pos + n;
      while (i-- > pos)
        construct(new_base + i, x);
      i = pos + n;
      while (i < new_used) {
        construct(new_base + i, base[i - n]);
        destroy(base + i++);
      }
      if (base)
        mem.deallocate(base);
    } else {
      i = new_used;
      while (i > used) {
	register bool f = i-- > pos + n;
        construct(base + i, f ? base[i - n] : x);
      }
      while (i > pos) {
	register bool f = i-- > pos + n;
        base[i] = f ? base[i - n] : x;
      }
    }
    used = new_used;
  }
#else
  iterator insert(iterator position, const T& x) {
    size_type pos = position - base;
    insert (position, 1, x);
    return base + pos;
  }

  iterator insert(iterator position) {
    size_type pos = position - base;
    insert (position, 1);
    return base + pos;
  }

  void insert(iterator position, size_type n, const T& x) {
    size_type pos = position - base;
    size_type new_used = used + n;
    register size_type i;
    if (new_used > allocated) {
      allocated
	= max(allocated ? allocated * 2 : mem.init_page_size(), new_used);
      Allocator::pointer new_base = mem.allocate(allocated);
      i = 0;
      while (i < pos) {
        construct(new_base + i, base[i]);
        destroy(base + i++);
      }
      i = pos + n;
      while (i-- > pos)
        construct(new_base + i, x);
      i = pos + n;
      while (i < new_used) {
        construct(new_base + i, base[i - n]);
        destroy(base + i++);
      }
      if (base)
        mem.deallocate(base);
    } else {
      i = new_used;
      while (i > used) {
	register bool f = i-- > pos + n;
        construct(base + i, f ? base[i - n] : x);
      }
      while (i > pos) {
	register bool f = i-- > pos + n;
        base[i] = f ? base[i - n] : x;
      }
    }
    used = new_used;
  }

  void insert(iterator position, size_type n) {
    size_type pos = position - base;
    size_type new_used = used + n;
    register size_type i;
    if (new_used > allocated) {
      allocated
	= max(allocated ? allocated * 2 : mem.init_page_size(), new_used);
      Allocator::pointer new_base = mem.allocate(allocated);
      i = 0;
      while (i < pos) {
        construct(new_base + i, base[i]);
        destroy(base + i++);
      }
      i = pos + n;
      while (i-- > pos)
        __construct(new_base + i);
      i = pos + n;
      while (i < new_used) {
        construct(new_base + i, base[i - n]);
        destroy(base + i++);
      }
      if (base)
        mem.deallocate(base);
    } else {
      i = new_used;
      while (i > used)
	if (i-- > pos + n)
	  construct(base + i, base[i - n]);
        else
	  __construct(base + i);
      while (i > pos) {
#ifndef TEMPLATE_STATIC
	static T def;
#endif
        register bool f = i-- > pos + n;
        base[i] = f ? base[i - n] : def;
      }
    }
    used = new_used;
  }
#endif

#ifdef TEMPLATE_MEMBER
  template <class InputIterator>
  void insert(iterator position, InputIterator first, InputIterator last);
  // XXX I'm holding this off since G++ does not support it yet.
#endif
  void pop_back() { if (used) destroy(base + --used); }
  void erase(iterator position) {
    iterator end = base + --used;
    while (++position < end)
      *(position - 1) = *position;
    destroy(position);
  }

  void erase(iterator first, iterator last) {
    iterator end = base + used;
    while (last != end)
      *first++ = *last++;
    while (first != end) {
      destroy(first++);
      used--;
    }
  }

private:			// XXX Should we use protected?
  Allocator mem;
  Allocator::pointer base;
  size_type used;
  size_type allocated;
#if !defined(PEDANTIC) && defined(TEMPLATE_STATIC)
  static T def;
#endif
};

#if !defined(PEDANTIC) && defined(TEMPLATE_STATIC)
template <class T, class Allocator> static T vector<T, Allocator>::def;
#endif

template <class T, class Allocator>
bool
operator ==(const vector<T, Allocator>& x, const vector<T, Allocator>& y) {
  vector<T, Allocator>::size_type sz = x.size();
  if (sz != y.size())
    return 0;
  while (sz--)
    if (x[sz] != y[sz])
      return 0;
  return 1;
}

template <class T, class Allocator>
bool
operator <(const vector<T, Allocator>& x, const vector<T, Allocator>& y) {
  vector<T, Allocator>::size_type x_sz = x.size();
  vector<T, Allocator>::size_type y_sz = x.size();
  vector<T, Allocator>::size_type i = 0;
  while (i < x_sz && i < y_sz) {
    if (x[i] < y[i])
      return 1;
    if (x[i] > y[i])
      return 0;
    i++;
  }
  return x_sz < y_sz;
}

#if 0
class vector<bool, allocator> {
};

bool
operator ==(vector<bool, allocator>& x, vector<bool, allocator>& y);

bool
operator <(vector<bool, allocator>& x, vector<bool, allocator>& y);
#endif

// 9.1.2 List (mrs, (niklas) incomplete)

template <class T,
#ifdef TEMPLATE_ARG
template <class U>
#endif
class Allocator
#ifdef TEMPLATE_DEFAULT_ARG
 = allocator
#endif
>
class list {
#ifdef TEMPLATE_ARG
  struct Element_pair {
    Allocator<Element_pair>::pointer next, previous;
    T data;
  public:
    Element_pair(const T& e) : data(e) { }
  };
  Allocator<Element_pair>::pointer first_, last_;
#else
  struct Element_pair {
    Element_pair *next, *previous;
    T data;
  public:
    Element_pair(const T& e) : data(e) { }
  } *first_, *last_;
#endif
public:
  class iterator : public bidirectional_iterator<T, Allocator::difference_type> {
    friend list<T, Allocator>;
#ifdef TEMPLATE_ARG
    Allocator<Element_pair>::pointer current;
#else
    Element_pair *current;
#endif
  public:
    iterator(const iterator& o) {
      current = o.current;
    }
#ifdef TEMPLATE_ARG
    iterator(Allocator<Element_pair>::pointer start = 0) {
#else
    iterator(Element_pair *start = 0) {
#endif
      current = start;
    }      
    iterator& operator ++() {
      current = current->next;
      return *this;
    }
    iterator operator ++(int) {
      iterator i = *this;
      current = current->next;
      return i;
    }
    iterator& operator --() {
      if (current == 0) {
	assert(0); // XXX we need to be able to backup from the end.
      }
      current = current->previous;
      return *this;
    }
    iterator operator --(int) {
      iterator i = *this;
      if (current == 0) {
	assert(0); // XXX we need to be able to backup from the end.
      }
      current = current->previous;
      return i;
    }
    int operator ==(iterator o) const {
      return current == o.current;
    }
    int operator !=(iterator o) const {
      return current != o.current;
    }
    T& operator *() {
      return current->data;
    }
    const T& operator *() const {
      return current->data;
    }
  };
  class const_iterator : public bidirectional_iterator<T, Allocator::difference_type> {
    friend list<T, Allocator>;
#ifdef TEMPLATE_ARG
    Allocator<Element_pair>::pointer current;
#else
    Element_pair *current;
#endif
  public:
    const_iterator(const const_iterator& o) {
      current = o.current;
    }
#ifdef TEMPLATE_ARG
    const_iterator(Allocator<Element_pair>::pointer start = 0) {
#else
    const_iterator(Element_pair *start = 0) {
#endif
      current = start;
    }      
    const_iterator& operator ++() {
      current = current->next;
      return *this;
    }
    const_iterator operator ++(int) {
      const_iterator i = *this;
      current = current->next;
      return i;
    }
    const_iterator& operator --() {
      if (current == 0) {
	assert(0); // XXX we need to be able to backup from the end.
      }
      current = current->previous;
      return *this;
    }
    const_iterator operator --(int) {
      const_iterator i = *this;
      if (current == 0) {
	assert(0); // XXX we need to be able to backup from the end.
      }
      current = current->previous;
      return i;
    }
    int operator ==(const_iterator o) const {
      return current == o.current;
    }
    int operator !=(const_iterator o) const {
      return current != o.current;
    }
    const T& operator *() const {
      return current->data;
    }
  };

#ifdef TEMPLATE_ARG
  typedef T value_type;
  typedef Allocator<value_type>::size_type size_type;
  typedef Allocator<value_type>::difference_type difference_type;
#else
  typedef T value_type;
  typedef Allocator::size_type size_type;
  typedef Allocator::difference_type difference_type;
#endif

  list() {
    first_ = 0;
    last_ = 0;
  }
  list(size_type n, const T& value = T()) {
    first_ = 0;
    last_ = 0;
    while (n-- != 0)
      push_front(value);
  }
#ifdef TEMPLATE_MEMBER
  template <class InputIterator>
  list(InputIterator first, InputIterator last);
#endif
  list(const list<T, Allocator>& x) {
    first_ = 0;
    last_ = 0;
    const_iterator i = x.begin();
    const_iterator e = x.end();
    while (i != e)
      push_back(*i++);
  }    
  ~list() {
    while (! empty())
      pop_front();
  }
  list<T, Allocator>& operator=(const list<T, Allocator>& x) {
    while (! empty())
      pop_front();
    first_ = 0;
    last_ = 0;
    const_iterator i = x.begin();
    const_iterator e = x.end();
    while (i != e)
      push_back(*i++);
  }

  iterator begin() {
    return iterator(first_);
  }
  const_iterator begin() const {
    return const_iterator(first_);
  }
  iterator end() {
    return iterator();
  }
  const_iterator end() const {
    return const_iterator();
  }
  bool empty() const {
    return first_ == 0;
  }
  size_type size() const;
  size_type max_size() const;
  T& front() {
    return first_->data;
  }
  const T& front() const {
    return first_->data;
  }
  T& back() {
    return last_->data;
  }
  const T& back() const {
    return last_->data;
  }

  void push_front(const T& x) {
#ifdef TEMPLATE_ARG
    Allocator<Element_pair>::pointer tmp = first_;
#else
    Element_pair *tmp = first_;
#endif
    first_ = new Element_pair(x);
    first_->previous = 0;
    first_->next = tmp;
    if (tmp)
      tmp->previous = first_;
    if (last_ == 0)
      last_ = first_;
  }
  void push_back(const T& x) {
#ifdef TEMPLATE_ARG
    Allocator<Element_pair>::pointer tmp = last_;
#else
    Element_pair *tmp = last_;
#endif
    last_ = new Element_pair(x);
    last_->next = 0;
    last_->previous = tmp;
    if (tmp)
      tmp->next = last_;
    if (first_ == 0)
      first_ = last_;
  }
  iterator insert(iterator position, const T& x = T()) {
    iterator i = position;
#ifdef TEMPLATE_ARG
    Allocator<Element_pair>::pointer tmp = i.current;
#else
    Element_pair *tmp = i.current;
#endif
    i.current = new Element_pair(x);
    i.current->next = tmp;
    if (tmp)
      i.current->previous = tmp->previous;
    else
      i.current->previous = last_;
    if (i.current->previous)
      i.current->previous->next = i.current;
    else
      first_ = i.current;
    if (tmp)
      tmp->previous = i.current;
    else
      last_ = i.current;
    i.current = i.current->next;
  }
  void insert(iterator position, size_type n, const T& x = T()) {
    while (n--)
      insert(position, x);
  }
#ifdef TEMPLATE_MEMBER
  template <class InputIterator>
  void insert(iterator position, InputIterator first, InputIterator last);
#endif
  void pop_front() {
    erase(begin());
  }
  void pop_back() {
    erase(--end());
  }
  void erase(iterator position) {
    iterator i = position;
#ifdef TEMPLATE_ARG
    Allocator<Element_pair>::pointer tmp = i.current;
#else
    Element_pair* tmp = i.current;
#endif
    i.current = i.current->next;
    if (i.current)
      i.current->previous = tmp->previous;
    else
      last_ = tmp->previous;
    if (tmp->previous)
      tmp->previous->next = i.current;
    else
      first_ = i.current;
    delete tmp;
  }
  void erase(iterator first, iterator last) {
    while (first != last) {
      iterator next = ++first;
      erase(first);
      first = next;
    }
  }

  void splice(iterator position, list<T, Allocator>& x);
  void splice(iterator position, list<T, Allocator>& x, iterator i);
  void splice(iterator position, list<T, Allocator>& x, iterator first,
	      iterator last);
  void remove(const T& value);
#ifdef TEMPLATE_MEMBER
  template <class Predicate> void remove_if(Predicate pred);
#endif
  void unique();
#ifdef TEMPLATE_MEMBER
  template <class BinaryPredicate> void unique(BinaryPredicate binary_pred);
#endif
  void merge(list<T, Allocator>& x) {
    iterator first2 = x.begin();
    iterator last2 = x.end();
    iterator first1 = begin();
    iterator last1 = end();
    while (first2 != last2) {
      while (first1 != last1 && *first1 <= *first2)
	++first1;
      insert(first1, *first2++);
    }
  }
#ifdef TEMPLATE_MEMBER
  template <class Compare> void merge(list<T, Allocator>& x, Compare comp);
#endif
  void reverse()
#ifdef __GNUG__
  // There is a bug in g++ that prevents things from compiling, if this
  // is given.
    ;
#else
  {
    _reverse(begin(), end(), value_type(begin()), distance_type(begin()));
  }
#endif
  void sort();
#ifdef TEMPLATE_MEMBER
  template <class Compare> void sort(Compare comp);
#endif
};

template <class T, class Allocator>
bool operator==(const list<T, Allocator>& x, const list<T, Allocator>& y) {
  return _compare(x.begin(), x.end(), y.begin(), y.end());
}

template <class T, class Allocator>
bool operator<(const list<T, Allocator>& x, const list<T, Allocator>& y);

// 9.1.3 Deque (ktm, incomplete)

// 9.2 Associative containers (schutte, incomplete, mostly done, buggy)

// __mapset is a base class for set, multiset, map and multimap. 
// A base class is used to prevent (source code) multiplication. 
// More source code can be saved if multiset and multimap also get
// a common base. Or set and multiset.
// The implementation of the multi* parts is very simple: just duplicate
// the nodes. As a consequence are lower_bound and upper_bound not as
// efficient as needed (O(Log(N) + count(K)) instead of O(Log(N))).
// __RBtree is a base class of mapset.
// For each Allocator used only one instantiation of an __RBtree is needed.
// This will prevent a lot of object code multiplication! (If the compiler
// does not inline all functions such as g++ does ;-)
// It seems that this should be implemented with a binary tree. 
// The tree is a Red Black tree, see also Introduction to Algorithms,
// Corman et al, MIT Press, 1990. Main difference: a double linked list
// with successors is also kept in the nodes.
// All functions can be shared, expect:
//  -- constructors (they are not inherited)
// Overhead by this code sharing:
//  -- If no function inlining: function call overhead
// Possible incompatibilities:
//  -- set::erase takes __mapset::iterator as argument rather then set::iterator
//     However, a cast operator is present to fix things.

// Also should the leaves of the binary tree have links such that a
// bidirectional iterator can be used. This is done using links to the
// next element given the ordening.
// This means that we have both a linear and a binary list.

// Certainly missing: const_iterator stuff
// All the associative containers are very poorly tested. Watch out! the
// code pushes current compilers (g++ 2.6.0, cfront 3 fails) to the limits!

template <
#ifdef TEMPLATE_ARG
template <class U>
#endif
class Allocator
#ifdef TEMPLATE_DEFAULT_ARG
 = allocator
#ifndef TEMPLATE_ARG
 <int>
#endif
#endif
>
class __RBtree {
protected:
  enum Color { Red, Black };
  enum Direction { Smaller, Larger };
  enum Direction anti( enum Direction dir ) { 
    return (dir == Smaller) ? Larger : Smaller;
  }
  class __RBnode { 
    friend class __RBtree;
  public:
    __RBnode *_parent, *children[2], *_prev, *_next;
    Color color;
    __RBnode(__RBnode &nil, __RBnode *par, __RBnode *p = 0, __RBnode *n = 0)
      : color(Red), _parent(par), _prev((p==0)?&nil:p), _next((n==0)?&nil:n) {
      	_smaller() = _larger() = &nil; 
      }
    __RBnode *&_smaller() { return children[Smaller]; }
    __RBnode *&_larger() { return children[Larger]; }			      
    __RBnode *_smaller() const { return children[Smaller]; }
    __RBnode *_larger() const { return children[Larger]; }			      
  } *nil, *_root;
   
  __RBnode *&_first() { return nil->_next; }
  __RBnode *&_last() { return nil->_prev; }
  __RBnode *_first() const { return nil->_next; }
  __RBnode *_last() const { return nil->_prev; }
  
  // Give 0 as you want nil is not yet known!
  __RBtree( __RBnode *root, __RBnode *first, __RBnode *last ) 
    : nil(new __RBnode(*nil,0)), _root((root == 0) ? nil : root ) {
      nil->color = Black;
      nil->_parent = nil->_smaller() = nil->_larger() = nil->_prev = 
	             nil->_next =nil;
    }
  __RBtree() : nil(new __RBnode(*nil,0)) {
      nil->color = Black;
      nil->_parent = nil->_smaller() = nil->_larger() = nil->_prev = 
	             nil->_next =nil;
    }
  ~__RBtree() { delete nil; }
		   
  void Rotate( __RBnode *x, enum Direction dir ) {
    __RBnode *y = x->children[anti(dir)];
    x->children[anti(dir)] = y->children[dir];
    if (y->children[dir] != nil)
      y->children[dir]->_parent = x;
    y->_parent = x->_parent;
    if (x->_parent == nil)
      _root = y;
    else if (x == x->_parent->_smaller())
      x->_parent->_smaller() = y;
    else
      x->_parent->_larger() = y;
    y->children[dir] = x;
    x->_parent = y;
  }
  void RBinsert( __RBnode *x ) {
    while((x != _root) && (x->_parent->color == Red)) {
      enum Direction dir = (x->_parent == x->_parent->_parent->_smaller()) 
			     ? Smaller : Larger;
      __RBnode *y = x->_parent->_parent->children[anti(dir)];
      if (y->color == Red) {
	x->_parent->color = Black;
	y->color = Black;
	x->_parent->_parent->color = Red;
	x = x->_parent->_parent;
      } else {
	if (x == x->_parent->children[anti(dir)]) {
	  x = x->_parent;
	  Rotate(x,dir);
	}
	x->_parent->color = Black;
	x->_parent->_parent->color = Red;
	Rotate(x->_parent->_parent,anti(dir));
      }
    }
    _root->color = Black;
  }
  void RBdelete_fixup( __RBnode *x ) {
    while ((x != _root) && (x->color == Black)) {
      enum Direction dir = (x == x->_parent->_smaller()) ? Smaller : Larger;
      __RBnode *w = x->_parent->children[anti(dir)];
      if (w->color == Red) {
	w->color = Black;
	x->_parent->color = Red;
	Rotate(x->_parent,dir);
	w = x->_parent->children[anti(dir)];
      }
      if ((w->_smaller()->color == Black) && (w->_larger()->color == Black)) {
	w->color = Red;
	x = x->_parent;
      } else {
	if (w->children[anti(dir)]->color == Black) {
	  w->children[dir]->color = Black;
	  w->color = Red;
	  Rotate(w,anti(dir));
	  w = x->_parent->children[anti(dir)];
	}
	w->color = x->_parent->color;
	x->_parent->color = Black;
	w->children[anti(dir)]->color =Black;
	Rotate(x->_parent,dir);
	x = _root;
      }
    }	
    x->color = Black;
  }
  // Erase a node: See Introduction to Algorithms, page 273
  // Main changes: - support for prev()/next() nodes
  //               - do not copy key, to keep valid iterators.
  void RBdelete( __RBnode *z ) {
    __RBnode *x, *y;

    // Handle prev() / next() links
    assert(z->_prev->_next == z);
    z->_prev->_next = z->_next;
    assert(z->_next->_prev == z);
    z->_next->_prev = z->_prev;
    
    if ((z->_smaller() == nil) || (z->_larger() == nil)) 
      y = z;
    else {
      y = z->_next;
      assert(y != nil);
    }
    if (y->_smaller() != nil)
      x = y->_smaller();
    else
      x = y->_larger();
    x->_parent = y->_parent;
    if (y->_parent == nil)
      _root = x;
    else if (y == y->_parent->_smaller())
      y->_parent->_smaller() = x;
    else
      y->_parent->_larger() = x;

    if (y != z) { 
      // Move y to z. No copy of value only, as that invalidates iterators.
      // prev() / next() links already are OK!
      ::swap(y->_larger(),z->_larger());
      ::swap(y->_smaller(),z->_smaller());
      ::swap(y->_parent,z->_parent);
      ::swap(y->color,z->color);
      if (y->_larger() != nil)
	y->_larger()->_parent = y;
      if (y->_smaller() != nil)
	y->_smaller()->_parent = y;
      if (y->_parent != nil)	// Can this ever be zero?
	if (y->_parent->_smaller() == z)
	  y->_parent->_smaller() = y;
        else {
	  assert(y->_parent->_larger() == z);
	  y->_parent->_larger() = y;
        }
    }
    if (z->color == Black)
      RBdelete_fixup(x);
  }
};
 
template <class Key, class T, class Compare
#ifdef TEMPLATE_DEFAULT_ARG
= less<Key>
#endif
,
#ifdef TEMPLATE_ARG
template <class U>
#endif
class Allocator
#ifdef TEMPLATE_DEFAULT_ARG
 = allocator
#ifndef TEMPLATE_ARG

 <int>
#endif
#endif
,bool multi>
class __mapset : public __RBtree<Allocator> {
public:
  // typedefs:
  typedef Key key_type;
  typedef Compare key_compare;
#ifdef TEMPLATE_ARG
  typedef Allocator<Key>::size_type size_type;
  typedef Allocator<Key>::difference_type difference_type;
#else
  typedef Allocator::size_type size_type;
  typedef Allocator::difference_type difference_type;
#endif
  // value_type should be typedef'ed to pair<const Key, T>, but I can't
  // get g++ into accepting such usage (schutte)
  typedef pair<Key, T> value_type;
  class value_compare : public binary_function<value_type, value_type, bool> {
    friend class map;
protected:
    Compare comp;
    value_compare(Compare c) : comp(c) { }
public:
    bool operator()(const value_type &x, const value_type &y) {
      return comp(x.first, y.first);
    }
  };

protected:
  struct mapsetNode : public __RBnode{
    mapsetNode *parent() const { return (mapsetNode *)_parent; }
    mapsetNode *prev() const { return (mapsetNode *)_prev; }
    mapsetNode *next() const { return (mapsetNode *)_next; }
    mapsetNode *smaller() const { return (mapsetNode *)children[Smaller]; }
    mapsetNode *larger() const { return (mapsetNode *)children[Larger]; }
    mapsetNode	*&parent() { return *(mapsetNode **)(void**)&_parent; }
    mapsetNode	*&prev() { return *(mapsetNode **)(void**)&_prev; }
    mapsetNode	*&next() { return *(mapsetNode **)(void**)&_next; }
    mapsetNode	*&smaller() { return *(mapsetNode **)(void**)&children[Smaller]; }
    mapsetNode	*&larger() { return *(mapsetNode **)(void**)&children[Larger]; }
    value_type v;
    mapsetNode(const value_type &val, __RBnode &nil, mapsetNode *par, 
	       mapsetNode *p = 0, mapsetNode *n = 0)
      : v(val), __RBnode(nil,par,p,n) { }
  };  
    void recursive_copy(__RBnode &oldnil, const mapsetNode *old, 
			       mapsetNode *& root,
			       mapsetNode *& first, mapsetNode *& last,
			       mapsetNode *par) {
      // Copy constructor for __mapset
      // This is simple minded, but working implementation
      // Knuth says that you can do it also without recursion.
      // This left as a job for volunteers.

      root = new mapsetNode( old->v, *nil, par);
      root->color = old->color;
      if (old->smaller() != &oldnil) {
	recursive_copy( oldnil, old->smaller(), root->smaller(), first, root->prev(), root);
	root->prev()->next() = root;
      } else
	first = root;
      if (old->larger() != &oldnil) {
	recursive_copy( oldnil, old->larger(), root->larger(), root->next(), last, root);
	root->next()->prev() = root;
      } else
	last = root;
    }
  Compare _comp;
  size_type _size;
  mapsetNode *root() const { return (mapsetNode *)_root; }
  mapsetNode *first() const { return (mapsetNode *)_first(); }
  mapsetNode *last() const { return (mapsetNode *)_last(); }
  mapsetNode *&root() { return *(mapsetNode **)(void**)&_root; }
  mapsetNode *&first() { return *(mapsetNode **)(void**)&_first(); }
  mapsetNode *&last() { return *(mapsetNode **)(void**)&_last(); }
  mapsetNode *nilp() const { return (mapsetNode *)nil; }

public:
  class iterator : public bidirectional_iterator<Key,ptrdiff_t> {
    friend class __mapset;

    mapsetNode *ptr; //, *prev, *next;	// have prev() and next() for end and start

  public:
    value_type &operator*() { return ptr->v; }
    const value_type &operator*() const { return ptr->v; }
    iterator(mapsetNode *n) : ptr(n) { } // , prev(0), next(0) { }
    //iterator(mapsetNode *c) //, mapsetNode *p, mapsetNode *n)
    //  : ptr(c) { } //, prev(p), next(n) { }
    iterator() {}
    iterator operator++() {
      //prev = ptr;
      //if (ptr == 0)
	//ptr = next;
      //else
	ptr = ptr->next();
      return *this;
    }
    iterator operator++(int) {
      iterator	tmp = *this;
      operator++();
      return tmp;
    }
    iterator operator--() {
      //next = ptr;
      //if (ptr == 0)
	//ptr = prev;
      //else
	ptr = ptr->prev();
      return *this;
    }
    iterator operator--(int) {
      iterator	tmp = *this;
      operator--();
      return tmp;
    }
    bool operator==( const iterator & x) const {
      return ptr == x.ptr;
    }
    bool operator!=( const iterator & x) const {
      return ptr != x.ptr;
    }
  };
  // allocations/deallocation:

  __mapset(const Compare &comp
#ifdef __GNUG__
      = Compare())
#else
    ) // Cfront can't handle the = Compare() default argument
#endif
      : __RBtree<Allocator>(0,0,0), _comp(comp), _size(0) { }
#ifdef TEMPLATE_MEMBER
    template <class InputIterator>
      __mapset(InputIterator first, InputIterator last,
	  const Compare & comp = Compare());
#endif
  __mapset(const __mapset<Key, T, Compare, Allocator, multi>& x)
    : _comp(x._comp), _size(x._size) {
      if (x.root() != x.nilp())
	recursive_copy( *x.nil, x.root(), root(), first(), last(), nilp());
      else
	root() = first() = last() = nilp();
  }
  ~__mapset() {
    for(iterator i(begin()); i != end();) {
      iterator tmp = i;
      ++tmp;
      delete i.ptr;
      i = tmp;
    }
  }

  __mapset<Key, T, Compare, Allocator, multi>&
  operator=( const __mapset& x) {
    // simple minded implementation: delete old, and create copy of x.
    this->~__mapset();

    _comp = x._comp;
    _size = x._size;

    if (x.root() != x.nilp())
      recursive_copy( *x.nil, x.root(), root(), first(), last(), nilp());
    else
      root() = first() = last() = nilp();

    return *this;
  }

  void swap(__mapset& x) {
    ::swap(_comp,x._comp);
    ::swap(_root,x._root);
    ::swap(_size,x._size);
    // ::swap(_first,x._first);
    // ::swap(_last,x._last);
    ::swap(nil,x.nil);
  }

  // accessors:

  key_compare key_comp() const { return _comp; }
  value_compare value_comp() const { return value_compare(_comp); }
  iterator begin() const { return first(); }
  iterator end() const { return iterator(nilp()); } //0,last(),0); }
  bool empty() const { return _size == 0; }
  size_type size() const { return _size; }
  size_type max_size() const { return INT_MAX; }

  // input/erase:
private:
public:
  pair<iterator, bool> insert(const value_type& x) {
    bool did_insert = false;
    mapsetNode *cur = root();
    while(cur != nilp()) {
      if (value_comp()(x, cur->v)) {	// x smaller()
	if (cur->smaller() != nilp()) {
	  cur = cur->smaller();
	  continue;
	} else {
	  did_insert = true;
	  _size++;
	  mapsetNode *node = new mapsetNode(x,*nil,cur,cur->prev(),cur);
	  cur->prev() = node;
	  cur->smaller() = node;
	  assert(node->prev()->next() == cur);
	  node->prev()->next() = node;
	  RBinsert(node);
	  return make_pair((iterator)node, did_insert);
	}
      }
      if (multi || value_comp()(cur->v, x)) {	// x bigger
	if (cur->larger() != nilp()) {
	  cur = cur->larger();
	  continue;
	} else {
	  did_insert = true;
	  _size++;
	  mapsetNode *node = new mapsetNode(x,*nil,cur,cur,cur->next());
	  cur->next() = node;
	  cur->larger() = node;
	  assert(node->next()->prev() == cur);
	  node->next()->prev() = node;
	  RBinsert(node);
	  return make_pair((iterator)node, did_insert);
	}
      }
      // x is not smaller() nor bigger than cur.v So it should equal it.
      return make_pair((iterator)cur, did_insert);
    }
    // Hmmm root() seems to be zero. So this must be the first() element!
    assert(_size == 0);
    did_insert = true;
    _size = 1;
    root() = new mapsetNode(x,*nil,nilp());
    first() = last() = root();
    
    root()->color = Black;
    return make_pair((iterator)root(), did_insert);
  }
  iterator insert(iterator position, const value_type &x) {
    iterator pnext(position);
    if (pnext != end())
      pnext++;

    // we could handle the case that x == *p faster if we wanted...
    // Just check whether they equal, and if so, return position.
    // And as __mapset is symmetrical, we also can speedup the case that x is just
    // before position.
    // But those are not required, and I don't know about common usage.
    if ((position == end()) || (!(value_comp()(*position, x) &&
	                     ((pnext == end()) || value_comp()(x, *pnext)))))
      return insert(x).first;

    // special case: add x right after position
    mapsetNode *p = position.ptr, *n = pnext.ptr;
    ++_size;
    mapsetNode *node = new mapsetNode(x,*nil,nilp(),p,n);
    p->next() = node;
      assert(n->prev() == p);
      n->prev() = node;
    if (p->larger() == nilp()) {
      p->larger() = node;
      node->parent() = p;
    } else {
      assert(n != nilp());
      assert(n->smaller() == nilp());
      n->smaller() = node;
      node->parent() = n;
    }
    RBinsert(node);
    return node;
  }
#ifdef TEMPLATE_MEMBER
  template <class InputIterator>
  void insert(InputIterator first, InputIterator last);
#endif
  void erase(iterator position) {
    mapsetNode *z = position.ptr;
    --_size;
      
    RBdelete(z);
    
    delete z;
  }
  size_type erase(const key_type &x) {
    iterator i(find(x));
    if (i == end())
      return 0;
    erase(i);
    return 1;
  }
  void erase(iterator first, iterator last) {
    for(iterator i = first; i != last; i++)
      erase(i);
  }
  // upper_bound, find, count and equal_range are all based on lower_bound
  iterator lower_bound(const key_type &x) const {
    // This function does not satisfy the complexity constraint if multi==true;
    // The complexity is Log(N) + count(K), and should be Log(N)
    if (root() == nilp())
      return end();

    mapsetNode *n = root();
    while (1)
      if (key_comp()(x, n->v.first)) {
	if (n->smaller() != nilp())
	  n = n->smaller();
	else
	  return n;
      } else if (key_comp()(n->v.first,x)) {
	if (n->larger() != nilp())
	  n = n->larger();
	else {
	  iterator i = n;
	  return ++i;
	}
      } else
	return n;
  }
  iterator upper_bound(const key_type &x) const {
    iterator i = lower_bound(x);
    return (key_comp()(x, (*i).first) ? i : ++i);
  }
  iterator find(const key_type &x) const {
    iterator i(lower_bound(x));
    return ((i != end()) && (key_comp()(x,(*i).first))) ? end() : i;
  }
  size_type count(const key_type &x) const {
    return find(x) != end() ? 1 : 0;
  }
  pair<iterator, iterator> equal_range(const key_type & x) const {
    return make_pair(lower_bound(x), upper_bound(x));
  }
  bool operator==(const __mapset<Key, T, Compare, Allocator, multi>&x) const {
  // What does __mapset a == __mapset b mean?
  // I assume that it means that __mapset a and b have the same members.

  if (size() != x.size())
    return false;

  iterator	i(begin()), j(x.begin());

  for( ; (i != end()) && (j != x.end()); i++, j++)
    if (*i != *j)
      return false;

  assert((i == end()) && (j == x.end()));

  return true;
  }
  bool operator<(const __mapset<Key, T, Compare, Allocator, multi>&x) const {
  // What does __mapset a == __mapset b mean?
  // I assume that it means that __mapset a and b have the same members.

  if (size() != x.size())
     return size() < x.size();
 
  iterator	i(begin()), j(x.begin());

  for( ; (i != end()) && (j != x.end()); i++, j++)
    if (*i != *j)
      return *i < *j;

  return false;
  }
};

// 9.2.1 Set (schutte, incomplete, mostly done)

struct _empty { };

inline bool
operator ==( const _empty&, const _empty&) {
  return true;
}

inline bool
operator < (const _empty&, const _empty&) {
  return false;
}

template <class Key, class Compare
#ifdef TEMPLATE_DEFAULT_ARG
 = less<Key>
#endif
,
#ifdef TEMPLATE_ARG
template <class U>
#endif
class Allocator
#ifdef TEMPLATE_DEFAULT_ARG
 = allocator
#endif
>	// It should not be _empty, but ::empty. However, we reallly don't want
	// internal errors by g++ ! (schutte)
class set : public __mapset<Key,_empty, Compare,Allocator, false> {
  typedef __mapset<Key,_empty,Compare,Allocator,false>::iterator _iter;
  typedef __mapset<Key,_empty,Compare,Allocator,false> Parent;
  typedef Parent::value_type _value_type;
  
public:
  // typedefs:
  typedef Key value_type;
  typedef Compare value_compare;
    class iterator : public _iter {
    public:
      Key &operator*()  { return (*(*((_iter *)this))).first; }
      const Key &operator*() const  { 
         return (*(*((const _iter *)this))).first; 
      }
       iterator() {}
      iterator( const _iter &i ) 
	: _iter(i) { } 
      iterator( const iterator &i )
	: _iter(i) { }
  };	
  typedef iterator const_iterator;
						
  value_compare value_comp() const { return _comp; }
  // Allocation / Deallocation
  set(const Compare &comp
#ifdef __GNUG__
      = Compare()
#endif
    ) // Cfront can't handle the = Compare() default argument
      : Parent(comp) { }

  // Accessors
  iterator begin() const {
    return Parent::begin();
  }
  iterator end() const {
    return Parent::end();
  }

  // insert / erase
  pair<iterator,bool> insert(const value_type &x) {
    pair<_iter,bool> p = Parent::insert(_value_type(x,_empty()));
    return pair<iterator,bool>(p.first,p.second);
  }
  iterator insert(iterator position, const value_type &x) {
    return Parent::insert(position, _value_type(x,_empty()));
  }

  // set operations:
  iterator find (const key_type &x) const {
    return Parent::find(x);
  }
  iterator lower_bound(const key_type &x) const {
    return Parent::lower_bound(x);
  }
  iterator upper_bound(const key_type &x) const {
    return Parent::upper_bound(x);
  }
  pair<iterator,iterator> equal_range(const key_type &x) const {
    pair<_iter,_iter> p = Parent::equal_range(x);
    return pair<iterator,iterator>(p.first,p.second);
  }
};

// 9.2.2 Multiset (schutte, incomplete, mostly done)

template <class Key, class Compare
#ifdef TEMPLATE_DEFAULT_ARG
 = less<Key>
#endif
,
#ifdef TEMPLATE_ARG
template <class U>
#endif
class Allocator
#ifdef TEMPLATE_DEFAULT_ARG
 = allocator
#endif
>	// It should not be _empty, but ::empty. However, we reallly don't want
	// internal errors by g++ ! (schutte)
class multiset : public __mapset<Key,_empty, Compare,Allocator,true> {
  typedef __mapset<Key,_empty,Compare,Allocator,true>::iterator _iter;
  typedef __mapset<Key,_empty,Compare,Allocator,true> Parent;
  typedef Parent::value_type _value_type;
  
public:
  // typedefs:
  typedef Key value_type;
  typedef Compare value_compare;
    class iterator : public _iter {
    public:
      Key &operator*()  { return (*(*((_iter *)this))).first; }
      const Key &operator*() const  { 
         return (*(*((const _iter *)this))).first; 
      }
       iterator() {}
      iterator( const _iter &i ) 
	: _iter(i) { } 
      iterator( const iterator &i )
	: _iter(i) { }
  };	
  typedef iterator const_iterator;
						
  value_compare value_comp() const { return _comp; }
  // Allocation / Deallocation
  multiset(const Compare &comp
#ifdef __GNUG__
      = Compare()
#endif
    ) // Cfront can't handle the = Compare() default argument
      : Parent(comp) { }

  // Accessors
  iterator begin() const {
    return Parent::begin();
  }
  iterator end() const {
    return Parent::end();
  }

  // insert / erase
  iterator insert(const value_type &x) {
    pair<_iter,bool> p = Parent::insert(_value_type(x,_empty()));
    return p.first;
  }
  iterator insert(iterator position, const value_type &x) {
    return Parent::insert(position, _value_type(x,_empty()));
  }
  void erase(iterator position) {
    Parent::erase(position);
  }
  size_type erase(const key_type &x) {
    iterator i(lower_bound(x));
    for (int cnt = 0; (i != end()) && (!key_comp()(x,*i)); cnt++ )
      erase(i++);
    return cnt;
  }
  void erase(iterator first, iterator last) {
    Parent::erase(first, last);
  }

  // set operations:
  iterator find (const key_type &x) const {
    return Parent::find(x);
  }
  iterator lower_bound(const key_type &x) const {
    iterator n = Parent::lower_bound(x);
    // Find first occurrence of n
    iterator p = n; p--;
    for(; (p != end()) && (!key_comp()(*p,x)); p-- )
      n = p;
    return n;
  }	  
  iterator upper_bound(const key_type &x) const {
    iterator i = Parent::lower_bound(x); 
    while((i != end()) && (! key_comp()(x, *i)))
      i++;
    return i;
  }
  pair<iterator, iterator> equal_range(const key_type & x) const {
    return make_pair(lower_bound(x), upper_bound(x));
  }
};


// 9.2.3 Map (schutte, incomplete, mostly done)

template <class Key, class T, class Compare
#ifdef TEMPLATE_DEFAULT_ARG
 = less<Key>
#endif
,
#ifdef TEMPLATE_ARG
template <class U>
#endif
class Allocator
#ifdef TEMPLATE_DEFAULT_ARG
 = allocator
#endif
>
class map : public __mapset<Key,T,Compare,Allocator,false> {			       
public:	
  map(const Compare &comp
#ifdef __GNUG__
      = Compare()
#endif
    ) // Cfront can't handle the = Compare() default argument
      : __mapset<Key,T,Compare,Allocator,false>(comp) { }								     
  T& operator [] (const key_type& k) {
    iterator i = lower_bound(k);
    if (i == end())
      i = insert(value_type(k, T())).first;
    else if ((*i).first != k)
      i = insert(--i, value_type(k, T()));
    return (*i).second;
  }
};

// 9.2.4 Multimap (schutte, incomplete, mostly done)

template <class Key, class T, class Compare
#ifdef TEMPLATE_DEFAULT_ARG
 = less<Key>
#endif
,
#ifdef TEMPLATE_ARG
template <class U>
#endif
class Allocator
#ifdef TEMPLATE_DEFAULT_ARG
 = allocator
#endif
>
class multimap : public __mapset<Key,T,Compare,Allocator,true> {			       
  typedef __mapset<Key,T,Compare,Allocator,true> Parent;
public:	
  multimap(const Compare &comp
#ifdef __GNUG__
      = Compare()
#endif
    ) // Cfront can't handle the = Compare() default argument
      : __mapset<Key,T,Compare,Allocator,true>(comp) { }								     
  T& operator [] (const key_type& k) {
    iterator i = lower_bound(k);
    if (i == end())
      i = insert(value_type(k,T())).first;
    else if ((*i).first != k)
      i = insert(--i, value_type(k, T()));
    return (*i).second;
  }
  void erase(iterator position) {
    Parent::erase(position);
  }
  size_type erase(const key_type &x) {
    iterator i(lower_bound(x));
    for (int cnt = 0; (i != end()) && (!key_comp()(x,(*i).first)); cnt++ )
      erase(i++);
    return cnt;
  }
  void erase(iterator first, iterator last) {
    Parent::erase(first, last);
  }

iterator lower_bound(const key_type &x) const {
    iterator n = Parent::lower_bound(x);
    // Find first occurrence of n
    iterator p = n; p--;
    for(; (p != end()) && (!key_comp()((*p).first,x)); p-- )
      n = p;
    return n;
  }	  
  iterator upper_bound(const key_type &x) const {
    iterator i = Parent::lower_bound(x); 
    while((i != end()) && (! key_comp()(x, (*i).first)))
      i++;
    return i;
  }
  pair<iterator, iterator> equal_range(const key_type & x) const {
    return make_pair(lower_bound(x), upper_bound(x));
  }
};

// 10 Stream iterators (huber, complete)

// 10.1 Istream iterator (huber, complete)

template <class T, class Distance
#ifdef TEMPLATE_DEFAULT_ARG
= ptrdiff_t
#endif
>
class istream_iterator : public input_iterator<T, Distance> {
  istream * istr;
  T val;

  bool bad() const { return !istr || !*istr; }

public:
  istream_iterator() : istr(0) {}

  istream_iterator(istream & _istr) : istr(&_istr) {
    (*istr) >> val;
  }

  istream_iterator(const istream_iterator<T,Distance> & i)
    : istr(i.istr), val(i.val) { }

  bool operator == (const istream_iterator<T,Distance> & i) const {
    return bad() && i.bad();
  }

  const T & operator * () const { return val; }

  istream_iterator<T,Distance> & operator ++ () {
    (*istr) >> val;
    return *this;
  }

  istream_iterator<T,Distance> operator ++ (int) {
    istream_iterator<T,Distance> temp = *this;
    (*istr) >> val;
    return temp;
  }
};

// 10.2 Ostream iterator (huber, complete)

template <class T>
class ostream_iterator : public output_iterator {
  ostream & ostr;
  const char * end;

public:
  ostream_iterator(ostream & _ostr) : ostr(_ostr), end(0) {}

  ostream_iterator(ostream & _ostr, const char * _end)
    : ostr(_ostr), end(_end) {}

  ostream_iterator(const ostream_iterator<T> & i)
    : ostr(i.ostr), end(i.end) {}

  ostream_iterator<T> & operator = (const T & val) {
    ostr << val;
    if (end)
      ostr << end;
    return *this;
    }

  ostream_iterator<T> & operator * () { return *this; }

  ostream_iterator<T> & operator ++ () { return *this; }

  ostream_iterator<T> operator ++ (int) {
    ostream_iterator<T> temp = *this;
    return temp;
  }
};

// 11 Algorithms (incomplete)

// 11.1 Non-mutating sequence operations (complete)

// 11.1.1 For each (mrs, complete)

template <class InputIterator, class Function>
inline void
for_each(InputIterator first, InputIterator last, Function f) {
  while (first != last) {
    f(*first++);
  }
}

// 11.1.2 Find (mrs, complete)

template <class InputIterator, class T>
inline InputIterator
find(InputIterator first, InputIterator last, const T& value) {
  while (first != last) {
    if (*first == value)
      break;
    ++first;
  }
  return first;
}

template <class InputIterator, class Predicate>
inline InputIterator
find_if(InputIterator first, InputIterator last, Predicate pred) {
  while (first != last) {
    if (pred(*first))
      break;
    ++first;
  }
  return first;
}

// 11.1.3 Adjacent find (huber, complete, buggy)

// XXX should not ++ the same iterator twice like this.
// The below is really a ForwardIterator solution, not an InputIterator
// solution.  The value should be stored, see unique for details on how
// to do this.
template <class InputIterator>
inline InputIterator
adjacent_find(InputIterator first, InputIterator last) {
  ForwardIterator scan = first;

  if (first != last)
    while (++scan != last && *first != *scan)
      ++first;

  return first;
}

// XXX should not ++ the same iterator twice like this.
// The below is really a ForwardIterator solution, not an InputIterator
// solution.  The value should be stored, see unique for details on how
// to do this.
template <class InputIterator, class BinaryPredicate>
inline InputIterator
adjacent_find(InputIterator first, InputIterator last,
	      BinaryPredicate binary_pred) {
  ForwardIterator scan = first;

  if (first != last)
    while (++scan != last && !binary_pred(*first, *scan))
      ++first;

  return first;
}

// 11.1.4 Count (mrs, complete)

template <class InputIterator, class T, class Size>
inline void
count(InputIterator first, InputIterator last, const T& value, Size& n) {
  while (first != last) {
    if (*first == value)
      n += 1;
    ++first;
  }
}

template <class InputIterator, class Predicate, class Size>
inline void
count_if(InputIterator first, InputIterator last, Predicate pred, Size& n) {
  while (first != last) {
    if (pred(*first))
      n += 1;
    ++first;
  }
}

// 11.1.5 Mismatch (huber, complete)

template <class InputIterator1, class InputIterator2>
inline pair<InputIterator1, InputIterator2>
mismatch(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2) {
  while (first1 != last1 && *first1 == *first2) {
    ++first1;
    ++first2;
  }

  return make_pair(first1, first2);
}

template <class InputIterator1, class InputIterator2, class BinaryPredicate>
inline pair<InputIterator1, InputIterator2>
mismatch(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2, BinaryPredicate binary_pred) {
  while (first1 != last1 && binary_pred(*first1, *first2)) {
    ++first1;
    ++first2;
  }

  return make_pair(first1, first2);
}

// 11.1.6 Equal (mrs, complete)

template <class InputIterator1, class InputIterator2>
inline bool
equal(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2) {
  while (first1 != last1 && *first1 == *first2) {
    ++first1;
    ++first2;
  }
  return first1 == last1;
}

template <class InputIterator1, class InputIterator2, class BinaryPredicate>
inline bool
equal(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2, BinaryPredicate binary_pred) {
  while (first1 != last1 && binary_pred(*first1, *first2)) {
    ++first1;
    ++first2;
  }
  return first1 == last1;
}

// 11.1.7 Search (huber, complete)

// XXX the below can be made more elegant
template <class ForwardIterator1, class ForwardIterator2>
inline ForwardIterator1
search(ForwardIterator1 first1, ForwardIterator1 last1, ForwardIterator2 first2,
ForwardIterator2 last2) {
  // mark position where scan ends (because target sequence is exhausted)
  ForwardIterator1 scan1_stop = first1;
  ForwardIterator2 temp = first2;

  for (; scan1_stop != last1 && temp != last2; temp++)
    scan1_stop++;

  if (temp != last2)    // [first1,last1) is shorter than [first2,last2)
    return last1;

  while (scan1_stop != last1) {
    ForwardIterator1 scan1 = first1;
    ForwardIterator2 scan2 = first2;

    // scan both sequences while predicate is satisfied
    while (scan1 != scan1_stop && *scan1 == *scan2)
      scan1++, scan2++;

    if (scan1 == scan1_stop)    // scan stopped without mismatch
      return first1;            // return beginning of match

    first1++, scan1_stop++;
  }

  return last1;                 // no match found
}

template <class ForwardIterator1, class ForwardIterator2, class BinaryPredicate>
inline ForwardIterator1
search(ForwardIterator1 first1, ForwardIterator1 last1, ForwardIterator2 first2,
 ForwardIterator2 last2, BinaryPredicate binary_pred) {
  // mark position where scan ends (because target sequence is exhausted)
  ForwardIterator1 scan1_stop = first1;
  ForwardIterator2 temp = first2;

  for (; scan1_stop != last1 && temp != last2; temp++)
    scan1_stop++;

  if (temp != last2)    // [first1,last1) is shorter than [first2,last2)
    return last1;

  while (scan1_stop != last1) {
    ForwardIterator1 scan1 = first1;
    ForwardIterator2 scan2 = first2;

    // scan both sequences while predicate is satisfied
    while (scan1 != scan1_stop && binary_pred(*scan1, *scan2))
      scan1++, scan2++;

    if (scan1 == scan1_stop)    // scan stopped without mismatch
      return first1;            // return beginning of match

    first1++, scan1_stop++;
  }

  return last1;                 // no match found
}

// 11.2 Mutating sequence operations (complete)

// 11.2.1 Copy (mrs, complete)

template <class InputIterator, class OutputIterator>
inline OutputIterator
copy(InputIterator first, InputIterator last, OutputIterator result) {
  while (first != last)
    *result++ = *first++;
  return result;
}

template <class BidirectionalIterator1, class BidirectionalIterator2>
inline BidirectionalIterator2
copy_backward(BidirectionalIterator1 first, BidirectionalIterator1 last, BidirectionalIterator2 result) {
  while (last != first)
    *--result = *--last;
  return result;
}

// 11.2.2 Swap (mrs, complete)

template <class T>
inline void
swap(T& a, T& b) {
  T tmp = a;
  a = b;
  b = tmp;
}

template <class ForwardIterator1, class ForwardIterator2>
inline ForwardIterator2
swap_ranges(ForwardIterator1 first1, ForwardIterator1 last1, ForwardIterator2 first2) {
  while (first1 != last1)
    {
      swap(*first1, *first2);
      ++first1;
      ++first2;
    }
  return first2;
}

// 11.2.3 Transform (wilson, complete)

template <class InputIterator, class OutputIterator, class UnaryOperation>
OutputIterator
transform(InputIterator first, InputIterator last, OutputIterator result,
	  UnaryOperation op) {
  for (; first != last; ++first, ++result) {
    *result = op(*first);
  }
  return result;
}

template <class InputIterator1, class InputIterator2, class OutputIterator,
	  class BinaryOperation>
OutputIterator
transform(InputIterator1 first1, InputIterator1 last1, InputIterator2 first2,
	  OutputIterator result, BinaryOperation binary_op) {
  for (; first1 != last1; ++first1, ++first2, ++result) {
    *result = op(*first1, *first2);
  }
  return result;
}

// 11.2.4 Replace (niklas, complete)

// Replace all occurrences of OLD_VALUE in the range [FIRST, LAST) with
// NEW_VALUE.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class ForwardIterator, class T>
inline void
replace(ForwardIterator first, ForwardIterator last, const T& old_value,
	const T& new_value) {
  for (; first != last; first++)
    if (*first == old_value)
      *first = new_value;
}

// Replace elements satisfying PRED in the range [FIRST, LAST) with NEW_VALUE.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class ForwardIterator, class Predicate, class T>
inline void
replace_if(ForwardIterator first, ForwardIterator last, Predicate pred,
	   const T& new_value) {
  for (; first != last; first++)
    if (pred(*first))
      *first = new_value;
}

// Copy elements from the range [FIRST, LAST) to
// [RESULT, RESULT + (LAST - FIRST)) replacing occurrences of OLD_VALUE with
// NEW_VALUE.  Return the resulting end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class InputIterator, class OutputIterator, class T>
inline OutputIterator
replace_copy(InputIterator first, InputIterator last, OutputIterator result,
             const T& old_value, const T& new_value)
{
  while (first != last) {
    register const T& x = *first++;
    *result++ = x == old_value ? new_value : x;
  }
  return result;
}

// Copy elements from the range [FIRST, LAST) to
// [RESULT, RESULT + (LAST - FIRST)) replacing values satisfying PRED with
// NEW_VALUE.  Return the resulting end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class InputIterator, class OutputIterator, class Predicate, class T>
inline OutputIterator
replace_copy_if(InputIterator first, InputIterator last,
		OutputIterator result, Predicate pred, const T& new_value) {
  while (first != last) {
    register const T& x = *first++;
    *result++ = pred(x) ? new_value : x;
  }
  return result;
}

// 11.2.5 Fill (niklas, complete)

// Fill range [FIRST, LAST) with VALUE
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class ForwardIterator, class T>
inline void
fill(ForwardIterator first, ForwardIterator last, const T& value) {
  while (first != last)
    *first++ = value;
}

// Fill N elements from FIRST and on with VALUE
// Assumption: N is non-negative, FIRST is dereferencable.
template <class OutputIterator, class Size, class T>
inline void
fill_n(OutputIterator first, Size n, const T& value) {
  while (n--)
    *first++ = value;
}

// 11.2.6 Generate (niklas, complete)

// Fill range [FIRST, LAST) with values generated by GEN
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class ForwardIterator, class Generator>
inline void
generate(ForwardIterator first, ForwardIterator last, Generator gen) {
  while (first != last)
    *first++ = gen();
}

// Fill N elements from FIRST and on with values generated by GEN
// Assumption: N is non-negative, FIRST is dereferencable.
template <class OutputIterator, class Size, class Generator>
inline void
generate_n(OutputIterator first, Size n, Generator gen) {
  while (n--)
    *first++ = gen();
}

// 11.2.7 Remove (niklas, complete)

// Remove from range [FIRST, LAST) all occurrences of VALUE.  Return the
// resulting end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class ForwardIterator, class T>
inline ForwardIterator
remove(ForwardIterator first, ForwardIterator last, const T& value) {
  ForwardIterator result = first;
  // Until we find the first occurrence of VALUE this is the fast way.
  while (first != last && *first++ != value)
    result++;
  while (first != last) {
    register const T& x = *first++;
    if (x != value)
      *result++ = x;
  }
  return result;
}

// Remove from range [FIRST, LAST) elements satisfying PRED.  Return the
// resulting end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class ForwardIterator, class Predicate>
inline ForwardIterator
remove_if(ForwardIterator first, ForwardIterator last, Predicate pred) {
  return __remove_if(first, last, pred, value_type(first));
}

template <class ForwardIterator, class Predicate, class T>
inline ForwardIterator
__remove_if(ForwardIterator first, ForwardIterator last, Predicate pred,
	    const T*) {
  ForwardIterator result = first;
  // Until we find the element satisfying PRED this is the fast way.
  while (first != last && !pred(*first++))
    result++;
  while (first != last) {
    register const T& x = *first++;
    if (!pred(x))
      *result++ = x;
  }
  return result;
}

// Copy elements from [FIRST, LAST) to RESULT and on removing occurrences of
// VALUE.  Return the resulting end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class InputIterator, class OutputIterator, class T>
inline OutputIterator
remove_copy(InputIterator first, InputIterator last, OutputIterator result,
	    const T& value) {
  while (first != last) {
    register const T& x = *first++;
    if (x != value)
      *result++ = x;
  }
  return result;
}

// Copy elements from [FIRST, LAST) to RESULT and on removing values
// satisfying PRED.  Return the resulting end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class InputIterator, class OutputIterator, class Predicate>
inline OutputIterator
remove_copy_if(InputIterator first, InputIterator last, OutputIterator result,
	       Predicate pred) {
  return __remove_copy_if(first, last, result, pred, value_type(first));
}

template <class InputIterator, class OutputIterator, class Predicate, class T>
inline OutputIterator
__remove_copy_if(InputIterator first, InputIterator last,
		 OutputIterator result, Predicate pred, const T*) {
  while (first != last) {
    register T& x = *first++;
    if (!pred(x))
      *result++ = x;
  }
  return result;
}

// 11.2.8 Unique (niklas, complete)

// Remove from range [FIRST, LAST) all but the first occurrence from each
// consecutive group of equal elements.  Return the new end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class ForwardIterator>
inline ForwardIterator
unique(ForwardIterator first, ForwardIterator last) {
  return __unique(first, last, value_type(first));
}

template <class ForwardIterator, class T>
ForwardIterator
__unique(ForwardIterator first, ForwardIterator last, const T*) {
  if (first == last)
    return first;
  // If operator* is expensive for ForwardIterator, this will save cycles.
  register const T* prev = &*first++;
  ForwardIterator result = first;
  // Until we see our first duplicate, do it fast.
  register const T* tmp;
  while (first != last && *(tmp = &*first++) != *prev) {
    result++;
    prev = tmp;
  }
  while (first != last) {
    register const T& x = *first++;
    if (x != *prev)
      *result++ = x;
    prev = &x;
  }
  return result;
}

// Remove from range [FIRST, LAST) all but the first occurrence from each
// consecutive group of elements satisfying BINARY_PRED pairwise with the
// former element as second operand, i.e. binary_pred(*i, *(i - 1)) is true.
// Return the new end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class ForwardIterator, class BinaryPredicate>
inline ForwardIterator
unique(ForwardIterator first, ForwardIterator last,
       BinaryPredicate binary_pred) {
  return __unique(first, last, binary_pred, value_type(first));
}

template <class ForwardIterator, class BinaryPredicate, class T>
ForwardIterator
__unique(ForwardIterator first, ForwardIterator last,
	 BinaryPredicate binary_pred, const T*) {
  if (first == last)
    return first;
  ForwardIterator result = first;
  // If operator* is expensive for ForwardIterator, this will save cycles.
  register T* prev = &*first++;
  // Until we see our first pair satisfying BINARY_PRED, do it fast.
  register T* tmp;
  while (first != last && !binary_pred(*(tmp = &*first++), *prev)) {
    result++;
    prev = tmp;
  }
  while (first != last) {
    register const T& x = *first++;
    if (!binary_pred(x, *prev))
      *result++ = x;
    prev = &x;
  }
  return result;
}

// Copy elements from [FIRST, LAST) to RESULT and on removing all but the
// first occurrence from each consecutive group of equal elements.  Return
// the new end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class InputIterator, class OutputIterator>
inline OutputIterator
unique_copy(InputIterator first, InputIterator last, OutputIterator result) {
  return __unique_copy(first, last, result, value_type(first));
}

template <class InputIterator, class OutputIterator, class T>
OutputIterator
__unique_copy(InputIterator first, InputIterator last, OutputIterator result,
	      const T*) {
  if (first == last)
    return first;
  // If operator* is expensive for InputIterator, this will save cycles.
  register T* prev = &*first++;
  *result++ = *prev;
  while (first != last) {
    register const T& x = *first++;
    if (x != *prev)
      *result++ = x;
    prev = &x;
  }
  return result;
}

// Copy elements from [FIRST, LAST) to RESULT and on removing all but the
// first occurrence from each consecutive group of elements satisfying
// BINARY_PRED pairwise with the former element as second operand, i.e.
// binary_pred(*i, *(i - 1)) is true.  Return the new end-of-range iterator.
// Assumption: LAST is reachable from FIRST, FIRST is dereferencable.
template <class InputIterator, class OutputIterator, class BinaryPredicate>
inline OutputIterator
unique_copy(InputIterator first, InputIterator last, OutputIterator result,
	    BinaryPredicate binary_pred) {
  return __unique_copy(first, last, result, binary_pred, value_type(first));
}

template <class InputIterator, class OutputIterator, class BinaryPredicate,
          class T>
OutputIterator
__unique_copy(InputIterator first, InputIterator last, OutputIterator result,
	      BinaryPredicate binary_pred, const T*) {
  if (first == last)
    return result;
  // If operator* is expensive for InputIterator, this will save cycles.
  register T* prev = &*first++;
  *result++ = *prev;
  while (first != last) {
    register const T& x = *first;
    if (!binary_pred(x, *prev))
      *result++ = x;
    prev = &x;
  }
  return result;
}

// 11.2.9 Reverse (mrs, complete)

template <class BidirectionalIterator>
inline void
reverse(BidirectionalIterator first, BidirectionalIterator last) {
  _reverse(first, last, value_type(first), distance_type(first));
}

// 11.2.10 Rotate (niklas, complete, optimizable)

// Rotate the range [FIRST, LAST) so that values at FIRST end up at MIDDLE.
// Assumption: both MIDDLE & LAST is reachable from FIRST.
//             LAST is reachable from MIDDLE.
//             FIRST & MIDDLE is dereferencable.
template <class ForwardIterator>
inline void
rotate(ForwardIterator first, ForwardIterator middle, ForwardIterator last) {
  __rotate(first, middle, last, value_type(first));
}

template <class ForwardIterator, class T>
inline void
__rotate(ForwardIterator first, ForwardIterator middle, ForwardIterator last,
	 const T*) {
  ForwardIterator target = middle;
  ForwardIterator limit = first;
  ForwardIterator next = target;
  next++;
  if (next == last)
    next == first;
  while (next != middle && target != limit) {
    source = limit;
    while (next != middle && source != middle) {
      if (target == limit)
	limit == source;
      swap(*source++, *target);
      target = next++;
      if (next == last)
	next = first;
    }
  }
}

// Copy elements from [FIRST, LAST) to RESULT and on rotated so that values
// at FIRST end up at the position in the resulting stream corresponding to
// MIDDLE.  Return the resulting end-of-stream iterator.
// Assumption: both MIDDLE & LAST is reachable from FIRST.
//             LAST is reachable from MIDDLE.
//             FIRST & MIDDLE is dereferencable.
template <class ForwardIterator, class OutputIterator>
inline OutputIterator
rotate_copy(ForwardIterator first, ForwardIterator middle,
	    ForwardIterator last, OutputIterator result) {
  return __rotate(first, middle, last, result, value_type(first));
}

// This implementation could (and will) be specialized for random-access
// iterators in order to win some cycles.
template <class ForwardIterator, class OutputIterator, class T>
inline OutputIterator
__rotate_copy(ForwardIterator first, ForwardIterator middle,
	      ForwardIterator last, OutputIterator result, const T*) {
  ForwardIterator i = first;
  ForwardIterator j = middle;
  if (first != middle) {
    while (j != last) {
      i++;
      j++;
    }
    j = first;
  }
  while (j != last) {
    *result++ = *i++;
    if (i == last)
      i == first;
    j++;
  }
  return result;
}

// 11.2.11 Random shuffle (niklas, complete, check)

// XXX There are several problems here: Is rand() portable enough?
// XXX Is it sufficient to use unsigned long precision?  Is the cast
// XXX below needed if I want to be sure not to lose precision
// XXX (assuming RAND_MAX is the maximum of its type, e.g. int)?
// XXX What if RAND_MAX is on the order (or even less than) the
// XXX number of elements in a collection to be shuffled randomly?
// XXX Maybe we should provide our own implementation?

#ifndef RAND_MAX
#ifdef __sun__
#define RAND_MAX 2147483647
#endif
#endif
struct __Rand {
  unsigned long operator ()(unsigned long n) {
    return rand() * n / ((unsigned long)RAND_MAX + 1);
  }
};

// Shuffle elements in the range [FIRST, LAST) uniformly.
// Assumptions: FIRST is dereferencable, LAST is reachable from FIRST.
template <class RandomAccessIterator>
inline void
random_shuffle(RandomAccessIterator first, RandomAccessIterator last) {
  __random_shuffle(first, last, __Rand(), distance_type(first));
}

// Shuffle elements in the range [FIRST, LAST) using RAND as a random number
// generator.
// Assumptions: FIRST is dereferencable, LAST is reachable from FIRST.
template <class RandomAccessIterator, class RandomNumberGenerator>
inline void
random_shuffle(RandomAccessIterator first, RandomAccessIterator last,
               RandomNumberGenerator& rand_) {
  __random_shuffle(first, last, rand_, distance_type(first));
}

template <class RandomAccessIterator, class RandomNumberGenerator,
	  class Distance>
void
__random_shuffle(RandomAccessIterator first, RandomAccessIterator last,
                 RandomNumberGenerator& rand_, Distance*) {
  Distance n = 0;
  for (distance(first, last, n); --n > 0; first++) {
    Distance m = rand_(n);
    if (m)
      swap(*first, first[m]);
  }
}

// 11.2.12 Partitions (niklas, complete)

// Partition the range [FIRST, LAST) into two sections, starting from FIRST
// with those elements satisfying PRED then those not.  Return an iterator
// denoting the first element *not* satisfying PRED, or LAST if all elements
// do.
// Assumptions: FIRST is dereferencable, LAST is reachable from FIRST.
template <class BidirectionalIterator, class Predicate>
inline BidirectionalIterator
partition(BidirectionalIterator first, BidirectionalIterator last,
	  Predicate pred) {
  while (first != last) {
    while (first != last && pred(*first))
      first++;
    if (first == last)
      break;
    while (first != --last && !pred(*last))
      ;
    if (first == last)
      break;
    swap (*first++, *last);
  }
  return first;
}

// Partition the range [FIRST, LAST) into two sections, starting from FIRST
// with those elements satisfying PRED then those not.  The relative order
// of elements in each partition is preserved.  Return an iterator denoting
// the first element *not* satisfying PRED, or LAST if all elements do.
// Assumptions: FIRST is dereferencable, LAST is reachable from FIRST.
template <class ForwardIterator, class Predicate>
inline ForwardIterator
stable_partition(ForwardIterator first, ForwardIterator last,
		 Predicate pred) {
  if (first == last)
    return first;
  return __stable_partition(first, last, pred, value_type(first),
			    distance_type(first));
}

template <class ForwardIterator, class Predicate, class T, class Distance>
ForwardIterator
__stable_partition(ForwardIterator first, ForwardIterator last,
		   Predicate pred, T*, Distance*) {
  Distance len = 0;
  distance(first, last, len);
  if (len == 1)
    return pred(*first) ? last : first;
  Pair<T*, ptrdiff_t> buf = get_temporary_buffer (len, (T*)0);
  if (buf.second != len) {
    return_temporary_buffer(buf.first);
    return __inplace_stable_partition(first, last, pred, len);
  }
  ForwardIterator i = first;
  int n = 0, m = len;
  while (i != last) {
    register const T& x = *i++;
    construct(buf.first + (pred(x) ? n++ : --m), x);
  }
  m = 0;
  ForwardIterator result = last;
  while (first != last) {
    if (m == n)
      result = first;
    *first++ = buf.first[m < n ? m++ : len - (m++ - n)];
  }
  while (len)
    destroy (buf.first + --len);
  return_temporary_buffer (buf.first);
  return result;
}

template <class ForwardIterator, class Predicate, class Distance>
ForwardIterator
__inplace_stable_partition(ForwardIterator first, ForwardIterator last,
			   Predicate pred, Distance len) {
  if (len == 1)
    return pred(*first) ? last : first;
  ForwardIterator middle = first;
  advance(middle, len / 2);
  ForwardIterator first_cut
    = __inplace_stable_partition(first, middle, pred, len / 2);
  ForwardIterator second_cut
    = __inplace_stable_partition(middle, last, pred, len - len / 2);
  rotate(first_cut, middle, second_cut);
  len = 0;
  distance(middle, second_cut, len);
  advance(first_cut, len);
  return first_cut;
}

// 11.3 Sorting and related operations (incomplete)

// 11.3.1 Sort (wilson, gafter, incomplete)

// 11.3.2 Nth element (mav, complete)

// Returns a random iterator in the range [first, last).
// FIXME: revise this when the random number generation for the
// library will be decided.
template<class ForwardIterator, class Distance>
ForwardIterator
__random_iterator(ForwardIterator first,
		  ForwardIterator last,
		  Distance* dummy) {
  Distance dist = 0;
  distance (first, last, dist);
  advance (first, rand () % dist);
  return first;
}

template<class ForwardIterator>
ForwardIterator
__random_iterator(ForwardIterator first, ForwardIterator last) {
  return __random_iterator (first, last, distance_type (first));
}

// partition elements in [first, last) around a randomly selected pivot.
template<class BidirectionalIterator, class Predicate>
BidirectionalIterator
__randomized_partition(BidirectionalIterator first,
		       BidirectionalIterator last,
		       Predicate pred) {
  // The nicer statement in the #else portion requires changes in the
  // randomized_select procedure (where we make sure that no partition is
  // *ever* empty.
#ifndef PIVOT_MUST_BE_FIRST
  swap (*first, *__random_iterator (first,last));
  return partition (first, last, bind2nd (pred, (*first)));
#else
  return partition (first, last, bind2nd (pred, *__random_iterator (first,last)));
#endif
}

template<class RandomAccessIterator, class Distance, class Predicate>
void
nth_element(RandomAccessIterator first,
	    RandomAccessIterator nth,
	    RandomAccessIterator last,
	    Predicate pred,
	    Distance* dummy) {
  Distance dist = 1;
  distance (first, nth, dist);
  swap (*nth, *__randomized_select (first, last, dist, pred));
}

template<class RandomAccessIterator, class Compare>
void
nth_element(RandomAccessIterator first,
	    RandomAccessIterator nth,
	    RandomAccessIterator last,
	    Compare comp) {
  return nth_element (first, nth, last, comp, distance_type (first));
}

template<class RandomAccessIterator, class ElementType>
void
__nth_element(RandomAccessIterator first,
	      RandomAccessIterator nth,
	      RandomAccessIterator last,
	      ElementType* dummy, 
	      bool overload_solve) {
  nth_element (first, nth, last, less<ElementType>() , distance_type (first));
}

template<class RandomAccessIterator>
void
nth_element(RandomAccessIterator first,
	    RandomAccessIterator nth,
	    RandomAccessIterator last) {
  __nth_element (first, nth, last, value_type (first), true);
}


// return an iterator in the range [first, last) pointing to the
// element that would take the POSITIONth place if the range were
// sorted. No provision is made for sorting small ranges because the
// treshold is probably container-dependent. We can think about it
// later (and we need the sort routines to experiment with).
template<class RandomAccessIterator, class Distance, class Predicate>
RandomAccessIterator
__randomized_select(RandomAccessIterator first,
		    RandomAccessIterator last,
		    Distance position,
		    Predicate pred) {
  if (first == last - 1) 
    return first;
  else {
    RandomAccessIterator q = __randomized_partition (first, last, pred);

    Distance k = 0;
    distance (first, q, k);

    // We cannot allow for a partition to be empty: endless loop ahead
    if (q == first || q == last) {
      q = first + 1;
      k = 1;
    }
    
    if (position <= k)
      return __randomized_select (first, q, position, pred);
    else
      return __randomized_select (q, last, position - k, pred);
  }
}

// 11.3.3 Binary search (brad, complete)

template <class ForwardIterator, class T, class Compare>
ForwardIterator lower_bound(ForwardIterator first, ForwardIterator last,
#ifndef __CONST_CONST_TEMPLATE_BUG
			    const
#endif
			    T& value, Compare comp,
			    forward_iterator_tag) {
  if (first == last) return first;
  int left = 0;
  int right = 0;
  ForwardIterator iter = first;
  while (iter != last) {
    iter++;
    right++;
  }
  iter = first;
  int pos = (right-left)/2;
  for (int i=0; i<pos; i++)
    iter++;
  while (iter != last) {
    if (comp(*iter,value)) {
      first = iter;
      left = pos;
      int dif = (right-pos);
      dif = dif/2 + (dif&1);
      for (int i=0; i<dif; i++) iter++;
      pos += dif;
    } else {
      last = iter;
      right = pos;
      iter = first;
      int dif = (pos-left)/2;
      for (int i=0; i<dif; i++) iter++;
      pos = left+dif;
    }
  }
  return iter;
}

template <class BidirectionalIterator, class T, class Compare>
BidirectionalIterator lower_bound(BidirectionalIterator first,
				  BidirectionalIterator last,
#ifndef __CONST_CONST_TEMPLATE_BUG
				  const
#endif
				  T& value, Compare comp,
				  bidirectional_iterator_tag) {
  return lower_bound(first,last,value,comp,forward_iterator_tag());
}

template <class RandomAccessIterator, class T, class Compare>
RandomAccessIterator lower_bound(RandomAccessIterator first,
				 RandomAccessIterator last,
#ifndef __CONST_CONST_TEMPLATE_BUG
				 const
#endif
				 T& value, Compare comp,
				 random_access_iterator_tag) {
  RandomAccessIterator iter = first + (last-first)/2;
  while (iter != last) {
    if (comp(*iter,value)) {
      first = iter;
      int dif = (last-iter);
      iter += dif/2 + (dif&1);
    } else {
      last = iter;
      iter = first + (iter-first)/2;
    }
  }
  return iter;
}

template <class ForwardIterator, class T, class Compare>
ForwardIterator lower_bound(ForwardIterator first, ForwardIterator last,
			    const T& value, Compare comp) {
  return lower_bound(first,last,value,comp,iterator_category(first));
}

template <class ForwardIterator, class T>
ForwardIterator lower_bound(ForwardIterator first, ForwardIterator last,
			    const T& value) {
  return lower_bound(first,last,value,less<T>(),iterator_category(first));
}

template <class Compare, class T>
struct __reverse_compare {
  bool operator ()(const T&x, const T&y) { return !Compare()(y,x); }
};

template <class ForwardIterator, class T, class Compare>
ForwardIterator upper_bound(ForwardIterator first, ForwardIterator last,
			    const T& value, Compare comp) {
  return lower_bound(first,last,value,__reverse_compare<Compare,T>(),
		     iterator_category(first));
}

template <class ForwardIterator, class T>
ForwardIterator upper_bound(ForwardIterator first, ForwardIterator last,
			    const T& value) {
  return lower_bound(first,last,value,__reverse_compare<less<T>,T>(),
		     iterator_category(first));
}

template <class ForwardIterator, class T, class Compare>
pair<ForwardIterator,ForwardIterator>
equal_range(ForwardIterator first, ForwardIterator last, const T& value,
	    Compare comp) {
  ForwardIterator lower = lower_bound(first,last,value,comp,
				      iterator_category(first));
  return
    pair<ForwardIterator,
  ForwardIterator>(lower,lower_bound(lower,last,value,
				     __reverse_compare<Compare,T>(),
				     iterator_category(first)));
}

template <class ForwardIterator, class T>
pair<ForwardIterator,ForwardIterator>
equal_range(ForwardIterator first, ForwardIterator last, const T& value) {
  ForwardIterator lower = lower_bound(first,last,value,less<T>(),
				      iterator_category(first));
  return
    pair<ForwardIterator,
         ForwardIterator>(lower,
			  lower_bound(lower,last,value,
				      __reverse_compare<less<T>,T>(),
				      iterator_category(first)));
}

template <class ForwardIterator, class T, class Compare>
bool binary_search(ForwardIterator first, ForwardIterator last, const T& value,
		   Compare comp) {
  ForwardIterator iter = lower_bound(first,last,value,comp,
				     iterator_category(first));
  return (iter != last) && !comp(value,*iter);
}

template <class ForwardIterator, class T>
bool binary_search(ForwardIterator first, ForwardIterator last,const T& value) {
  ForwardIterator iter = lower_bound(first,last,value,less<T>(),
				     iterator_category(first));
  return (iter != last) && !(value < *iter);
}

// 11.3.4 Merge (mcroberts, incomplete, mostly done)

// Remember: A and B are equal iff !(A < B) && !(B < A)

//--------------------------------------------------------------------
// merge
//--------------------------------------------------------------------

// Note: Contrary to the description in Stepanov & Lee, this algorithm,
// although fundamentally conforming to Knuth's Algorithm 5.2.4M, 
// requires (last1 - first1) + (last2 + first2) comparisons.  This is
// due to two consraints: 1) the definition of "equal" requires two
// comparisons, and 2) since the algorithm is required to be stable,
// we cannot output elements from both A and B when we know they are
// equal, since all equal elements from A must be output before the
// equal elements from B.  I don't think anyone will lose any sleep
// over 1 extra comparison in the worst case (all equal elements).
// Note that in Knuth's algorithm, <= is the operator of choice, which
// we must simulate by checking for <, then >.

template <class InputIterator1, class InputIterator2, class OutputIterator>
OutputIterator
merge (InputIterator1 first1, InputIterator1 last1,
       InputIterator2 first2, InputIterator2 last2,
       OutputIterator result) {
  while (first1 != last1 && first2 != last2) {
    if (*first1 < *first2) {
      *result++ = *first1++;
    } else if (*first2 < *first1) {
      *result++ = *first2++;
    } else {
      *result++ = *first1++;
    }
  }
  while (first1 != last1) {
    *result++ = *first1++;
  }
  while (first2 != last2) {
    *result++ = *first2++;
  }
  return result;
}

template <class InputIterator1, class InputIterator2,
	  class OutputIterator, class Compare>
OutputIterator
merge (InputIterator1 first1, InputIterator1 last1,
       InputIterator2 first2, InputIterator2 last2,
       OutputIterator result, Compare comp) {
  while (first1 != last1 && first2 != last2) {
    if (*first1 < *first2) {
      *result++ = *first1++;
    } else if (*first2 < *first1) {
      *result++ = *first2++;
    } else {
      *result++ = *first1++;
    }
  }
  while (first1 != last1) {
    *result++ = *first1++;
  }
  while (first2 != last2) {
    *result++ = *first2++;
  }
  return result;
}

//--------------------------------------------------------------------
// inplace_merge
//--------------------------------------------------------------------

template <class BidirectionalIterator>
void

inplace_merge(BidirectionalIterator first, BidirectionalIterator middle,
	      BidirectionalIterator last) {
  // in-place merging requires temporaries--need access to value type
  // so we use this template trick
  __inplace_merge(first, middle, last,
  		  value_type(first), distance_type(first));
}

template <class BidirectionalIterator, class T, class Distance>
void
__inplace_merge(BidirectionalIterator first, BidirectionalIterator middle,
		BidirectionalIterator last, T*, Distance*);

template <class BidirectionalIterator, class Compare>
void
inplace_merge(BidirectionalIterator first, BidirectionalIterator middle,
	      BidirectionalIterator last, Compare comp) {
  // in-place merging requires temporaries--need access to value type
  // so we use this template trick
  __inplace_merge(first, middle, last,
  		  value_type(first), distance_type(first));
}

template <class BidirectionalIterator, class T, class Distance,
	  class Compare>
void
__inplace_merge(BidirectionalIterator first, BidirectionalIterator middle,
		BidirectionalIterator last, T*, Distance*, Compare comp);

// 11.3.5 Set operations on sorted structures (mcroberts, complete)

// Remember: A and B are equal iff !(A < B) && !(B < A)

//--------------------------------------------------------------------
// includes
//--------------------------------------------------------------------

// At most ((last1 - first1) + (last2 - first2)) * 2 - 1 comparisons

template <class InputIterator1, class InputIterator2>
bool
includes(InputIterator1 first1, InputIterator1 last1,
         InputIterator2 first2, InputIterator2 last2) {
  while (first2 != last2)  {
    while (first1 != last1 && *first1 < *first2) first1++;
    if (first1 == last1 || *first2 < *first1) {
      // Didn't find *first2 before end or greater element
      return false;
    }
    first2++;
  }
  return true;    // Found them all  
}

template <class InputIterator1, class InputIterator2, class Compare>
bool
includes(InputIterator1 first1, InputIterator1 last1,
         InputIterator2 first2, InputIterator2 last2, Compare comp) {
  while (first2 != last2)  {
    while (first1 != last1 && comp(*first1, *first2)) first1++;
    if (first1 == last1 || comp(*first2, *first1)) {
      // Didn't find *first2 before end or greater element
      return false;
    }
    first2++;
  }
  return true;    // Found them all  
}

//--------------------------------------------------------------------
// set_union
//--------------------------------------------------------------------

// At most ((last1 - first1) + (last2 - first2)) * 2 - 1 comparisons

// set_union is stable--if an element is present in both lists, the
// one from the first range is copied to the result list

template <class InputIterator1, class InputIterator2, class OutputIterator>
OutputIterator
set_union(InputIterator1 first1, InputIterator1 last1,
          InputIterator2 first2, InputIterator2 last2,
          OutputIterator result) {
  while (first1 != last1 && first2 != last2) {
    if (*first1 < *first2) {
      *result++ = *first1++;
    } else if (*first2 < *first1) {
      *result++ = *first2++;
    } else {
      // they are equal
      *result++ = *first1++;
      first2++;
    }
  }
  while (first1 != last1) {
    *result++ = *first1++;
  }
  while (first2 != last2) {
    *result++ = *first2++;
  }
  return result;
}

template <class InputIterator1, class InputIterator2,
          class OutputIterator, class Compare>
OutputIterator
set_union(InputIterator1 first1, InputIterator1 last1,
          InputIterator2 first2, InputIterator2 last2,
          OutputIterator result, Compare comp) {
  while (first1 != last1 && first2 != last2) {
    if (comp(*first1, *first2)) {
      *result++ = *first1++;
    } else if (comp(*first2, *first1)) {
      *result++ = *first2++;
    } else {
      // they are equal
      *result++ = *first1++;
      first2++;
    }
  }
  while (first1 != last1) {
    *result++ = *first1++;
  }
  while (first2 != last2) {
    *result++ = *first2++;
  }

  return result;
}

//--------------------------------------------------------------------
// set_intersection
//--------------------------------------------------------------------

template <class InputIterator1, class InputIterator2, class OutputIterator>
OutputIterator
set_intersection(InputIterator1 first1, InputIterator1 last1,
                 InputIterator2 first2, InputIterator2 last2,
                 OutputIterator result) {
  while (first1 != last1 && first2 != last2) {
    if (*first1 < *first2) {
      first1++;
    } else if (*first2 < *first1) {
      first2++;
    } else {
      // they are equal
      *result++ = *first1++;
      first2++;
    }
  }
  return result;
}

template <class InputIterator1,
          class InputIterator2,
          class OutputIterator,
          class Compare>
OutputIterator
set_intersection(InputIterator1 first1, InputIterator1 last1,
                 InputIterator2 first2, InputIterator2 last2,
                 OutputIterator result, Compare comp) {
  while (first1 != last1 && first2 != last2) {
    if (comp(*first1, *first2)) {
      first1++;
    } else if (comp(*first2, *first1)) {
      first2++;
    } else {
      // they are equal
      *result++ = *first1++;
      first2++;
    }
  }
  return result;
}

//--------------------------------------------------------------------
// set_difference
//--------------------------------------------------------------------
// All members of A that are not also members of B

template <class InputIterator1, class InputIterator2, class OutputIterator>
OutputIterator
set_difference(InputIterator1 first1, InputIterator1 last1,
               InputIterator2 first2, InputIterator2 last2,
               OutputIterator result) {
  while (first1 != last1 && first2 != last2) {
    if (*first1 < *first2) {
      *result++ = *first1++;
    } else if (*first2 < *first1) {
      first2++;
    } else {
      first1++;
      first2++;
    }
  }
  while (first1 != last1) {
    *result++ = *first1++;
  }
  return result;
}

template <class InputIterator1, class InputIterator2,
          class OutputIterator, class Compare>
OutputIterator
set_difference(InputIterator1 first1, InputIterator1 last1,
               InputIterator2 first2, InputIterator2 last2,
               OutputIterator result, Compare comp) {
  while (first1 != last1 && first2 != last2) {
    if (comp(*first1, *first2)) {
      *result++ = *first1++;
    } else if (comp(*first2, *first1)) {
      first2++;
    } else {
      first1++;
      first2++;
    }
  }
  while (first1 != last1) {
    *result++ = *first1++;
  }
  return result;
}

//--------------------------------------------------------------------
// set_symmetric_difference (union(a,b) - intersection(a,b))
//--------------------------------------------------------------------

template <class InputIterator1, class InputIterator2, class OutputIterator>
OutputIterator
set_symmetric_difference(InputIterator1 first1, InputIterator1 last1,
                         InputIterator2 first2, InputIterator2 last2,
                         OutputIterator result) {
  while (first1 != last1 && first2 != last2) {
    if (*first1 < *first2) {
      *result++ = *first1++;
    } else if (*first2 < *first1) {
      *result++ = *first2++;
    } else {
      first1++;
      first2++;
    }
  }
  while (first1 != last1) {
    *result++ = *first1++;
  }
  while (first2 != last2) {
    *result++ = *first2++;
  }
  return result;
}

template <class InputIterator1, class InputIterator2,
          class OutputIterator, class Compare>
OutputIterator
set_symmetric_difference(InputIterator1 first1, InputIterator1 last1,
                         InputIterator2 first2, InputIterator2 last2,
                         OutputIterator result, Compare comp) {
  while (first1 != last1 && first2 != last2) {
    if (comp(*first1, *first2)) {
      *result++ = *first1++;
    } else if (comp(*first2, *first1)) {
      *result++ = *first2++;
    } else {
      first1++;
      first2++;
    }
  }
  while (first1 != last1) {
    *result++ = *first1++;
  }
  while (first2 != last2) {
    *result++ = *first2++;
  }
  return result;
}

// 11.3.6 Heap operations (mcroberts, complete)

//--------------------------------------------------------------------
// push_heap
//--------------------------------------------------------------------

template <class RandomAccessIterator>
void
push_heap(RandomAccessIterator first, RandomAccessIterator last) {
  __push_heap(first, last, value_type(first), distance_type(first));
}

template <class RandomAccessIterator, class T, class Distance>
void 
__push_heap(RandomAccessIterator first, RandomAccessIterator last,
	   T*, Distance*) {
  T K = *(last - 1);
  Distance j = (last - first);
  while (true) {
    Distance i = j / 2;
    if (i == 0 || K < *(first + i - 1)) {
      *(first + j - 1) = K;
      return;
    }
    *(first + j - 1) = *(first + i - 1);
    j = i;
  }
}

template <class RandomAccessIterator, class Compare>
void
push_heap(RandomAccessIterator first, RandomAccessIterator last,
	  Compare comp) {
  __push_heap(first, last, value_type(first), distance_type(first), comp);
}

template <class RandomAccessIterator, class T, class Distance, class Compare>
void 
__push_heap(RandomAccessIterator first, RandomAccessIterator last,
	   T*, Distance*, Compare comp) {
  T K = *(last - 1);
  Distance j = (last - first);
  while (true) {
    Distance i = j / 2;
    if (i == 0 || comp(K, *(first + i - 1))) {
      *(first + j - 1) = K;
      return;
    }
    *(first + j - 1) = *(first + i - 1);
    j = i;
  }
}


//--------------------------------------------------------------------
// pop_heap
//--------------------------------------------------------------------

template <class RandomAccessIterator>
void
pop_heap(RandomAccessIterator first, RandomAccessIterator last) {
  __pop_heap(first, last, value_type(first), distance_type(first));
}

template <class RandomAccessIterator, class T, class Distance>
void 
__pop_heap(RandomAccessIterator first, RandomAccessIterator last,
	   T*, Distance*) {
  // Leftmost item is largest since we used Knuth's algo.
  // Swap with rightmost to leave highest in *(last -1), then
  // make what's left into a heap.
  T tmp = *first;
  *first = *(last - 1);
  *(last - 1) = tmp;

  // Just need to integrate the leftmost item back into heap;

  Distance left  = 1;
  Distance right = (last - first) - 2;

  while (left--) { 		// H2 [Decrease l]
    T RK = *(first + left);	// R & K always same
    // H3 [Prepare for "sift-up"]
    Distance j = left;
    Distance i = j;
    while (true) {
      // H4 [Advance downward]
      i = j;
      j = ((j + 1) * 2) - 1;
      if (right < j) {
      	break;	// goto H8
      } else {
	if (j < right) {
	  // H5 [Find "larger" son]
	  if (*(first + j) < *(first + j + 1)) j++;
	}
	// H6 [Larger than K?]
	if (*(first + j) < RK) {
	  break;	// goto H8
	}
	// H7 [Move it up]
	*(first + i) = *(first + j);
      }
    } // while true
    // H8 [Store R]
    *(first + i) = RK;
  } // while left--
}

template <class RandomAccessIterator, class Compare>
void
pop_heap(RandomAccessIterator first, RandomAccessIterator last,
	  Compare comp) {
  __pop_heap(first, last, value_type(first), distance_type(first),
	      Compare comp);
}

template <class RandomAccessIterator, class T, class Distance, class Compare>
void
__pop_heap(RandomAccessIterator first, RandomAccessIterator last,
	   T*, Distance*, Compare comp) {
  // Leftmost item is largest since we used Knuth's algo.
  // Swap with rightmost to leave highest in *(last -1), then
  // make what's left into a heap.
  T tmp = *first;
  *first = *(last - 1);
  *(last - 1) = tmp;

  // Just need to integrate the leftmost item back into heap;

  Distance left  = 1;
  Distance right = (last - first) - 2;

  while (left--) { 		// H2 [Decrease l]
    T RK = *(first + left);	// R & K always same
    // H3 [Prepare for "sift-up"]
    Distance j = left;
    Distance i = j;
    while (true) {
      // H4 [Advance downward]
      i = j;
      j = ((j + 1) * 2) - 1;
      if (comp(right, j)) {
      	break;	// goto H8
      } else {
	if (comp(j, right)) {
	  // H5 [Find "larger" son]
	  if (*(first + j) < *(first + j + 1)) j++;
	}
	// H6 [Larger than K?]
	if (comp(*(first + j), RK)) {
	  break;	// goto H8
	}
	// H7 [Move it up]
	*(first + i) = *(first + j);
      }
    } // while true
    // H8 [Store R]
    *(first + i) = RK;
  } // while left--
}

//--------------------------------------------------------------------
// make_heap
//--------------------------------------------------------------------

// Note: This algorithm is adapted from Knuth 5.2.3H, heapsort, with
// the sort part left off (stop when l == 1).
// Note, however, that Knuth uses a 1-based array model, whereas we
// in C++ use a 0-based model, so there are a few adjustments.
// In particular, l is initialized to N/2 rather than N/2 + 1 and
// ranges down to 0 rather than 1.

template <class RandomAccessIterator>
void
make_heap(RandomAccessIterator first, RandomAccessIterator last) {
  __make_heap(first, last, value_type(first), distance_type(first));
}

template <class RandomAccessIterator, class T, class Distance>
void
__make_heap(RandomAccessIterator first, RandomAccessIterator last,
	    T*, Distance*) {
  // H1 [Initialize]
  Distance right = (last - first) - 1;	// r in Knuth
  Distance left  = (right + 1) / 2;	// l in Knuth

  while (left--) { 		// H2 [Decrease l]
    T RK = *(first + left);	// R & K always same
    // H3 [Prepare for "sift-up"]
    Distance j = left;
    Distance i = j;
    while (true) {
      // H4 [Advance downward]
      i = j;
      j = ((j + 1) * 2) - 1;
      if (right < j) {
      	break;	// goto H8
      } else {
	if (j < right) {
	  // H5 [Find "larger" son]
	  if (*(first + j) < *(first + j + 1)) j++;
	}
	// H6 [Larger than K?]
	if (*(first + j) < RK) {
	  break;	// goto H8
	}
	// H7 [Move it up]
	*(first + i) = *(first + j);
      }
    } // while true
    // H8 [Store R]
    *(first + i) = RK;
  } // while left--
}

template <class RandomAccessIterator, class Compare>
void
make_heap(RandomAccessIterator first, RandomAccessIterator last,
	  Compare comp) {
  __make_heap(first, last, value_type(first), distance_type(first),
	      Compare comp);
}

template <class RandomAccessIterator, class T, class Distance, class Compare>
void
__make_heap(RandomAccessIterator first, RandomAccessIterator last,
	    T*, Distance*, Compare comp) {
  // H1 [Initialize]
  Distance right = (last - first) - 1;	// r in Knuth
  Distance left  = (right + 1) / 2;	// l in Knuth

  while (left--) { 		// H2 [Decrease l]
    T RK = *(first + left);	// R & K always same
    // H3 [Prepare for "sift-up"]
    Distance j = left;
    Distance i = j;
    while (true) {
      // H4 [Advance downward]
      i = j;
      j = ((j + 1) * 2) - 1;
      if (comp(right, j)) {
      	break;	// goto H8
      } else {
	if (comp(j, right)) {
	  // H5 [Find "larger" son]
	  if (comp(*(first + j), *(first + j + 1))) j++;
	}
	// H6 [Larger than K?]
	if (comp(*(first + j), RK)) {
	  break;	// goto H8
	}
	// H7 [Move it up]
	*(first + i) = *(first + j);
      }
    } // while true
    // H8 [Store R]
    *(first + i) = RK;
  } // while left--
}

//--------------------------------------------------------------------
// sort_heap
//--------------------------------------------------------------------

// Note: This algorithm is adapted from Knuth 5.2.3H, heapsort, with
// the heap part left off (start when l == 1).
// Note, however, that Knuth uses a 1-based array model, whereas we
// in C++ use a 0-based model, so there are a few adjustments.

template <class RandomAccessIterator>
void
sort_heap(RandomAccessIterator first, RandomAccessIterator last) {
  __sort_heap(first, last, value_type(first), distance_type(first));
}

template <class RandomAccessIterator, class T, class Distance>
void
__sort_heap(RandomAccessIterator first, RandomAccessIterator last,
	    T*, Distance*) {
  // H1 [Initialize]
  Distance right = (last - first) - 1;	// r in Knuth

  while (true) {
    T RK = *(first + right);		// R & K always same
    *(first + right) = *(first);
    // H2 [Decrease r]
    if (!--right) {
    	*first = RK;
    	return;		// Done!
    }
    // H3 [Prepare for "sift-up"]
    Distance j = 0;
    Distance i = j;
    while (true) {
      // H4 [Advance downward]
      i = j;
      j = ((j + 1) * 2) - 1;
      if (right < j) {
      	break;	// goto H8
      } else {
	if (j < right) {
	  // H5 [Find "larger" son]
	  if (*(first + j) < *(first + j + 1)) j++;
	}
	// H6 [Larger than K?]
	if (*(first + j) < RK) {
	  break;	// goto H8
	}
	// H7 [Move it up]
	*(first + i) = *(first + j);
      }
    } // while true
    // H8 [Store R]
    *(first + i) = RK;
  } // while true
}

template <class RandomAccessIterator, class Compare>
void
sort_heap(RandomAccessIterator first, RandomAccessIterator last,
	  Compare comp) {
  __sort_heap(first, last, value_type(first), distance_type(first),
	      Compare comp);
}

template <class RandomAccessIterator, class T, class Distance, class Compare>
void
__sort_heap(RandomAccessIterator first, RandomAccessIterator last,
	    T*, Distance*, Compare comp) {
  // H1 [Initialize]
  Distance right = (last - first) - 1;	// r in Knuth

  while (true) {
    T RK = *(first + right);		// R & K always same
    *(first + right) = *(first);
    // H2 [Decrease r]
    if (!--right) {
    	*first = RK;
    	return;		// Done!
    }
    // H3 [Prepare for "sift-up"]
    Distance j = 0;
    Distance i = j;
    while (true) {
      // H4 [Advance downward]
      i = j;
      j = ((j + 1) * 2) - 1;
      if (comp(right, j)) {
      	break;	// goto H8
      } else {
	if (comp(j, right)) {
	  // H5 [Find "larger" son]
	  if (comp(*(first + j), *(first + j + 1))) j++;
	}
	// H6 [Larger than K?]
	if (comp(*(first + j), RK)) {
	  break;	// goto H8
	}
	// H7 [Move it up]
	*(first + i) = *(first + j);
      }
    } // while true
    // H8 [Store R]
    *(first + i) = RK;
  } // while true
}

// 11.3.7 Minimum and maximum (mrs, complete)

template <class T>
const T& min(const T& a, const T& b) {
  if (b < a)
    return b;
  return a;
}

template <class T, class Compare>
const T& min(const T& a, const T& b, Compare comp) {
  if (comp(b, a))
    return b;
  return a;
}

template <class T>
const T& max(const T& a, const T& b) {
  if (a < b)
    return b;
  return a;
}

template <class T, class Compare>
const T& max(const T& a, const T& b, Compare comp) {
  if (comp(a, b))
    return b;
  return a;
}

template <class InputIterator>
InputIterator max_element(InputIterator first, InputIterator last) {
  InputIterator max = first;
  while (first != last) {
    if (*max < *first)
      max = first;
    ++first;
  }
  return max;
}

template <class InputIterator, class Compare>
InputIterator max_element(InputIterator first, InputIterator last, Compare comp) {
  InputIterator max = first;
  while (first != last) {
    if (comp(*max, *first))
      max = first;
    ++first;
  }
  return max;
}

template <class InputIterator>
InputIterator min_element(InputIterator first, InputIterator last) {
  InputIterator min = first;
  while (first != last) {
    if (*first < *min)
      min = first;
    ++first;
  }
  return min;
}

template <class InputIterator, class Compare>
InputIterator min_element(InputIterator first, InputIterator last, Compare comp) {
  InputIterator min = first;
  while (first != last) {
    if (comp(*first, *min))
      min = first;
    ++first;
  }
  return min;
}

// 11.3.8 Lexicographical comparison (jimg, complete)

// XXX Why does STL think it can be done in min((last1-first1),
// (last2-first2)) applications of the comparison?  Does Alex assume that ==
// is used?  Should it be?

// lexicographical_compare() returns true if the sequence of elememts defined
// by the range [first1, last1) is lexicographically less than the sequence
// of elemens defined by the range [first2, last2), otherwise it returns
// false. At most 2 * min((last1-first1), (last2-first2)) applications of the
// corresponding comparison are performed.
//
// Reference: Knuth, Vol.1, p.20

template <class InputIterator1, class InputIterator2>
bool
lexicographical_compare(InputIterator1 first1, InputIterator1 last1,
			InputIterator2 first2, InputIterator2 last2) {
  while (first1 != last1 && first2 != last2) {
    if (*first1 < *first2)
      return true;
    if (*first2 < *first1)
      return false;
    ++first1; ++first2;
  }

  if (first2 != last2 && first1 == last1) // first is range shorter, hence <
    return true;

  return false;
}

// Second version of lexicographical_compare() with fifth parameter a
// comparison object.

template <class InputIterator1, class InputIterator2, class Compare>
bool
lexicographical_compare(InputIterator1 first1, InputIterator1 last1,
			InputIterator2 first2, InputIterator2 last2,
			Compare comp) {
  while (first1 != last1 && first2 != last2) {
    if (comp(*first1, *first2))
      return true;
    if (comp(*first2, *first1))
      return false;
    ++first1; ++first2;
  }

  if (first2 != last2 && first1 == last1)
    return true;

  return false;
}

// 11.3.9 Permutation generators (jimg, incomplete)

// 11.4 Generalized numeric operations (sean, complete)

// 11.4.1 Accumulate (sean, complete)

template <class InputIterator, class T>
T accumulate(InputIterator first, InputIterator last, T init) {
  T accum(init);

  while (first != last) {
    accum = accum + *first;
    ++first;
  }

  return accum;
}

template <class InputIterator, class T, class BinaryOperation>
T accumulate(InputIterator first, InputIterator last, T init,
	     BinaryOperation binary_op) {
  T accum(init);

  while (first != last) {
    accum = binary_op(accum, *first);
    ++first;
  }

  return accum;
}

// 11.4.2 Inner product (sean, complete)

template <class InputIterator1, class InputIterator2, class T>
T
inner_product(InputIterator1 first1, InputIterator1 last1,
	      InputIterator2 first2, T init) {
  T prod(init);

  while (first1 != last1) {
    prod = prod + (*first1) * (*first2);
    ++first1;
    ++first2;
  }

  return prod;
}

template <class InputIterator1, class InputIterator2, class T,
	  class BinaryOperation1, class BinaryOperation2>
T
inner_product(InputIterator1 first1, InputIterator1 last1,
	      InputIterator2 first2, T init,
	      BinaryOperation1 binary_op1, BinaryOperation2 binary_op2) {
  T prod(init);

  while (first1 != last1) {
    prod = binary_op1(prod, binary_op2(*first1, *first2));
    ++first1;
    ++first2;
  }

  return prod;
}

// 11.4.3 Partial sum (sean, complete)

// auxilliary partial sum functions
template <class InputIterator, class OutputIterator, class T>
OutputIterator
__aux_partial_sum(InputIterator first, InputIterator last,
		  OutputIterator result, T accum) {
  // assert: accum == *first && first != last
  *result = accum;
  ++result;
  ++first;
  while (first != last) {
    accum = accum + *first;
    *result = accum;
    ++result;
    ++first;
  }

  return result;
}

template<class InputIterator, class OutputIterator, class BinaryOperation, class T>
OutputIterator
__aux_partial_sum(InputIterator first, InputIterator last,
		  OutputIterator result, BinaryOperation binary_op,
		  T accum) {
  // assert: accum == *first && first != last
  *result = accum;
  ++result;
  ++first;
  while (first != last) {
    accum = binary_op(accum, *first);
    *result = accum;
    ++result;
    ++first;
  }

  return result;
}

template <class InputIterator, class OutputIterator>
OutputIterator
partial_sum(InputIterator first, InputIterator last,
	    OutputIterator result) {
  if (first != last) {
    return __aux_partial_sum(first, last, result, *first);
  }

  return result;
}

template <class InputIterator, class OutputIterator, class BinaryOperation>
OutputIterator
partial_sum(InputIterator first, InputIterator last,
	    OutputIterator result, BinaryOperation binary_op) {
  if (first != last) {
    return __aux_partial_sum(first, last, result, binary_op, *first);
  }

  return result;
}

// 11.4.4 Adjacent difference (sean, complete)

// auxiliary adjacent difference functions
template<class InputIterator, class OutputIterator, class T>
OutputIterator
__aux_adjacent_difference(InputIterator first, InputIterator last,
			  OutputIterator result, T prev) {
  // assert: prev == *first && first != last
  *result = prev;
  ++result;
  ++first;
  while (first != last) {
    T current(*first);
    *result = current - prev;
    ++result;
    ++first;
    prev = current;
  }

  return result;
}

template<class InputIterator, class OutputIterator, class BinaryOperation, class T>
OutputIterator
__aux_adjacent_difference(InputIterator first, InputIterator last,
			  OutputIterator result, BinaryOperation binary_op, T prev) {
  // assert: prev == *first && first != last
  *result = prev;
  ++result;
  ++first;
  while (first != last) {
    T current(*first);
    *result = binary_op(current, prev);
    ++result;
    ++first;
    prev = current;
  }

  return result;
}

template <class InputIterator, class OutputIterator>
OutputIterator
adjacent_difference(InputIterator first, InputIterator last,
		    OutputIterator result) {
  if (first != last) {
    return __aux_adjacent_difference(first, last, result, *first);
  }

  return result;
}

template <class InputIterator, class OutputIterator, class BinaryOperation>
OutputIterator
adjacent_difference(InputIterator first, InputIterator last,
		    OutputIterator result, BinaryOperation binary_op) {
  if (first != last)
    {
      return __aux_adjacent_difference(first, last, result, binary_op,
				       *first);
    }

  return result;
}

// 12 Adaptors (mrs, complete)

// 12.1 Container adaptors (mrs, complete)

// 12.1.1 Stack (mrs, complete)

template <class Container>
class stack {
  friend bool operator ==(const stack<Container>& x,
			  const stack<Container>& y);
public:
  typedef Container::value_type value_type;
  typedef Container::size_type size_type;
protected:
  Container c;
public:
  bool empty() const {
    return c.empty();
  }
  size_type size() const {
    return c.size();
  }
  value_type& top() {
    return c.back();
  }
  const value_type& top() const {
    return c.back();
  }
  void push(const value_type& x) {
    c.push_back(x);
  }
  void pop() {
    c.pop_back();
  }
};

template <class Container>
bool
operator==(const stack<Container>& x, const stack<Container>& y) {
  return x.c == y.c;
}

// 12.1.2 Queue (mrs, complete)

template <class Container>
class queue {
  friend bool operator ==(const queue<Container>& x,
			  const queue<Container>& y);
public:
  typedef Container::value_type value_type;
  typedef Container::size_type size_type;
protected:
  Container c;
public:
  bool empty() const {
    return c.empty();
  }
  size_type size() const {
    return c.size();
  }
  value_type& front() {
    return c.front();
  }
  const value_type& front() const {
    return c.front();
  }
  value_type& back() {
    return c.back();
  }
  const value_type& back() const {
    return c.back();
  }
  void push(const value_type& x) {
    c.push_back(x);
  }
  void pop() {
    c.pop_front();
  }
};

template <class Container>
bool
operator ==(const queue<Container>& x, const queue<Container>& y) {
  return x.c == y.c;
}

// 12.1.3 Priority queue (mrs, complete)

template <class Container, class Compare
#ifdef TEMPLATE_DEFAULT_ARG
= less<Container::value_type>
#endif
>
class priority_queue {
public:
  typedef Container::value_type value_type;
  typedef Container::size_type size_type;
protected:
  Container c;
  Compare comp;
public:
  priority_queue(const Compare& x = Compare()) : c(), comp(x) {
  }
#ifdef TEMPLATE_MEMBER
  template <class InputIterator>
  priority_queue(InputIterator first, InputIterator last,
		 const Compare& x = Compare()) : c(first, last), comp(x) {
    make_heap(c.begin(), c.end(), comp);
  }
#endif
  bool empty() const {
    return c.empty();
  }
  size_type size() const {
    return c.size();
  }
  const value_type& top() const {
    return c.front();
  }
  void push(const value_type& x) {
    c.push_back(x);
    push_heap(c.begin(), c.end(), comp);
  }
  void pop() {
    pop_heap(c.begin(), c.end(), comp);
    c.pop_back();
  }
};

// 12.2 Iterator adaptors (mrs, complete)

// 12.2.1 Reverse iterators (mrs, complete)

template <class BidirectionalIterator, class T, class Distance
#ifdef TEMPLATE_DEFAULT_ARG
= ptrdiff_t
#endif
>
class reverse_bidirectional_iterator : public bidirectional_iterator<T, Distance> {
#ifdef TEMPLATE_FRIENDS
  friend template <class BidirectionalIterator, class T, class Distance>
  bool operator ==(const reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>& x,
		   const reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>& y);
#else
public:
  bool operator ==(const reverse_bidirectional_iterator& x) {
    return current == x.current;
  }
#endif
protected:
  BidirectionalIterator current;
public:
  reverse_bidirectional_iterator() {
  }
  reverse_bidirectional_iterator(BidirectionalIterator x) : current(x) {
  }
  operator BidirectionalIterator() {
    return current;
  }
  T operator * () {
    BidirectionalIterator tmp = current;
    return *--tmp;
  }
  reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>&
  operator ++() {
    --current;
    return *this;
  }
  reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>
  operator ++(int) {
    reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>
      tmp = *this;
    --current;
    return tmp;
  }
  reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>&
  operator -- () {
    ++current;
    return *this;
  }
  reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>
  operator -- (int) {
    reverse_bidirectional_iterator<BidirectionalIterator, T, Distance> tmp
      = *this;
    ++current;
    return tmp;
  }
};

#ifndef TEMPLATE_FRIENDS
template <class BidirectionalIterator, class T, class Distance>
inline bool
operator ==(const reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>& x,
	    const reverse_bidirectional_iterator<BidirectionalIterator, T, Distance>& y) {
  return x.current == y.current;
}
#endif

template <class RandomAccessIterator, class T, class Distance
#ifdef TEMPLATE_DEFAULT_ARG
= ptrdiff_t
#endif
>
class reverse_iterator : public random_access_iterator<T, Distance> {
#ifdef TEMPLATE_FRIENDS
  friend template <class RandomAccessIterator, class T, class Distance>
  bool operator ==(const reverse_iterator<RandomAccessIterator,
		   T,
		   Distance>& x,
		   const reverse_iterator<RandomAccessIterator,
		   T,
		   Distance>& y);
  friend template <class RandomAccessIterator, class T, class Distance>
  bool operator < (const reverse_iterator<RandomAccessIterator,
		   T,
		   Distance>& x,
		   const reverse_iterator<RandomAccessIterator,
		   T,
		   Distance>& y);
  friend template <class RandomAccessIterator, class T, class Distance>
  Distance operator - (const reverse_iterator<RandomAccessIterator,
			 T,
			 Distance>& x,
                         const reverse_iterator<RandomAccessIterator,
			 T,
			 Distance>& y);
  friend template <class RandomAccessIterator, class T, class Distance>
  reverse_iterator<RandomAccessIterator, T, Distance>
  operator +(Distance n,
	      const reverse_iterator<RandomAccessIterator,
	      T,
	      Distance>& x);
#endif
protected:
  RandomAccessIterator current;
public:
  reverse_iterator() {
  }
  reverse_iterator(RandomAccessIterator x) : current(x) {
  }
  operator RandomAccessIterator() {
    return current;
  }
  T operator * () {
    RandomAccessIterator tmp = current;
    return *--tmp;
  }
  reverse_iterator<RandomAccessIterator, T, Distance>&
  operator ++() {
    --current;
    return *this;
  }
  reverse_iterator<RandomAccessIterator, T, Distance>
  operator ++(int) {
    reverse_iterator<RandomAccessIterator, T, Distance> tmp = *this;
    --current;
    return tmp;
  }
  reverse_iterator<RandomAccessIterator, T, Distance>&
  operator -- () {
    ++current;
    return *this;
  }
  reverse_iterator<RandomAccessIterator, T, Distance>
  operator -- (int) {
    reverse_iterator<RandomAccessIterator, T, Distance> tmp = *this;
    ++current;
    return tmp;
  }
  reverse_iterator<RandomAccessIterator, T, Distance>
  operator +(Distance n) const {
    return reverse_iterator<RandomAccessIterator, T, Distance>(current - n);
  }
  reverse_iterator<RandomAccessIterator, T, Distance>&
  operator +=(Distance n) const {
    current -= n;
    return *this;
  }
  reverse_iterator<RandomAccessIterator, T, Distance>
  operator - (Distance n) const {
    return reverse_iterator<RandomAccessIterator, T, Distance>(current + n);
  }
  reverse_iterator<RandomAccessIterator, T, Distance>
  operator -= (Distance n) const {
    current += n;
    return *this;
  }
  T operator[](Distance n) {
    return *(*this + n);
  }
};

template <class RandomAccessIterator, class T, class Distance>
inline bool
operator ==(const reverse_iterator<RandomAccessIterator,
	    T,
	    Distance>& x,
	    const reverse_iterator<RandomAccessIterator,
	    T,
	    Distance>& y) {
  return x.current == y.current;
}

template <class RandomAccessIterator, class T, class Distance>
inline bool
operator < (const reverse_iterator<RandomAccessIterator,
	    T,
	    Distance>& x,
	    const reverse_iterator<RandomAccessIterator,
	    T,
	    Distance>& y) {
  return y.current < x.current;
}

template <class RandomAccessIterator, class T, class Distance>
inline Distance
operator - (const reverse_iterator<RandomAccessIterator,
	    T,
	    Distance>& x,
	    const reverse_iterator<RandomAccessIterator,
	    T,
	    Distance>& y) {
  return y.current - x.current;
}

template <class RandomAccessIterator, class T, class Distance>
inline reverse_iterator<RandomAccessIterator, T, Distance>
operator +(Distance n,
	    const reverse_iterator<RandomAccessIterator,
	    T,
	    Distance>& x) {
  return reverse_iterator<RandomAccessIterator, T, Distance>(current - n);
}

// 12.2.2 Insert iterators (mrs, complete)

template <class Container>
class back_insert_iterator : public output_iterator {
protected:
  Container& container;
public:
  back_insert_iterator(Container& x) : container(x) {}
  back_insert_iterator<Container>&
  operator =(const Container::value_type& value) {
    container.push_back(value);
    return *this;
  }
  back_insert_iterator<Container>& operator * () {
    return *this;
  }
  back_insert_iterator<Container>& operator ++() {
    return *this;
  }
  back_insert_iterator<Container> operator ++(int) {
    return *this;
  }
};

template <class Container>
back_insert_iterator<Container>
back_inserter(Container& x) {
  return back_insert_iterator<Container>(x);
}

template <class Container>
class front_insert_iterator : public output_iterator {
protected:
  Container& container;
public:
  front_insert_iterator(Container& x) : container(x) {
  }
  front_insert_iterator<Container>&
  operator =(const Container::value_type& value) {
    container.push_front(value);
    return *this;
  }
  front_insert_iterator<Container>& operator*() {
    return *this;
  }
  front_insert_iterator<Container>& operator ++() {
    return *this;
  }
  front_insert_iterator<Container> operator ++(int) {
    return *this;
  }
};

template <class Container>
front_insert_iterator<Container>
front_inserter(Container& x) {
  return front_insert_iterator<Container>(x);
}

template <class Container>
class insert_iterator : public output_iterator {
protected:
  Container& container;
  Container::iterator iter;
public:
  insert_iterator(Container& x, Container::iterator i)
    : container(x), iter(i) {}

  insert_iterator<Container>& operator =(const Container::value_type& value) {
    iter = container.insert(iter, value);
    ++iter;
    return *this;
  }
  insert_iterator<Container>& operator*() { return *this; }
  insert_iterator<Container>& operator ++() { return *this; }
  insert_iterator<Container> operator ++(int) { return *this; }
};

template <class Container, class Iterator>
insert_iterator<Container>
inserter(Container& x, Iterator i) {
  return insert_iterator<Container>(x, Container::iterator(i));
}

// 12.2.3 Raw storage iterator (mrs, complete)

template <class OutputIterator, class T>
class raw_storage_iterator : public output_iterator, restrictor<OutputIterator> {
public:
  raw_storage_iterator(OutputIterator x) : restrictor<OutputIterator>(x) {
  }
  raw_storage_iterator<OutputIterator, T>& operator*() {
    return *this;
  }
  raw_storage_iterator<OutputIterator, T>& operator=(const T& element) {
    construct(&*value, element);
    return *this;
  }
  raw_storage_iterator<OutputIterator, T>& operator ++() {
    ++value;
    return *this;
  }
  raw_storage_iterator<OutputIterator, T> operator ++(int) {
    raw_storage_iterator<OutputIterator, T> tmp = *this;
    ++value;
    return tmp;
  }
};

// 12.3 Function adaptors (mrs, complete)

// 12.3.1 Negators (mrs, complete)

template <class Predicate>
class unary_negate : public unary_function<Predicate::argument_type, bool>,
    restrictor<Predicate> {
public:
  unary_negate(const Predicate& x) : restrictor<Predicate>(x) {
  }
  bool operator()(const argument_type& x) const {
    return !value(x);
  }
};

template <class Predicate>
unary_negate<Predicate>
not1(const Predicate& pred) {
  return unary_negate<Predicate>(pred);
}

template <class Predicate>
class binary_negate : public binary_function<Predicate::first_argument_type, Predicate::second_argument_type, bool>, restrictor<Predicate> {
public:
  binary_negate(const Predicate& x) : restrictor<Predicate>(x) {
  }
  bool operator () (const first_argument_type& x,
		    const second_argument_type& y) const {
    return !value(x, y);
  }
};

template <class Predicate>
binary_negate<Predicate>
not2(const Predicate& pred) {
  return binary_negate<Predicate>(pred);
}

// 12.3.2 Binders (mrs, complete)

template <class Operation>
class binder1st : public unary_function<Operation::second_argument_type, Operation::result_type> {
protected:
  Operation op;
  argument_type value;
public:
  binder1st(const Operation& x, const Operation::first_argument_type& y)
    : op(x), value(y) {
    }
  result_type operator () (const argument_type& x) const {
    return op(value, x);
  }
};

template <class Operation, class T>
binder1st<Operation> bind1st(const Operation& op, const T& x) {
  return binder1st<Operation>(op, Operation::first_argument_type(x));
}

template <class Operation>
class binder2nd : public unary_function<Operation::first_argument_type, Operation::result_type> {
protected:
  Operation op;
  argument_type value;
public:
  binder2nd(const Operation& x, const Operation::second_argument_type& y)
    : op(x), value(y) {}
  result_type operator()(const argument_type& x) const {
    return op(x, value);
  }
};

template <class Operation, class T>
binder2nd<Operation> bind2nd(const Operation& op, const T& x) {
  return binder2nd<Operation>(op, Operation::second_argument_type(x));
}

// 12.3.3 Adaptors for pointers to functions (mrs, complete)

template <class Arg, class Result>
class pointer_to_unary_function : public unary_function<Arg, Result>,
    restrictor<Result (*)(Arg)> {
public:
  pointer_to_unary_function(Result (*x)(Arg))
    : restrictor<Result (*)(Arg)>(x) {}
  Result operator () (const Arg& x) const {
    return value(x);
  }
};

template <class Arg, class Result>
pointer_to_unary_function<Arg, Result>
ptr_fun(Result (*x)(Arg)) {
  return pointer_to_unary_function<Arg, Result>(x);
}

template <class Arg1, class Arg2, class Result>
class pointer_to_binary_function : public binary_function<Arg1, Arg2, Result>,
    restrictor<Result (*)(Arg1, Arg2)> {
public:
  pointer_to_binary_function(Result (*x)(Arg1, Arg2))
    : restrictor<Result (*)(Arg1, Arg2)>(x) {}
  Result operator () (const Arg1& x, const Arg2& y) const {
    return value(x, y);
  }
};

template <class Arg1, class Arg2, class Result>
pointer_to_binary_function<Arg1, Arg2, Result>
ptr_fun(Result (*x)(Arg1, Arg2)) {
  return pointer_to_binary_function<Arg1, Arg2, Result>(x);
}

// 13 Memory handling (mrs, complete)

// 13.1 Primitives (mrs, complete, extra)

template <class T>
inline T*
allocate(ptrdiff_t n, T*) {
  return (T*)operator new (n*sizeof(T));
}

template <class T>
inline void
deallocate(T* buffer) {
  operator delete (buffer);
}

template <class T1, class T2>
inline void
construct(T1* p, const T2& value) {
  new (p) T1(value);
}

template <class T>
inline void
destroy(T* pointer) {
  pointer->~T();
}

// XXX to be really good, this should be specialized for underlying allocator
// XXX should catch allocation failures, and return n == 0 for them.
template <class T>
pair<T*, ptrdiff_t>
get_temporary_buffer(ptrdiff_t n, T*) {
  T* pointer = (T*)operator new (n*sizeof(T));
  return pair<T*, ptrdiff_t>(pointer, n);
}

template <class T>
void
return_temporary_buffer(T* pointer) {
  operator delete (pointer);
}

// 13.2 Specialized algorithms (mrs, complete)

template <class ForwardIterator>
void
destroy(ForwardIterator first, ForwardIterator last) {
  while (first != last)
    destroy(&*first++);
}

template <class InputIterator, class ForwardIterator>
ForwardIterator
uninitialized_copy(InputIterator first, InputIterator last,
		   ForwardIterator result) {
  while (first != last)
    construct(&*result++, *first++);
  return result;
}

template <class ForwardIterator, class T>
void
uninitialized_fill(ForwardIterator first, ForwardIterator last, const T& x) {
  while (first != last)
    construct(&*first++, x);
}

template <class ForwardIterator, class Size, class T>
void
uninitialized_fill_n(ForwardIterator first, Size n, const T& x) {
  while (n--)
    construct(&*first++, x);
}

#endif	// _stl_h
