/*
	This file contains a program which attempts to
	simulate communication in a ring.For the sake of
	simplicity, the nodes in the ring are identified
	using a single character.
*/

/* Socket related include files */
#include <stdio.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <signal.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#if 0
#define MSG1 "Message 1A"
#define MSG2 "Message 2A"
#define MSG3 "Message 3A"
#define MSG4 "Message 4A"
#endif

#define ME		argv[1]
#define LEFT		argv[2]
#define RIGHT		argv[3]
#define NUM_NODES	atoi(argv[4])
#define MAX_SENDS	500	/* Configureable? */


/* Local Functions */
int send_now();
char* new_dest();

/* Global Variables */
char buf[11];
int num_nodes;
char *left_neighbor, *right_neighbor;
char *me ;
int message_count = 0;
struct timeval time_out;
void usage();
long mysock_l, mysock_r;


void
sig_ALM(int x)
{
	close(mysock_l) ;

	close(mysock_r) ; 

	delserver( me ) ;

	exit(0) ;
}

void
sig_INT(int x)
{
	close(mysock_l) ;

	close(mysock_r) ; 

	delserver( me ) ;

}

main(int argc, char* argv[])
{
	/* argv[1] = name of process
	   argv[2] = name of left neighbour
	   argv[3] = name of right neighbour
	   argv[4] = # of nodes in the ring. (MAX 26)
	   argv[5] = maximum time of execution
        */
	fd_set read_set;
	int max = 0 ; 

	struct linger lingerOnClose = {1,1} ;

	if (argc != 6)
	{
		usage();
		exit(1);
	}

	srand(time(0)) ;

	me = ME ; 
	left_neighbor = LEFT;
	right_neighbor = RIGHT;
	num_nodes = NUM_NODES;

	signal( SIGALRM, sig_ALM ) ;
	signal( SIGINT, sig_INT ) ;

	alarm( atoi(argv[5]) ) ;  
	
	if (argv[1][0] == 'A')
	{	
		/* A meets it's right neighbour, then it's left neighbour.
		   Everyone else meets left neighbour before right neighbour.
		*/
	   
		fprintf(stderr, "Me = %s, Right = %s\n", ME, RIGHT);

		if ((mysock_r = meet(ME, RIGHT)) < 0)
		{
			fprintf(stderr,"Error on meet w/ right neighbour\n");
			exit(0);
		}

		if ((mysock_l = meet(ME, LEFT)) < 0)
		{
			fprintf(stderr,"Error on meet w/ left neighbour\n");
			exit(0);
		}

	}
	else
	{
		if ((mysock_l = meet(ME, LEFT)) < 0)
		{
			fprintf(stderr,"Error on meet w/ left neighbour\n");
			exit(0);
		}

		if ((mysock_r = meet(ME, RIGHT)) < 0)
		{
			fprintf(stderr,"Error on meet w/ right neighbour\n");
			exit(0);
		}
	}


	setsockopt( mysock_l, SOL_SOCKET, SO_LINGER,
			 (char *)&lingerOnClose, sizeof(struct linger) ) ;

	setsockopt( mysock_r, SOL_SOCKET, SO_LINGER,
			 (char *)&lingerOnClose, sizeof(struct linger) ) ;

	max = mysock_r > mysock_l ? mysock_r : mysock_l ;

	/*
	 *  All the meets are complete. Now, we commence the
	 *  random send/receive operations.
	 *  If a send times out, then we terminate the process.
	 *  Ultimately, all sends will timeout, and every
	 *  process will terminate.
	 */

	/* A initiates the message transfer */ 
	/*
	if ( ME[0] == 'A' )
	{
		strcpy( buf,new_dest() ) ;

		if (send(mysock_r, &buf[0], strlen(buf)+1) < 0)
		{
			fprintf(stderr,"Error on send\n");
			exit(1);
		}
		fprintf(stderr,"Send a message to %s\n", RIGHT);
	}
	*/

	while (1)
	{
		int s;

		fflush(stderr) ; 
		fflush(stdout) ; 
		FD_ZERO(&read_set);
		FD_SET(mysock_l, &read_set);
		FD_SET(mysock_r, &read_set);

		time_out.tv_sec = rand()%30 + 1 ;
		time_out.tv_usec = 0 ;

		printf("Timeout value %d\n",time_out.tv_sec ) ;

		/* TIME_OUT is randomly generated */
		s = select(max+1 , &read_set, NULL, NULL, &time_out); 

		printf("Select done %d\n",s) ; 

		if (s == 0)
		{
			fprintf(stderr,"Select timed out\n") ; 

			/* Select has timed out */
			/* Generate a new message with some probability */
			if (send_now())
			{
				strcpy( buf, new_dest() );
				
				if (send(mysock_r, &buf[0], strlen(buf)+1) < 0)
				{
					fprintf(stderr,"Error on send\n");
					exit(1);
				}
				fprintf(stderr,"Send message to %s\n",buf) ;
				buf[0] = '\0' ; 
			}
		}
		else if ( s < 0 ) 
		{
			exit(1) ;
		}
		else
		{
			/* A message has been received.
			 * It is either for me or it has
			 * to be routed further.
			 */
		
			if ( FD_ISSET(mysock_l,&read_set) && read(mysock_l, &buf[0], sizeof(buf)) < 0)
			{
				fprintf(stderr,"Error on read\n");
				exit(1);
			}
			
			if ( FD_ISSET(mysock_r,&read_set) && read(mysock_r, &buf[0], sizeof(buf)) < 0)
			{
				fprintf(stderr,"Error on read\n");
				exit(1);
			}
			
			if (strcmp(buf, ME) == 0)
			{
				fprintf(stderr,"Message for me %s\n",buf) ;
				buf[0] = '\0';
			}
			else if (buf[0] == '\0')
			{
				fprintf(stderr, "Terminating....\n");
				exit(0);
			}
			else
			{
				if (send(mysock_r, &buf[0], strlen(buf)+1) < 0)
				{
					fprintf(stderr,"Error on send\n");
					exit(1);
				}
				fprintf(stderr,"Forward message to %s\n",buf) ;

				buf[0] = '\0' ; 
			}
		}
	}
}


int send_now()
{
	/*
	return ( rand()%2 ) ;  
	*/
	return 1 ; 
}


char*
new_dest()
{
	/* Will be modified to use probabalistic methods */
	return left_neighbor;
}

	

void usage()
{
	fprintf(stderr,"ring <your id> <left neighbor> <right neighbor> <num of nodes> <time of running>\n");
}
