/*  Copyright 1987 Bart Miller.  All rights reserved.
 *  Explicit permission required to redistributed this software.
 */

#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <stdio.h>

#include "swbmsg.h"
#include "table.h"

#define ERROR		-2
#define BADCOUNT	-1
#define ENDOFFILE	0
#define GOOD		1
#define TRUE		1
#define FALSE		0

fd_set readmask;

struct table *descriptors [NOFILE];

/*
 *		*------------------------------*
 *		|                              |
 *		|      WaitForRequest          |
 *		|                              |
 *		*------------------------------*
 */

WaitForRequest (servsock)
    long servsock;
{
	fd_set selectmask;
	int count, rv;
	int stat;
	struct servrequest request;
	struct serventry *FindServer();

/*	acceptmask = 1 << servsock;
 *	selectmask = readmask | acceptmask;
 */

	bcopy ((char *)&readmask, (char *)&selectmask, sizeof (fd_set));
	FD_SET (servsock, &selectmask);

	/*
	 *	wait for a connection request or message
	 */
	count = select (NOFILE, &selectmask, 0, 0, 0);
	if (count < 0)
	{
		perror ("on select");
		fprintf (stderr, "selectmask=");
		PrintMask (stderr, selectmask);
		fprintf (stderr, "\n");
		exit (-1);
	}

	/*
	 *	First, check for new connection requests
	 */
	if (FD_ISSET (servsock, &selectmask))
	{
		rv = accept (servsock, 0, 0);
		if (rv < 0)
		{
			perror ("on accept");
			fprintf (stderr, "Error on Accept\n");
			exit (-1);
		} else
			SetMaskOn (rv);
	} else {
		/*
	 	*	message has arrived
	 	*/
		rv = FirstOne (&selectmask);
		if (rv == 0)
		{
			fprintf (stderr, "help: empty selectmask=");
			PrintMask (stderr, selectmask);
			fprintf (stderr, "\n");
			exit (-1);
		}

#ifdef DEBUG
		fprintf (stderr, "message on socket %d \n", rv-1);
#endif
		/* go read the request */
		stat = GetRequest (rv-1, &request);
	
		if (stat == GOOD)
			ProcessRequest (rv-1, &request);
		else if (stat == ENDOFFILE) {
			if (FindServer (rv-1) != (struct serventry *)0)
			/*---- server died------------------------- */
			{
				DeleteServer (FindServer (rv-1));
			}
	
			if (descriptors[rv-1] != (struct table *) 0)
			{
				CleanUp (descriptors[rv-1]->myname, descriptors[rv-1]->hername);
				DeleteEntry (descriptors[rv-1]);
			}
			SetMaskOff (rv-1);
			close (rv-1);
		} else if ((stat == BADCOUNT) || (stat == ERROR)) {
			if (FindServer (rv-1) != (struct serventry *) 0) /* server died */
			{
				DeleteServer (FindServer (rv-1));
			} else {
				Respond (rv-1, SERV_ERROR);
			}
			SetMaskOff (rv-1);
			close (rv-1);
		}

	}	
} /* WaitForRequest */


/*
 *		--------------------------------
 *		|                              |
 *		|      GetRequest              |
 *		|                              |
 *		--------------------------------
 */

GetRequest (sock, request)
    long sock;
    struct servrequest *request;
{
	int rv;

	rv = read (sock, request, sizeof (struct servrequest));
	
	if (rv == 0)
		return (ENDOFFILE);
	else if (rv < 0) {
		perror ("error on reading request");
		return (ERROR);
	} else if (rv != sizeof (struct servrequest)) {
		return (BADCOUNT);
	}
	return (GOOD);
}


/*
 *		--------------------------------
 *		|                              |
 *		|      ProcessRequest          |
 *		|                              |
 *		--------------------------------
 */

ProcessRequest (sock, request)
    long sock;
    struct servrequest *request;
{
	switch (request->function)
	{
	case  SERV_PEER_MEET:	/* client called meet() */
		PeerMeetRequest (request, sock);
		break;
	case SERV_SERV_MEET:	/* server propagated meet request */
		ServMeetRequest (request, sock);
		break;
	case  SERV_PUTSERVER:	/* put server entry in table */
		PutServer (request, sock);
		break;
	case  SERV_ACTIVE:	/* server sent active message in handshake */
		ActiveServer (request, sock);
		break;
	case  SERV_PASSIVE:	/* server sent passive message in handshake */
		PassiveServer (request, sock);
		break;
	case  SERV_CLEANUP:	/* sent from server, remove entries from table */
		DeleteFromTable (request, sock);
		break;
	case  SERV_NEWSERV:	/* new server has arrived, add it to list of servers */
		NewServer(sock);
		break;
	case  SERV_DELETE:	/* delete entry from table */
		DeleteRequest (request, sock);
		break;
	case  SERV_INIT:	/* clear table */
		ClearRequest (request, sock);
		break;
	case SERV_SEND_INFO:	/* new server wants server list and table info */
		SendServerInfo (sock); 
		AddServer (sock);
		break;
	default:		/* bogus message */
		Respond (sock, SERV_BADFUNC);
		close (sock);
	}
}


/*
 *		--------------------------------
 *		|                              |
 *		|      NewServer               |
 *		|                              |
 *		--------------------------------
 */

NewServer(sock)
    long sock;
{
	/*
	 *	add new server's socket to list of servers
	 */
	AddServer (sock);
}


/*
 *		--------------------------------
 *		|                              |
 *		|      ClearRequest            |
 *		|                              |
 *		--------------------------------
 */

ClearRequest (request, sock)
    struct servrequest *request;
    long sock;
{
	if (SentFromPeer (sock) == TRUE)
	{
		PropagateUpdate (request, sock);
		Respond (sock, SERV_OK);
		close (sock);
		SetMaskOff (sock);
	}
	/*
	 *	remove all entries from table, closing any open sockets
	 */
	ClearTable ();
}


/*
 *		--------------------------------
 *		|                              |
 *		|      PeerMeetRequest         |
 *		|                              |
 *		--------------------------------
 */

PeerMeetRequest (request, sock)
    struct servrequest *request;
    long sock;
{
	struct table *tabent;
	struct table *FindEntry();
	struct table *AddEntry();
	struct in_addr hostaddr;

	/*
	 *	look in the table for another process with the same name
	 *	wanting to meet the same process (i.e. meet("red", "green") and
	 *	then another meet("red", "green")
	 */
	/*********************************************************************
	*tabent = FindEntry (request->hername, request->myname);
	*if (tabent != (struct table *) 0)
	*{
	*	if (tabent->hostid == 0)
	*	{
	*		Respond (sock, SERV_ERROR);
	*		SetMaskOff (sock);
	*		close (sock);
	*		return;
	*	}
	*}
	*********************************************************************/
#ifdef DEBUG
	fprintf (stderr, "in PeerMeetRequest: myname=%s, hername=%s\n",
	request->myname, request->hername);
#endif

	/*
	 *	add entry to table in all cases
	 */
	SockToHostAddr (sock, &hostaddr);
	request->myaddr = hostaddr;
	tabent = AddEntry (SERV_PEER_MEET, request, sock, 0);
	SetDescriptor (sock, tabent);

#ifdef DEBUG
	fprintf (stderr, "in PeerMeetRequest: after AddEntry\n");
#endif

	/*
	 * 	look up matching request in table
	 */
	tabent = FindEntry (request->myname, request->hername);
#ifdef DEBUG
	fprintf (stderr, "in PeerMeetRequest: after FindEntryEntry\n");
#endif

	if (tabent == (struct table *)0)
		/*-----not found; send meet request to all other servers--- */
	{
#ifdef DEBUG
	fprintf (stderr, "in PeerMeetRequest: trying to propagate (shouldn't)\n");
#endif
		request->function = SERV_SERV_MEET;
		PropagateUpdate (request, sock);
		request->function = SERV_PEER_MEET;
	} else {
#ifdef DEBUG
	fprintf (stderr, "in PeerMeetRequest: found entry\n");
	fprintf (stderr, "   myname=%s, hername=%s, hostsock=%d\n",
	   tabent->myname, tabent->hername, tabent->hostsock);
#endif
		/*-----found other side in table-----------------*/
		/*
	 	 *	check hostsock field of found entry; if = 0, my entry,
	 	 *	otherwise, another server's entry
	 	 */
		if (tabent->hostsock == 0)	/* my entry */
		{
			/*
			 *	both peers connected to me, respond to both and cleanup
			 */
			if (tabent->entrytype == SERV_PEER_MEET)   /* added 4/15/86 to facil. putserver */
	  			GoodRespond (SERV_PAS, tabent->mysock, hostaddr,
					request->myport, request->myname);
			GoodRespond (SERV_ACT, sock, tabent->myhost, tabent->myport,
				  	tabent->myname);

			
			CleanUp (tabent->myname, tabent->hername);

			DeleteFromTable(request, sock);
		} else {	/* other peer connected to some other server */
				/* must send passive message to other server */
				/***not if meeting server***/
				if (ServSend (SERV_PASSIVE, tabent->hostsock, request->myname,
					request->hername, request->myport, hostaddr) < 0)
				{
					fprintf(stderr, "error on send to server");
				}
		} /* else */
	} /* else */
} /* PeerMeetRequest */



/*
 *		---------------------------------
 *		|				|
 *		|	ServMeetRequest		|
 *		|				|
 *		---------------------------------
 */		

ServMeetRequest (request, sock)
    struct servrequest *request;
    long sock;
{
	struct table *tabent;
	struct table *FindEntry();
	struct table *AddEntry();

	/*
 	 *	look for matching entry in table
 	 */
	tabent = FindEntry (request->myname, request->hername);
	if (tabent == (struct table *) 0)
	{
		/*
		 *	other side not in table, add to table
		 */
		tabent = AddEntry (SERV_PEER_MEET, request, 0, sock);
	} else {	/* found in table */
		/*
		 *	check hostsock field of found entry--if 0, my entry
		 */
		if (tabent->hostsock == 0)	/* my entry */
		{
			/* arbitrate */
			if (gethostid() < request->hostid)	/* win */
			if (ServSend (SERV_PASSIVE, sock, tabent->myname, tabent->hername,
					tabent->myport, tabent->myhost) < 0)
			{
				fprintf (stderr, "Error on sending to server");
			}
		} else {	/* lose */
			/*
			 *	add other entry to table, for backup purposes
			 */
			tabent = AddEntry (SERV_PEER_MEET, request, 0, sock);
		}
	} /* else */
} /* ServMeetRequest */



/*
 *		--------------------------------
 *		|	                       |
 *		|	  ActiveServer         |
 *		|                              |
 *		--------------------------------
 */

ActiveServer(request, sock)
    struct servrequest *request;
    long sock;
{
	struct table *tabent;
	struct table *FindEntry();

/*
 *	find entry in table
 */
	tabent = FindEntry (request->myname, request->hername);
	if (tabent == (struct table *) 0)
	{
		/* bad news */
		fprintf (stderr, "Active Server Could Not Find Peer in Table");
	} else {
		/*
		 *	respond active to peer found in table
		 */
		GoodRespond (SERV_ACT, tabent->mysock, request->myaddr,
				request->myport, request->myname);
		CleanUp (request->myname, request->hername);
		DeleteFromTable(request, sock);
	}
}


/*
 *		--------------------------------
 *		|	                       |
 *		|	  PassiveServer        |
 *		|                              |
 *		--------------------------------
 */

PassiveServer (request, sock)
    struct servrequest *request;
    long sock;
{
	struct table *tabent;
	struct table *FindEntry();

	/*
 	 *	find entry in table
 	 */
	tabent = FindEntry (request->myname, request->hername);
	if (tabent == (struct table *) 0)
	{
		fprintf (stderr, "Passive Server Could Not Find Peer");
	} else {
		/*
		 *	send to the other server as active server
		 */
		if (ServSend (SERV_ACTIVE, sock, tabent->myname, tabent->hername,
				tabent->myport, tabent->myhost) < 0)
		{
			fprintf (stderr, "Error on sending to server");
		} else {
			GoodRespond (SERV_PAS, tabent->mysock, request->myaddr, 
					request->myport, request->myname);
		}
	}
} /* PassiveServer */




/*
 *		--------------------------------
 *		|	                       |
 *		|	  PutServer            |
 *		|                              |
 *		--------------------------------
 */

PutServer (request, sock)
    struct servrequest *request;
    long sock;
{
	struct table *tabent;
	struct table *FindEntry();
	struct table *AddEntry();
	struct in_addr hostaddr;

#ifdef DEBUG
	fprintf (stderr, "in PutSever: myname=%s, hername=%s\n",
	request->myname, request->hername);
#endif

	/*----Need to go through table for pending server requests------------*/
	/*tabent = FindEntry (request->myname, "");
	if (tabent == (struct table *)0)*/

	request->hername[0] = '\000';
	SockToHostAddr(sock, &hostaddr);
	request->myaddr = hostaddr;
	tabent = AddEntry (SERV_PUTSERVER, request, sock, 0);

	Respond (sock, SERV_OK);
	close (sock);
	SetMaskOff (sock);
#ifdef DEBUG
	fprintf (stderr, "done PutSever: myname=%s, hername=%s\n",
	request->myname, request->hername);
#endif
} /* PutServer */


/*
 *		--------------------------------
 *		|			       |
 *		|	  DeleteRequest        |
 *		|                              |
 *		--------------------------------
 */

DeleteRequest (request, sock)
    struct servrequest *request;
    long sock;
{
	struct table *tabent;
	struct table *FindEntry();

	if (SentFromPeer (sock) == TRUE)
		PropagateUpdate (request, sock);
	/*---- (note that the params in this find are in reverse order from
	 *	  the find in AddEntry)
	 */
	tabent = FindEntry (request->hername, request->myname);
	if (tabent == (struct table *)0)
	{
		/*-----Not found in table----------------------------*/
		if (SentFromPeer (sock) == TRUE)
			Respond (sock, SERV_NOTFOUND);
	} else {
		if (SentFromPeer (sock) == TRUE)
		{
			close (tabent->mysock);
			SetMaskOff (tabent->mysock);
			Respond (sock, SERV_OK);
		}
		DeleteEntry (tabent);
	}
}

/*
 *		--------------------------------
 *		|	                       |
 *		|	  DeleteFromTable      |
 *	        |                              |
 *		--------------------------------
 */

DeleteFromTable (request, sock)
    struct servrequest *request;
    long sock;
{
	struct table *tabent;
	struct table *FindEntry();

	/*---- (note that the params in this find are in reverse order from
	 *	  the find in AddEntry)
	 */
	tabent = FindEntry (request->hername, request->myname);
	if ((tabent != (struct table *)0) && (tabent->entrytype != SERV_PUTSERVER))
	{
		while (tabent != (struct table *) 0)
		{
			if (tabent->entrytype == SERV_PEER_MEET)
			{
				if (tabent->hostid == 0)
				{
					close (tabent->mysock);
					SetMaskOff (tabent->mysock);
				}
				DeleteEntry (tabent);
			}
			tabent = FindEntry (request->hername, request->myname);
		}
	}

	/* other entry */
	tabent = FindEntry (request->myname, request->hername);
	if ((tabent != (struct table *)0) && (tabent->entrytype != SERV_PUTSERVER))
	{
		while (tabent != (struct table *) 0)
		{
			if (tabent->entrytype == SERV_PEER_MEET)
			{
				if (tabent->hostid == 0)
				{
					close (tabent->mysock);
					SetMaskOff (tabent->mysock);
				}
				DeleteEntry (tabent);
			}
			tabent = FindEntry (request->myname, request->hername);
		}
	}
} /* DeleteFromTable */


/*
 *		--------------------------------
 *		|	                       |
 *		|	  SendServerInfo       |
 *		|                              |
 *		--------------------------------
 */

int
SendServerInfo(sock)
    long sock;
{
	struct inforesponse info;
	struct serventry *se;
	struct serventry *GetFirstServer();
	struct in_addr hostaddr;
	int i;

	se = GetFirstServer(); /* get pointer to head of list */

	/* fill up 'info' with addresses of active servers */
	for (i=0; se != (struct serventry *)0; se = se->next)
		{
		SockToHostAddr(se->sock, &hostaddr);
		info.servaddr[i++] = hostaddr.s_addr;
		}
	info.numaddrs = i;

	/* send it off */
	if (send(sock, &info, sizeof(info), 0) < 0)
		fprintf(stderr, "client died.\n");
}



/*
 *		--------------------------------
 *		|	                       |
 *		|	  Respond              |
 *	        |                              |
 *		--------------------------------
 */

Respond (sock, status)
    long sock;
    long status;
{
	struct servresponse resp;

#ifdef DEBUG
	fprintf (stderr, "in respond: status = %d\n", status);
#endif
	resp.status = status;
	resp.herhost.s_addr = (u_long) 0;
	resp.herport = 0;
	if (send (sock, &resp, sizeof (struct servresponse), 0) < 0)
		fprintf (stderr, "warn: on Respond");
} /* Respond */


/*
 *		--------------------------------
 *		|	                       |
 *		|	  GoodRespond          |
 *		|                              |
 *		--------------------------------
 */

GoodRespond (status, sock, herhost, herport, hername)
    long status;
    long sock;
	struct in_addr herhost;
	u_short herport;
	servname hername;
{
	struct servresponse resp;

#ifdef DEBUG
	fprintf (stderr, "in GoodRespond: status = %d\n", status);
#endif
	resp.status = status;
	resp.herhost.s_addr = herhost.s_addr;
	resp.herport = herport;
	strcpy (resp.hername, hername);
	if (send (sock, &resp, sizeof (struct servresponse), 0) < 0)
		fprintf (stderr, "warn: on GoodRespond");
}

/*
 *		--------------------------------
 *		|	                       |
 *		|	  SetMaskOff/On        |
 *		|                              |
 *		--------------------------------
 */

SetMaskOff (bit)
    long bit;
{
    FD_CLR (bit, &readmask);
    descriptors[bit] = (struct table *) 0;
}

SetMaskOn (bit)
    long bit;
{
    FD_SET (bit, &readmask);
}

int
TestBit (bit)
    long bit;
{
    return (FD_ISSET (bit, &readmask));
}


int
FirstOne (mask)
    fd_mask mask[];
{
    int i, size;
    int offset;

    size = howmany (FD_SETSIZE, NFDBITS);

    for (i = 0; i < size; i++)
    {
	if (offset = (ffs (mask[i])))
	{
	    return ((i * sizeof (fd_mask) * NBBY) + offset);
	}
    }
    return (0);
}


SetDescriptor (fd, tabent)
    struct table *tabent;
{
	if (descriptors[fd] != (struct table *) 0)
	{
		fprintf (stderr, "Help! Setting descriptor %d twice\n", fd);
	}
	descriptors[fd] = tabent;
} /* SetDescriptor */



SentFromPeer (sock)
    long sock;
{
	struct serventry *FindServer();

	/*
	 *	return true if message sent from peer, false otherwise
	 */
	if (FindServer (sock) == (struct serventry *) 0)
		return TRUE;
	else
		return FALSE;
}



PrintMask (file, mask)
    FILE *file;
    fd_mask mask[];
{
    int i;

    for (i = 0; i < howmany(FD_SETSIZE, NFDBITS); i++)
    {
	fprintf (file, "%x ", mask[i]);
    }
}
