.\" Copyright (c) 1986 Regents of the University of Wisconsin.
.\" All rights reserved.
.\"
.\"	@(#)swbd.8c	1.0 (Wisconsin) 11/15/85
.\"
.TH SWBD UW "June 15, 1986"
.UC 5
.SH NAME
swbd \- Switchboard connection server
.SH SYNOPSIS
.B /etc/swbd
.SH DESCRIPTION
.I Swbd
the internet connection server process.
A process connects to the
.I swbd
server by requesting the "swbd" service, with the "tcp" protocol
using a
.I "getservbyname()"
call.
Processes can make several types of requests to
.I swbd
server.
The request types are defined in
.IR "<swbmsg.h>" .
Each request to the swb is made using a
.I swbrequest
record.
Process typically do not call the
.I swbd
process directly, but rather use the library routines in ``-lswb''.
.PP
.nf
.RS
.DT
typedef char swbname[SWBNAMELEN];

struct swbrequest {
	long		function;		/* function for this request */
	swbname	myname;		/* name of this process */
	u_short	myport;		/* port number for this process */
	swbname	hername;		/* name of other process */
};
.RE
.fi
.PP
The
.I swbd
server finds the host address of the requesting process by using a
.I "getpeername()"
call on the request connection.
Responses from the
.I swbd
have the following format:
.PP
.nf
.RS
.DT
struct swbresponse {
	long		status;		/* return status of function */
	swbname	hername;		/* name of peer process */
	struct	in_addr herhost;	/* peer internet host addr */
	u_short	herport;		/* port number of that process */
};
.RE
.fi
.PP
The following requests can be made to the
.IR swbd :
.TP "\w'SWB_PUTSERVER: 'u"
.BI SWB_MEET:
This request sends a name for the current process and a name for the process
to which we want to connect.
The port number of a TCP/IP socket is passed with the request.
The response from this is the internet host address of the other process,
and the port number of the socket to which we wish to connect.
A response message is not sent until both processes have sent requests
to the
.I swbd
server (or on error).
Response status codes from this request are list below.
.TP
.BI SWB_PUTSERVER:
This request sends a name for the current process, which is a server to which
many process will want to connect.
The server name and the port number are passed in this call.
The
.I hername
parameter is not used with this call.
A response message is sent immediately.
.TP
.BI SWB_DELETE:
This request deletes a
.I swbd
entry for a server, whose name is specified in the
.I myname
field of the request message.
.PP
The following response codes can be returned from the
.I swbd
server:
.TP "\w'SWB_NOTFOUND:'u"
.BI "SWB_ACT:"
This is a successful response to a SWB_MEET request.
This process is expected to take the active role in establishing a connection
to its peer process.
.TP
.BI "SWB_PAS:"
This is a successful response to a SWB_MEET request.
This process is expected to take the passive role in establishing a connection
to its peer process.
.TP
.BI "SWB_OK:"
This is a successful response to a SWB_PUTSERVER or SWB_DELETE request.
.TP
.BI "SWB_BADFUNC:"
This response indicates an invalid function requested of the
.IR swbd .
.TP
.BI "SWB_NOTFOUND:"
This response indicates that server name in a SWB_DELETE request was not
found in the
.IR swbd .
.TP
.BI "SWB_ERROR:"
This response indicates any other error to a
.I swbd
request.
.SH FILES
.ta \w'/usr/include/swbmsg.h  'u
/usr/adm/swb.log	log file for swb errors
.br
/usr/include/swbmsg.h	include file with format for swb requests
.SH "SEE ALSO"
meet(3), putsever(3), delserver(3), setswbhost(3), getpeername(2), getservbyname(3)
.SH BUGS
SWB_PUTSERVER and SWB_DELETE are currently not implemented.
.PP
The server must run as the super-user
to create its socket with privileged port numbers.
.PP
There is no security checking.
Processes can specify any name they wish on requests.
Any process can perform a SWB_DELETE call.
.PP
This is an experimental server.
Bugs and comments to bart@asiago.wisc.edu.
