/*  Copyright 1987 Bart Miller.  All rights reserved.
 *  Explicit permission required to redistributed this software.
 */

#include <sys/param.h>
#include <netinet/in.h>
#include <stdio.h>

#include "swbmsg.h"
#include "table.h"

struct table *requesttable;

/*
 *		*------------------------------*
 *		|                              |
 *		|      InitTable               |
 * 		|                              |
 *		*------------------------------*
 *
 */

InitTable ()
{
	requesttable = (struct table *) 0;
}


/*
 *		*------------------------------*
 *		|                              |
 *		|      ClearTable              |
 *		|                              |
 *		*------------------------------*
 *
 */

ClearTable ()
{
	struct table *p;
	struct table *old_p;

	/*
	 *	remove all entries from the table, closing any open sockets
	 *	as they are found
	 */
	p = requesttable;
	while (p != (struct table *)0)
	{
		if (p->hostsock == 0)	/* my entry */
			close (p->mysock);
		old_p = p;
		free (old_p);
		p = p->next;
	}
}



/*
 *		--------------------------------
 *		|                              |
 *		|      DelServEntry            |
 *		|                              |
 *		--------------------------------
 *
 */

DelServEntry (sock)
	long sock;
{
	struct table *p;
	struct table *next_p;

	/*
	 *	delete all entries from a particular server
	 */
	p = requesttable;
	while (p != (struct table *) 0)
	{
		next_p = p->next;
		if (p->hostsock == sock)
		{

			DeleteEntry (p);
		}
		p = next_p;
	}
}


/*
 *		--------------------------------
 *		|                              |
 *		|      FindEntry               |
 *		|                              |
 *		--------------------------------
 *
 */

struct table *
FindEntry (myname, hername)
	servname myname;
	servname hername;
{
	struct table *p;
	int found;

	/*
	 *	find a particular entry in the table 
	 */
	p = requesttable;
	while (p != (struct table *)0)
	{
		if (p->entrytype == SERV_PEER_MEET)
		{
			found =
		 	(!(strcmp (hername, p->myname)) && !(strcmp (myname, p->hername)));
		}
		else if (p->entrytype == SERV_PUTSERVER)
		{
			found = !(strcmp (hername, p->myname));
		}
		if (found)
		{
			return (p);
		} else
			p = p->next;
	}
	return ((struct table *) 0);

}


/*
 *		--------------------------------
 *		|                              |
 *		|      AddEntry                |
 *		|                              |
 *		--------------------------------
 *
 */

struct table *
AddEntry (entrytype, request, sock, hostsock)
	long entrytype;
	struct servrequest *request;
	long sock;
	long hostsock;
{
	struct table *p;

	p = (struct table *) malloc (sizeof (struct table));
	strcpy (p->myname, request->myname);
	p->myhost = request->myaddr;
	p->myport = request->myport;
	p->mysock = sock;
	p->entrytype = entrytype;

	strcpy (p->hername, request->hername);

	p->hostsock = hostsock;
	p->hostid = request->hostid;

	p->next = requesttable;
	requesttable = p;
	return (p);
}


/*
 *		--------------------------------
 *		|                              |
 *		|      DeleteEntry             |
 *		|                              |
 *		--------------------------------
 *
 */

DeleteEntry (tabent)
	struct table *tabent;
{
	struct table *p;
	struct table *prev_p;

	/*
	 *	delete a particular entry from the table
	 */
	if (tabent == requesttable)
	{
		requesttable = requesttable->next;
		free (tabent);
		return;

	} else {

		p = requesttable;
		while (p != (struct table *)0)
		{
			if (tabent == p)
			{
			prev_p->next = p->next;
			free (p);
			return;
			}
			prev_p = p;
			p = p->next;
		}
	}
	fprintf (stderr, "help! can find the one to delete\n");
}
